unit fMHA_Main;

// 3/13/2013: Added feature to check YS_MHA_AUX.dll to ensure it
// exists and is current.

// 3/28/2012: Corrected AdministerAllItems so that default values in spin-edit
// would not crash when evaluating for rule. The OnExit event for the spin-edit
// will immediately move to the next item, so if the next item is to be disabled
// by skip-logic, a crash occurs. Corrected in Instrument Display too.

// 3/29/2012: Expanded the graph function so that min and max scores were more
// visible. Added one or minus one to max and min ordinates.

// 6/15/2012: An AUDC score, when two digits long, would appear as "1~" instead of
// "10". Bug fixed in YS_MHA_AUX.dll.

// 6/22/2012: Corrected ASI bug. When the user starts a Lite ASI, and stops to
// finish later, the FULL ASI is presented instead of the Lite ASI when the user
// restarts the incomplete form.

// 6/6/2013: Patch 108, in order to report totals on 2 reports in addition to the scale
// sub-totals have added code in uMHA_Report_OPS to execute logic if the instrument name
// is either 'YBOCSII'or 'SIP-2L'

// Patch 108: Removed ability to administer a GAF
//            Removed (by commenting out) appending the word "button" to a button
//                caption when a screen reader is in use, it created an inappropriate redundancy
//            Modified fIA_Dialogs to remove all Borland code, replacing it with code to display message boxes
//            Modifications so that Patient Label would be read by a screen reader, not eloquent

// 03/02/2017: Patch 108 was never released. Therefore references to 108 have been changed
//             to 129 as that is the patch most of the changes will be released in.
//             Modifications/Changes that are not being implemented (released in 129) will
//             remain commented as in patch 108.

interface

uses

  Windows,
  Messages,
  SysUtils,
  System.UITypes,
  Classes,
  Graphics,
  Controls,
  Forms,
  StdCtrls,
  Menus,
  ExtCtrls,
  RzCommon,
  RzLabel,
  ShellAPI,
  jpeg,
  RzShellDialogs;


type
  TfrmMHA_Main = class(TForm)
    MainMenu1: TMainMenu;
    mnuFile: TMenuItem;
    N1: TMenuItem;
    mnuExit: TMenuItem;
    mnuTools: TMenuItem;
    mnuHelp: TMenuItem;
    mnuOptions: TMenuItem;
    mnuSelectPatient: TMenuItem;
    pnlImage: TPanel;
    pnlHeader: TPanel;
    mnuAbout: TMenuItem;
    imgASI: TImage;
    imgGAF: TImage;
    imgResults: TImage;
    imgTests: TImage;
    lblResults: TRzLabel;
    lblGAF: TRzLabel;
    lblASI: TRzLabel;
    mnuMetric: TMenuItem;
    N2: TMenuItem;
    N3: TMenuItem;
    mnuUploadResults: TMenuItem;
    N4: TMenuItem;
    mnuOnlineSupport: TMenuItem;
    imgLogo: TImage;
    lblInstAdmin: TRzLabel;
    lblGAFDue: TLabel;
    pnlScreenReader: TPanel;
    btnInstAdmin: TButton;
    btnResults: TButton;
    btnASI: TButton;
    btnGAF: TButton;
    dlgDir: TRzSelectFolderDialog;
    lblPatientScrnRder: TStaticText;
    lblPatient: TLabel;

    procedure FormCreate(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure ConfigureForm;

    procedure mnuExitClick(Sender: TObject);
    procedure mnuOptionsClick(Sender: TObject);
    procedure mnuSelectPatientClick(Sender: TObject);
    procedure mnuASIClick(Sender: TObject);
    procedure mnuGAFClick(Sender: TObject);
    procedure mnuAboutClick(Sender: TObject);
    procedure mnuResultsClick(Sender: TObject);
    procedure mnuTestsClick(Sender: TObject);
    procedure lblASIMouseEnter(Sender: TObject);
    procedure lblGAFMouseEnter(Sender: TObject);
    procedure lblTestsMouseEnter(Sender: TObject);
    procedure lblResultsMouseEnter(Sender: TObject);
    procedure WaitingDisplay;
    procedure mnuUploadResultsClick(Sender: TObject);
    procedure mnuMetricClick(Sender: TObject);

    procedure DefaultHandler(var Message); override;
    procedure lblTestsMouseLeave(Sender: TObject);
    procedure lblResultsMouseLeave(Sender: TObject);
    procedure lblASIMouseLeave(Sender: TObject);
    procedure lblGAFMouseLeave(Sender: TObject);
    procedure ShowSingleInstrumentMenu;
    //procedure ShowSingleInstrument;
    procedure mnuOnlineSupportClick(Sender: TObject);
    procedure SetupScreenReader(Sender: TObject);
    procedure btnInstAdminClick(Sender: TObject);
    procedure btnResultsClick(Sender: TObject);
    procedure btnASIClick(Sender: TObject);
    procedure btnGAFClick(Sender: TObject);
    procedure FormKeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure FormActivate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);

  private

    { BEGIN CPRS Tools menu}
    FVistaMsg: Word;
    { END CPRS Tools menu}
    function GetDPI: Integer;
    procedure LoadPatientSelectionForm;
    procedure LoadInstrumentSelectForm;
    procedure ShowTestsSelect;
    //ver 70 patch 108, removed reference
    //procedure ShowGAF;
    procedure ShowASI;
    procedure ShowResults;
    procedure PromptPatientSelection(AMessage: string);
    procedure SetASI;
    procedure SetGAF;
    procedure SetResults;
    procedure SetTests;
    procedure ShowOrHideImage;
    function EnterDataForDeadPatient: BOOLEAN;
    procedure IsScreenReaderInUse;
    { Private declarations }
  public
    { Public declarations }
  end;

var
  frmMHA_Main                 : TfrmMHA_Main;

implementation

uses
  // XE3 migration llh;  changed logic of splash screen, moved to project file
  //fMHA_Splash, // Creates & displays splash screen. Do not add this unit to the
               // project.
  // XE3 migration llh; added new SplashScreen to uses clause
  fMHA_SplashScreen,

  uIA_Const,
  uMHA_Const,
  uIA_Utils,
  uMHA_Registry,
  uIA_Patient,
  uIA_Patient_OPS,
  uIA_ThisUser,
  uIA_GUIControl,
  fIA_dialogs,
  Dialogs,
  uMHA_LocalFiles,
  fMHA_GAF,
  uMHA_GAF_OPS,
  fMHA_Admin,
  uMHA_Proctor,
  fMHA_SingleInstrumentMenu,
  fMHA_SelectPatient,
  fMHA_Options,
  fMHA_InstrumentSelect,
  fMHA_InstrumentResults,
  fMHA_ASI_Main,
  fMHA_About,
  uMHA_DM,
  fMHA_ASI_Entry,
  fMHA_AdministerAllItems,
  fMHA_AdministerOneItem,
  fMHA_UploadResults,
  fMHA_ASI_Entry_508,
  fMHA_Options_508;

{$R *.DFM}

var
  lvMenuMarker                : Integer;
  lvDateSeparator,
    lvTimeSeparator           : Char;
  lvLongTimeFormat,
    lvShortDateFormat         : string;
  lvInProcess                 : Boolean;

  { BEGIN CPRS Tools menu}

procedure TfrmMHA_Main.DefaultHandler(var Message);
// FOR Conversations with CPRS
// adds check to the message handling for this form to get a registered message
var
  buf                         : array[0..255] of Char;
  CPRSMessageText             : string;
  OpenFormHandle              : HWND;

begin

  // do the default message handling
  inherited DefaultHandler(Message);
  // if the message is 'VistA Event - Clinical' and not posted from self...
  // wParam=Handle of message sender, lParam=entry in global atom table

  with TMessage(Message) do
    if (Msg = FVistaMsg) and (Int(wParam) <> Int(Handle)) then
      begin

        // Make sure that the original instance of CPRS is the sender of this
        // message. If not, then skip this procedure altogether
        if gvCPRS_Instance_Handle = 0 then
          gvCPRS_Instance_Handle := wParam;

        if gvCPRS_Instance_Handle <> Int(wParam) then
          begin
//ShowMessage('This session of MHA should be closed.');
            exit;
          end;

        // retrieve the text pointed to by lParam into a buffer
        GlobalGetAtomName(lParam, buf, 255);
        CPRSMessageText := StrPas(buf);

        // CPRS Selected Patient event has occurred
        if (Piece(CPRSMessageText, '^', 1) = 'XPT') and
          (Piece(CPRSMessageText, '^', 2) = 'CPRS') then
          begin
            //  if same patient selected, exit
            if (Patient.DFN = Piece(CPRSMessageText, '^', 3)) then
              Exit;

            // CPRS has changed patient:
            if (Patient.DFN <> Piece(CPRSMessageText, '^', 3)) then
              begin
                if OpenFormName = ofAll then
                  begin
                    frmMHA3_AdministerAllItems.ChangePt;
                    OpenFormName := ofSelect;
                  end;

                if OpenFormName = ofSingle then
                  begin
                    frmMHA3_AdministerSingleItems.ChangePt;
                    OpenFormName := ofSelect;
                  end;

                if OpenFormName = ofASI then
                  begin
                    frmMHA_ASI_Entry.BringToFront;
                    frmMHA_ASI_Entry.ChangePt;
                    OpenFormName := ofASIMenu;
                  end;

                if OpenFormName = ofASI_508 then
                  begin
                    frmMHA_ASI_Entry_508.BringToFront;
                    frmMHA_ASI_Entry_508.ChangePt;
                    OpenFormName := ofASIMenu;
                  end;

                // New patient but no work is in progress
                // Do this before changing patients
                OpenFormHandle := FindWindow('TfrmMHA_EditGAF', nil);
                if not (OpenFormHandle = 0) then
                  SendMessage(OpenFormHandle, WM_SYSCOMMAND, SC_CLOSE, 0);

                //Assign pt selected in CPRS to our own.
                Patient.Load(Piece(CPRSMessageText, '^', 3),
                  DataModule1.RPCBroker1);

                if (OpenFormName = ofSingleMenu) then
                  frmMHA_SingleInstrumentMenu.UpdateAllDisplays;

                if (OpenFormName = ofResult) then
                //frmMHA_InstrumentResults.UpdateAllDisplays;
                  begin
                    frmMHA_InstrumentResults.Close;
                    ShowResults;
                  end;

                if (OpenFormName = ofSelect) then
                  frmMHA_InstrumentSelect.NewPatientDisplay;
                if (OpenFormName = ofGAF) then
                  frmMHA_GAF.UpdateAllDisplays;
                if (OpenFormName = ofASIMenu) then
                  frmMHA_ASI_Main.UpdateAllDisplays;
                if (OpenFormName = ofUpload) then
                  frmMHA_UploadResults.DisplayCPRSPatientProperties;

                // The caption on the Main form always should be changed,
                // regardless of which form is active when a new patient is selected
                // patch 108 changes for 508 compliance llh
                // next line way it was, added if else block
                //lblPatient.caption := Patient.CaptionInfo;
                if not gvUseScreenReader then
                  begin
                    lblPatient.Visible := true;
                    lblPatientScrnRder.Visible := false;
                    lblPatient.caption := Patient.CaptionInfo;
                    pnlHeader.TabStop := false;
                  end
                else
                  begin
                    lblPatient.Visible := false;
                    lblPatientScrnRder.Visible := true ;
                    lblPatientScrnRder.Caption := 'Mental Health Assistant ' + Patient.CaptionInfo ;
                    pnlHeader.TabStop := true ;
                    ActiveControl := lblPatientScrnRder ;
                  end;

                //BringWindowToTop(Application.Handle);
              end; // Change of patient
          end;

        // CPRS is closing. Close MHA as well?
        if (Piece(CPRSMessageText, '^', 1) = 'END') and
          (Piece(CPRSMessageText, '^', 2) = 'CPRS') then
          begin
            gvCPRSShutDown := True;

            // Allow MHA to stay open until the form is done.
            // Then close MHA.

            if (OpenFormName = ofAll) or
              (OpenFormName = ofSingle) or
              (OpenFormName = ofASI) or
              (OpenFormName = ofASI_508) then
              exit;

            Application.Terminate;
          end;

      end; // with TMessage(Message)
end;
{ END CPRS Tools menu}


procedure TfrmMHA_Main.FormCreate(Sender: TObject);
var
  CPRSFont                    : HFONT;
  i                           : integer;
begin

  // Use the same font as CPRS, if connected, otherwise it use
  // the last font (or default font from Windows).
  gvFont := TFont.Create;
  gvDPI := GetDPI; // This is used in registry, keep it ahead of reg.read

  MHA_Registry.ReadRegistry;

  // patch 105 set AUX dll file into variable
  gvAuxFile := gvEnvVar + YS_MHA_AUX ;

  gvAPICalls := TStringList.Create;

  // Look for CPRS main window's class
  gvCPRS_Instance_Handle := FindWindow('TfrmFrame', nil); //CPRS
  if gvCPRS_Instance_Handle <> 0 then // Window found
    begin
      CPRSFont := SendMessage(gvCPRS_Instance_Handle, WM_GETFONT, 0, 0);
      gvFont := MakeFontFromHFont(CPRSFont);
      MHA_Registry.SaveRegistry;
    end;

  gvID := '';

  // Since TDate & TDateTime, used in this program, are set by the user
  // for this PC, we will set them to our standard and return them to user
  // settings at close.
  lvDateSeparator := FormatSettings.DateSeparator;
  FormatSettings.DateSeparator := '/';
  lvTimeSeparator := FormatSettings.TimeSeparator;
  FormatSettings.TimeSeparator := ':';
  lvShortDateFormat := FormatSettings.ShortDateFormat;
  FormatSettings.ShortDateFormat := 'MM/dd/yyyy';
  lvLongTimeFormat := FormatSettings.LongTimeFormat;
  FormatSettings.LongTimeFormat := 'hh:nn:ss AM/PM';

  gvCPRSShutDown := False;
  gvCheckResourceFiles := True;

  // Do this once, or when font changes in Options
  GUI_Control.ConfigureGUI(gvFont);
  // Do not place "Position" in ConfigureForm
  Position := poDesigned;

  IsScreenReaderInUse;

  DataBaseMode := dbNil;

  {- There are three ways MHA can be started:
       1. Off-line. No connection to CPRS is detected. User can enter some
          instruments but results are not available. When the user reconnects
          to the VA network, they will be prompted to upload the off-line
          administrations.
       2. CPRS Tools menu (Standard). Parameters are passed to MHA identifying
          the server, port, user, patient, etc. MHA displays its standard main menu.
       3. CPRS Tools menu (Identified Instrument). The main menu
          is not presented, rather a sub-menu displaying the identified instrument
          previous administrations for this patient and options for new administrations
          and/or edit previous ones.


     The gatekeeper is located in DataModule1 which counts the number of passed
     parameters: 0 for Off-line, 5 for standard, 6 for Identified Instrument. -}

  if DataModule1.ConnectToVista(self) then
    begin
      FVistaMsg := RegisterWindowMessage('VistA Event - Clinical'); //FOR CPRS
      Patient.Load(Piece(ParamStr(3), '=', 2), DataModule1.RPCBroker1);
      ThisUser.Load(DataModule1.RPCBroker1.User.DUZ, DataModule1.RPCBroker1);
      // patch 108 llh -- the GAF is being removed in this patch
      //GAF_OPS := TGAF_OPS.Create;
    end;

  // Off-line Start
  if not (DataBaseMode = dbVista) then
    begin
      // XE3 migration llh, - replaced code
      //SplashScreen.Release;
      if (MsgDlgFrm('MHA is not able to connect to VistA.  '
        + 'The only thing you can do off-line is administer an instrument.  '
        + 'You can upload the results the next time you are connected to VistA.'
        + CRLF + CRLF
        + '(If you can start CPRS from this machine, you should do so and then '
        + 'start MHA from the CPRS tools menu.)'
        + CRLF + CRLF
        + 'Do you want to work off-line?',
        MHA_Caption, mtConfirmation, [mbYes, mbNo], 0) = mrYes) then
        begin
          DataBaseMode := dbOffLine;
          ThisUser.DUZ := NO_ID;
          ThisUser.Name := 'Off Line';
          Caption := Caption + OFFLINE_CAPTION;
          // XE3 migration llh -- new splashscreen stuff
          frmMHA_SplashScreen.Hide ;
          frmMHA_SplashScreen.Release;
          LoadPatientSelectionForm;
          Halt;
          Exit;
        end
      else
        Halt;
    end; // Off-line

  // Identified Instrument: Sub-menu start up
  if (DataBaseMode = dbVista) and (ParamCount > 5) then
  // s=XXXXXX.DOMAIN.EXT p=9278 DFN=238 DUZ=6627 m=MREF CAGE
    begin
      gvNameOfInstrument := '';
      for i := 6 to ParamCount do
        gvNameOfInstrument := gvNameOfInstrument + ParamStr(i) + ' ';
      gvNameOfInstrument := Trim(UpperCase(gvNameOfInstrument));

      // XE3 migration llh; replaced code
      //SplashScreen.Release;
      // XE3 migration llh -- new splashscreen stuff
      frmMHA_SplashScreen.Hide ;
      frmMHA_SplashScreen.Release;

      ShowSingleInstrumentMenu;

      if gvID <> 'Return' then
        begin
          Close;
          Halt;
          exit;
        end;
    end;

  ConfigureForm; // Don't put this in FormShow as it will be run everytime the
    // user returns to the main menu.

  mnuMetric.Enabled := (DataBaseMode = dbVista);
  mnuOnlineSupport.Enabled := (DataBaseMode = dbVista);

  ShowOrHideImage;
  // XE3 migration llh; replaced code
  //SplashScreen.Release;
  // XE3 migration llh -- new splashscreen stuff
  frmMHA_SplashScreen.Hide ;
  frmMHA_SplashScreen.Release ;

  if (DataBaseMode = dbVista) then
    begin
      mnuUploadResults.Enabled := LocalFiles.OffLineAdministrationFilesExist;

      if mnuUploadResults.Enabled then
        if (MsgDlgFrm('Do you want to upload to VistA the results of instruments '
          + 'that were administered off-line?', MHA_Caption, mtInformation,
          [mbYes, mbNo], 0) = mrYes) then
          mnuUploadResultsClick(Self);
    end; //if DataBaseMode = dbVista

  //ver 70 patch 108, remove GAF from Option menu, need to adjust itemindex by 1
  if (gvStart = 4 {5}) and gvUseScreenReader then
    gvStart := 0;

  case gvStart of
    1: ShowTestsSelect;
    2: ShowResults;
    //ver 70 patch 108, GAF removed from option menu
    //3: ShowGAF;
    3 {4}: ShowASI;
    4 {5}: ShowResults; // The Special Result is picked up later
  end;

end;

procedure TfrmMHA_Main.FormShow(Sender: TObject);
begin

  Screen.Cursor := crDefault;
  OpenFormName := ofMain;
  lvInProcess := False;

end;

procedure TfrmMHA_Main.FormActivate(Sender: TObject);
begin

  Left := GUI_Control.GetLeft(Width);
  Top := GUI_Control.GetTop(Height);

  // User may have changed fonts in previous menu item
  if GUI_Control.NeedGUIUpdate then
    ConfigureForm;

  // Update "GAF needed" reminder
  // patch 108 llh -- the GAF is being removed in this patch
  {
  if DataBaseMode = dbVista then
    begin
      GAF_OPS.GetPreviousGAFRatings(Patient.DFN);

      if GAF_OPS.RatingIsDue then
        lblGAFDue.Caption := 'GAF rating is due'
      else
        lblGAFDue.Caption := '';
    end;
   }
  SetTests;
end;


procedure TfrmMHA_Main.FormClose(Sender: TObject;
  var Action: TCloseAction);
begin
  FormatSettings.DateSeparator := lvDateSeparator;
  FormatSettings.TimeSeparator := lvTimeSeparator;
  FormatSettings.ShortDateFormat := lvShortDateFormat;
  FormatSettings.LongTimeFormat := lvLongTimeFormat;

  gvLeft := Left;
  gvTop := Top;
  MHA_Registry.SaveRegistry;
end;

procedure TfrmMHA_Main.FormDestroy(Sender: TObject);
begin
  if GAF_OPS <> nil then
    GAF_OPS.Free;

  gvAPICalls.Free;
  gvFont.Free;
end;

procedure TfrmMHA_Main.ConfigureForm;
var
  i                           : Integer;
begin

  with GUI_Control do
    begin
      NeedGUIUpdate := False;
      Scaled := True;
      AutoScroll := False;
      // Note: Set AutoSize to true to handle 120 DPI. Otherwise, the lower
      // portion is truncated and no GAF text appears.
      AutoSize := True;
      PixelsPerInch := 120;

      BorderStyle := bsSingle; // don't allow resizing on Main form
      KeyPreview := True;
      HorzScrollBar.Range := 0; // Ensures that buttons will move when re-sized
      VertScrollBar.Range := 0;
      Font.Assign(gvFont);

      Caption := MHA_Caption + ' in use by ' + ThisUser.Name;
      pnlHeader.Height := pnlHeaderHeight;

      for i := 0 to ComponentCount - 1 do
        begin
          if (Components[i] is TLabel) then
            begin
              TLabel(Components[i]).Font.Name := gvFont.Name;
              TLabel(Components[i]).Font.Size := lblPatientFontSize;
            end;

          if (Components[i] is TRzLabel) then
            begin
              TRzLabel(Components[i]).Font.Name := gvFont.Name;
              TRzLabel(Components[i]).Font.Size := FontSizeMenuSelections;
              TRzLabel(Components[i]).Font.Style := gvFont.Style;
            end;
        end;

      // patch 108 changes for 508 compliance llh
      // next line way it was, added if else block
      // lblPatient.caption := Patient.CaptionInfo;
      if not gvUseScreenReader then
        begin
          lblPatient.Visible := true;
          lblPatientScrnRder.Visible := false;
          lblPatient.caption := Patient.CaptionInfo;
          pnlHeader.TabStop := false;
        end
      else
        begin
          lblPatient.Visible := false;
          lblPatientScrnRder.Visible := true ;
          lblPatientScrnRder.Caption := 'Mental Health Assistant ' + Patient.CaptionInfo ;
          pnlHeader.TabStop := true ;
        end;

      Left := GetLeft(Width);
      Top := GetTop(Height);
      // patch 108 llh -- the GAF is being removed in this patch
      {
      lblGAFDue.width := lblGAF.width;
      lblGAFDue.Left := lblGAF.left;
      lblGAFDue.Font.Name := 'MS Sans Serif';
      lblGAFDue.Font.Size := 9;
      lblGAFDue.Font.Style := [fsBold];
      lblGAFDue.Top := frmMHA_Main.Height - pnlHeader.Height -
        (3 * lblGAFDUE.Height);
      }
    end;

  if gvUseScreenReader then
    SetupScreenReader(nil); // visual stuff
end;


function TfrmMHA_Main.GetDPI: integer;
// Usual setting is 96, but some use 120.
var
  DC                          : hDC;
begin
  DC := GetDC(Handle);
  Result := GetDeviceCaps(DC, LOGPIXELSX);
  ReleaseDC(Handle, DC);
end;

procedure TfrmMHA_Main.IsScreenReaderInUse;
var
  ListStateOn                 : longbool;
  Success                     : longbool;
begin

  // Determine if a screen reader is currently being used.
  Success := SystemParametersInfo(SPI_GETSCREENREADER, 0, @ListStateOn, 0);

  if Success and ListStateOn then
    begin
      gvUseScreenReader := True;
      gvUseDesignFont := false;
      gvFont.Name := 'MS Sans Serif';
      gvFont.Size := 8;
      gvFont.Style := [];
    end
  else
    begin
    // Allow user's preference to remain
      if gvUseScreenReader = True then
        if (MsgDlgFrm('The presence of a screen reader has not been ' +
          'detected, yet the user preference is to present MHA as if a ' +
          'screen reader were in use. ' + #13 + #13 +
          'Do you wish to continue in the screen reader mode?',
          MHA_Caption, mtConfirmation,
          [mbYes, mbNo], 0) = mrYes) then
          exit;

      gvUseScreenReader := False;
    end;

  MHA_Registry.SaveRegistry;
end;

procedure TfrmMHA_Main.SetupScreenReader(Sender: TObject);
// XE3 migration llh, commented out, not used
//var
  //reply                       : TModalResult;


  function FormatSSN(ASSN: string): string;
  begin
    if (Length(ASSN) > 8)
      and not (Copy(ASSN, 1, 1) = '*') then
      Result := Copy(ASSN, 1, 3) + '-' + Copy(ASSN, 4, 2) + '-'
        + Copy(ASSN, 6, Length(ASSN))
    else
      Result := ASSN;
  end;

begin

  Caption := 'MHA Main Menu. Patient is ' + Patient.Name +
    ' SSN ' + FormatSSN(Patient.SSN);
  imgTests.visible := false;
  imgResults.visible := false;
  // patch 108 llh -- the GAF is being removed in this patch
  //imgGAF.visible := false;
  imgASI.visible := false;
  lblInstAdmin.visible := False;
  lblResults.visible := False;
  // patch 108 llh -- the GAF is being removed in this patch
  //lblGAF.visible := False;
  lblASI.visible := False;
  pnlScreenReader.Visible := True;
  pnlScreenReader.align := alClient;
  refresh;

  with GUI_Control do
    begin
      btnInstAdmin.Font.Assign(gvFont);
      btnResults.Font.Assign(gvFont);
      btnASI.Font.Assign(gvFont);
      // patch 108 llh -- the GAF is being removed in this patch
      //btnGAF.Font.Assign(gvFont);
      btnInstAdmin.Height := ButtonHeight;
      btnResults.Height := btnInstAdmin.Height;
      btnASI.Height := btnInstAdmin.Height;
      // patch 108 llh -- the GAF is being removed in this patch
      //btnGAF.Height := btnInstAdmin.Height;

      btnInstAdmin.Left := 0;
      btnResults.Left := btnInstAdmin.Left;
      btnASI.Left := btnInstAdmin.Left;
      // patch 108 llh -- the GAF is being removed in this patch
      //btnGAF.Left := btnInstAdmin.Left;

      btnInstAdmin.Width := ClientWidth;
      btnResults.Width := btnInstAdmin.Width;
      btnASI.Width := btnInstAdmin.Width;
      // patch 108 llh -- the GAF is being removed in this patch
      //btnGAF.Width := btnInstAdmin.Width;

      btnInstAdmin.Top := (pnlScreenReader.Height - (4 * btnInstAdmin.Height))
        div 5;
      btnResults.Top := 2 * btnInstAdmin.Top + btnInstAdmin.Height;
      btnASI.Top := (3 * btnInstAdmin.Top) + (2 * btnInstAdmin.Height);
      // patch 108 llh -- the GAF is being removed in this patch
      //btnGAF.Top := (4 * btnInstAdmin.Top) + (3 * btnInstAdmin.Height);
    end;

  // patch 108 llh -- the GAF is being removed in this patch
  //btnGAF.Visible := (DataBaseMode = dbVista);
  btnASI.Visible := (DataBaseMode = dbVista);
  btnResults.Visible := (DataBaseMode = dbVista);

(*
  // Ensure that configuration file is present.
  if not FileExists(gvJAWS_Dir) then
    begin
      if FileExists('YS_MHA.JCF') then
        begin
          CopyFile(PChar('YS_MHA.JCF'), PChar(gvJAWS_Dir), False);

          if GetLastError <> 0 then // zero = No errors
            begin
              reply := MsgDlgFrm('If you have JAWS (a screen reader program) ' +
                'installed on this PC, the JAWS configuration file, YS_MHA.JCF, ' +
                'is not properly installed. ' + #13 + 'To find the proper directory ' +
                'use the OK button, otherwise use the Cancel button.',
                MHA_Caption, mtInformation, [mbOK, mbCancel], 0);

              if reply = mrCancel then
                Exit;

              dlgDir := TRzSelectFolderDialog.Create(self);
              dlgDir.Title := 'Select directory where JAWS configuration files are located. ' +
                'This program will copy its configuration file to that location.';
              // dlgDir.BaseFolder uses 'C:\Documents and Settings\All Users\application data\freedom scientific\jaws';
              dlgDir.BaseFolder.PathName := 'C:\Documents and Settings\All Users\application data\freedom scientific\jaws';
              if dlgDir.Execute then
                begin
                  gvJAWS_Dir := dlgDir.SelectedPathName + 'YS_MHA.JCF';
                  CopyFile(PChar('YS_MHA.JCF'), PChar(gvJAWS_Dir), False);
                end;
              dlgDir.Free;

            end;
        end;
    end;*)
end;

procedure TfrmMHA_Main.ShowSingleInstrumentMenu;
begin
  gvLeft := Left;
  gvTop := Top;
  gvHeight := Height;
  gvWidth := Width;
  MHA_Registry.SaveRegistry;

  Hide; // Main form

  Screen.Cursor := crHourglass;

  OpenFormName := ofSingleMenu;

  if UpperCase(ParamStr(6)) = 'ASI' then
    begin
      OpenFormName := ofASIMenu;
      try
        frmMHA_ASI_Main := TfrmMHA_ASI_Main.Create(nil);
        frmMHA_ASI_Main.ShowModal;
      finally
        frmMHA_ASI_Main.Free;
      end;
      exit;
    end;

  //ver 70 patch 108 llh - remove ability to run a GAF
  {
  if UpperCase(ParamStr(6)) = 'GAF' then
    begin
      OpenFormName := ofGAF;
      try
        frmMHA_GAF := TfrmMHA_GAF.Create(nil);
        frmMHA_GAF.ShowModal;
      finally
        frmMHA_GAF.Free;
      end;
      exit;
    end;
  }
  try
    frmMHA_SingleInstrumentMenu := TfrmMHA_SingleInstrumentMenu.Create(nil);
    frmMHA_SingleInstrumentMenu.ShowModal;
  finally
    frmMHA_SingleInstrumentMenu.Free;
  end;

end;


procedure TfrmMHA_Main.mnuExitClick(Sender: TObject);
begin
  Close;
end;


procedure TfrmMHA_Main.WaitingDisplay;
begin
  // Save current location when this form is re-activated.
  gvTop := Top;
  gvLeft := Left;
  gvWidth := Width;
  gvHeight := Height;
  MHA_Registry.SaveRegistry;
  Screen.cursor := crHourGlass;
end;


procedure TfrmMHA_Main.LoadPatientSelectionForm;
// Working off-line: Select patient, then the instrument(s),
// administer the instrument(s), save data to local files (to be
// up-loaded later, when connected to Vista).
begin
  WaitingDisplay;

  try
    frmMHA_SelectPatient := TfrmMHA_SelectPatient.Create(nil);
    frmMHA_SelectPatient.ShowModal;
  finally
    frmMHA_SelectPatient.Free;
  end;
end;


procedure TfrmMHA_Main.LoadInstrumentSelectForm;
begin
  OpenFormName := ofSelect;

  try
    frmMHA_InstrumentSelect := TfrmMHA_InstrumentSelect.Create(nil);
    frmMHA_InstrumentSelect.ShowModal;
  finally
    frmMHA_InstrumentSelect.Free;
  end; //try

  SetTests;
end;

procedure TfrmMHA_Main.ShowTestsSelect;
begin
  if lvInProcess then
    Exit;

  lvInProcess := True;

  //if no patient selected, select one
  if not Patient.IsSelected then
    PromptPatientSelection('No instruments can be administered ');

  //if there still isn't one selected, exit
  if not Patient.IsSelected then
    Exit;

  if not EnterDataForDeadPatient then
    Exit;

  if not gvUseScreenReader then
    begin
      lblASI.Visible := False;
      // patch 108 llh -- the GAF is being removed in this patch
      //lblGAF.Visible := False;
      lblResults.Visible := False;
      // patch 108 508 changes llh
      pnlHeader.TabStop := false;
    end;

  WaitingDisplay;
  LoadInstrumentSelectForm;
  lvInProcess := False;

  if not gvUseScreenReader then
    begin
      lblInstAdmin.Visible := True;
      // patch 108 llh -- the GAF is being removed in this patch
      //lblGAF.Visible := True;
      lblASI.Visible := True;
      //patch 108 508 changes llh
      pnlHeader.TabStop := false;
    end
    // patch 108 508 changes
  else
    begin
      pnlHeader.TabStop := true ;
      ActiveControl := lblPatientScrnRder;
    end;
end;

procedure TfrmMHA_Main.mnuOptionsClick(Sender: TObject);
begin
  if gvUseScreenReader then
    begin
      try
        frmMHA_Options_508 := TfrmMHA_Options_508.Create(nil);
      finally
        frmMHA_Options_508.Free;
      end;

    end
  else
    begin
      WaitingDisplay;

      gvShowOptionTabs := '02';

      try
        frmMHA_Options := TfrmMHA_Options.Create(nil);
      finally
        frmMHA_Options.Free;
      end;
    end;

  if GUI_Control.NeedGUIUpdate then
    ConfigureForm;

  if gvUseScreenReader then
    begin
      SetupScreenReader(Sender);
      ActiveControl := btnInstAdmin;
    end
  else
    begin
      ShowOrHideImage;
      SetTests;
    end;

  Screen.Cursor := crDefault;

end;


procedure TfrmMHA_Main.mnuSelectPatientClick(Sender: TObject);
var
  Save_Cursor                 : TCursor;
  OldDFN                      : string;

begin
  OldDFN := Patient.DFN;
  OpenFormName := ofMain;

  Save_Cursor := Screen.Cursor;
  Screen.Cursor := crHourglass; { Show hourglass cursor }
  try
    Patient_OPS.SelectUsingCPRS;
  finally
    Screen.Cursor := Save_Cursor; { Always restore to normal }
  end;

  ShowOrHideImage;
end;

//- patch 108 ver 70 llh, - removed GAF frm MHA; originally missed
{procedure TfrmMHA_Main.ShowGAF;
begin

  // patch 108 llh -- the GAF is being removed in this patch
  exit;

  if lvInProcess then
    Exit;

  lvInProcess := True;

  if not gvUseScreenReader then
    begin
      lblInstAdmin.Visible := False;
      lblASI.Visible := False;
      lblResults.Visible := False;
    end;

  //if no patient selected, select one
  if not Patient.IsSelected then
    PromptPatientSelection('GAF ratings cannot be made or shown ');

  //if there still isn't one selected, exit
  if not Patient.IsSelected then
    Exit;

  WaitingDisplay;
  OpenFormName := ofGAF;

  try
    frmMHA_GAF := TfrmMHA_GAF.Create(nil);
    frmMHA_GAF.ShowModal;
  finally
    frmMHA_GAF.Free;
  end;

  lvInProcess := False;

  if not gvUseScreenReader then
    begin
      lblInstAdmin.Visible := True;
      lblGAF.Visible := True;
      lblASI.Visible := True;
    end;
end;
}

procedure TfrmMHA_Main.ShowASI;
begin

  if lvInProcess then
    Exit;

  lvInProcess := True;

  if not gvUseScreenReader then
    begin
      lblInstAdmin.Visible := False;
      // patch 108 llh -- the GAF is being removed in this patch
      //lblGAF.Visible := False;
      lblResults.Visible := False;
    end;

  // if no patient selected, select one
  if not Patient.IsSelected then
    PromptPatientSelection('ASI results cannot be entered or shown ');

  // if there still isn't one selected, exit
  if not Patient.IsSelected then
    Exit;

  WaitingDisplay;
  OpenFormName := ofASIMenu;

  try
    frmMHA_ASI_Main := TfrmMHA_ASI_Main.Create(nil);
    frmMHA_ASI_Main.ShowModal;
  finally
    frmMHA_ASI_Main.Free;
  end;

  if not gvUseScreenReader then
    begin
      lblInstAdmin.Visible := True;
      // patch 108 llh -- the GAF is being removed in this patch
      //lblGAF.Visible := True;
      lblResults.Visible := True;
      //patch 108 508 changes llh
      pnlHeader.TabStop := false;
    end
    // patch 108 508 changes
  else
    begin
      pnlHeader.TabStop := true ;
      ActiveControl := lblPatientScrnRder;
    end;  // end 508 changes
    
  lvInProcess := False;
end;

procedure TfrmMHA_Main.PromptPatientSelection(AMessage: string);
begin
  mnuSelectPatientClick(Self);

  // if there still isn't a patient selected, show message
  if not Patient.IsSelected then
    MsgDlgFrm(AMessage + 'because no patient is selected.', MHA_Caption,
      mtInformation, [mbOK], 0);
end;

procedure TfrmMHA_Main.ShowResults;
begin
  if lvInProcess then
    Exit;

  lvInProcess := True;

  if not gvUseScreenReader then
    begin
      lblInstAdmin.Visible := False;
      // patch 108 llh -- the GAF is being removed in this patch
      //lblGAF.Visible := False;
      lblASI.Visible := False;
      //patch 108 508 changes llh
      pnlHeader.TabStop := false;
     end;
     
  // if no patient selected, select one
  if not Patient.IsSelected then
    PromptPatientSelection('No results can be shown ');

  // if there still isn't one selected, exit
  if not Patient.IsSelected then
    Exit;

  OpenFormName := ofResult;
  Application.ProcessMessages;
  WaitingDisplay;

  try
    frmMHA_InstrumentResults := TfrmMHA_InstrumentResults.Create(nil);
    frmMHA_InstrumentResults.ShowModal;
  finally
    frmMHA_InstrumentResults.Free;
  end; //try

  lvInProcess := False;

  if not gvUseScreenReader then
    begin
      lblInstAdmin.Visible := True;
      // patch 108 llh -- the GAF is being removed in this patch
      //lblGAF.Visible := True;
      lblASI.Visible := True;
      //patch 108 508 changes llh
      pnlHeader.TabStop := false;
    end
    // patch 108 508 changes
  else
    begin
      pnlHeader.TabStop := true ;
      ActiveControl := lblPatientScrnRder;
    end;

  // If Special Results Wizard has changed Special Result Tab, then
  // re-open Results
  if gvShowOptionTabs = 'SR' then
    begin
      gvShowOptionTabs := '';
      ShowResults;
    end;

end;

procedure TfrmMHA_Main.SetASI;
begin
  Screen.Cursor := crDefault;
  lvMenuMarker := 2;
  if gvUseScreenReader then
    Exit;

  if gvShowImage then
    lblASI.Font.Color := clGreen
  else
    lblASI.Font.Color := clBlack;
  // patch 108 llh -- the GAF is being removed in this patch
  //lblGAF.Font.Color := clBlack;
  lblResults.Font.Color := clBlack;
  lblInstAdmin.Font.Color := clBlack;
  // patch 108 llh -- the GAF is being removed in this patch
  //imgGAF.Visible := false;
  imgResults.Visible := false;
  imgTests.Visible := False;

  lblInstAdmin.Font.Style := [];
  lblASI.Font.Style := [fsUnderline];
  // patch 108 llh -- the GAF is being removed in this patch
  //lblGAF.Font.Style := [];
  lblResults.Font.Style := [];

  if gvShowImage then
    imgASI.Visible := True;
end;

procedure TfrmMHA_Main.SetGAF;
begin
  Screen.Cursor := crDefault;
  lvMenuMarker := 3;
  if gvUseScreenReader then
    Exit;

  // patch 108 llh -- the GAF is being removed in this patch
  {
  if gvShowImage then
    lblGAF.Font.Color := clGreen
  else
    lblGAF.Font.Color := clBlack;
  }
  lblASI.Font.Color := clBlack;
  lblResults.Font.Color := clBlack;
  lblInstAdmin.Font.Color := clBlack;
  imgASI.Visible := False;
  imgResults.Visible := false;
  imgTests.Visible := false;

  lblInstAdmin.Font.Style := [];
  lblASI.Font.Style := [];
  // patch 108 llh -- the GAF is being removed in this patch
  //lblGAF.Font.Style := [fsUnderline];
  lblResults.Font.Style := [];

  // patch 108 llh -- the GAF is being removed in this patch
  //if gvShowImage then imgGAF.Visible := True;
end;

procedure TfrmMHA_Main.SetResults;
begin
  Screen.Cursor := crDefault;
  lvMenuMarker := 1;
  if gvUseScreenReader then
    Exit;

  if gvShowImage then
    lblResults.Font.Color := clGreen
  else
    lblResults.Font.Color := clBlack;
  lblASI.Font.Color := clBlack;
  // patch 108 llh -- the GAF is being removed in this patch
  //lblGAF.Font.Color := clBlack;
  lblInstAdmin.Font.Color := clBlack;

  lblInstAdmin.Font.Style := [];
  lblASI.Font.Style := [];
  // patch 108 llh -- the GAF is being removed in this patch
  //lblGAF.Font.Style := [];
  lblResults.Font.Style := [fsUnderline];

  imgASI.Visible := false;
  // patch 108 llh -- the GAF is being removed in this patch
  //imgGAF.Visible := False;
  imgTests.Visible := False;
  if gvShowImage then
    imgResults.Visible := True;
  lvMenuMarker := 1;
end;

procedure TfrmMHA_Main.SetTests;
begin
  Screen.Cursor := crDefault;
  lvMenuMarker := 0;

  if gvUseScreenReader then
    begin
      // patch 108 to make 508 compliant llh  commented out next line, added AC := pnlHeader...
      //ActiveControl := btnInstAdmin;
      pnlHeader.TabStop := true ;
      ActiveControl := lblPatientScrnRder;
      Exit;
    end;

  lblInstAdmin.Visible := True;
  lblASI.Font.Color := clBlack;
  // patch 108 llh -- the GAF is being removed in this patch
  //lblGAF.Font.Color := clBlack;
  lblResults.Font.Color := clBlack;

  if gvShowImage then
    lblInstAdmin.Font.Color := clGreen
  else
    lblInstAdmin.Font.Color := clBlack;

  lblInstAdmin.Font.Style := [fsUnderline];
  lblASI.Font.Style := [];
  // patch 108 llh -- the GAF is being removed in this patch
  //lblGAF.Font.Style := [];
  lblResults.Font.Style := [];

  // patch 108 llh -- the GAF is being removed in this patch
  //lblGAF.Visible := (DataBaseMode = dbVista);
  lblASI.Visible := (DataBaseMode = dbVista);
  lblResults.Visible := (DataBaseMode = dbVista);

  if gvShowImage then
    imgTests.Visible := True;

  imgASI.Visible := false;
  // patch 108 llh -- the GAF is being removed in this patch
  //imgGAF.Visible := false;
  imgResults.Visible := False;
end;

procedure TfrmMHA_Main.ShowOrHideImage;
begin
  if gvUseScreenReader then
    Exit
  else
    pnlScreenReader.Visible := False;

  if gvShowImage then
    begin
      imgTests.visible := true;
      imgResults.visible := true;
      imgGAF.visible := true;
      imgASI.visible := true;
      lblInstAdmin.Font.Color := clBlack;
      lblResults.Font.Color := clBlack;
      // patch 108 llh -- the GAF is being removed in this patch
      //lblGAF.Font.Color := clBlack;
      lblASI.Font.Color := clBlack;
      lblInstAdmin.TextStyle := tsShadow;
      lblResults.TextStyle := tsShadow;
      // patch 108 llh -- the GAF is being removed in this patch
      //lblGAF.TextStyle := tsShadow;
      lblASI.TextStyle := tsShadow;
    end
  else
    begin
      imgTests.visible := false;
      imgResults.visible := false;
      // patch 108 llh -- the GAF is being removed in this patch
      //imgGAF.visible := false;
      imgASI.visible := false;
      lblInstAdmin.Font.Color := clBlack;
      lblResults.Font.Color := clBlack;
      // patch 108 llh -- the GAF is being removed in this patch
      //lblGAF.Font.Color := clBlack;
      lblASI.Font.Color := clBlack;
      lblInstAdmin.TextStyle := tsNormal;
      lblResults.TextStyle := tsNormal;
      // patch 108 llh -- the GAF is being removed in this patch
      //lblGAF.TextStyle := tsNormal;
      lblASI.TextStyle := tsNormal;
    end;
end;

procedure TfrmMHA_Main.lblASIMouseEnter(Sender: TObject);
begin
  SetASI;
  lblASI.Font.Style := [fsUnderline];
end;


procedure TfrmMHA_Main.lblGAFMouseEnter(Sender: TObject);
begin
  SetGAF;
  lblGAF.Font.Style := [fsUnderline];
end;

procedure TfrmMHA_Main.lblTestsMouseEnter(Sender: TObject);
begin
  SetTests;
  lblInstAdmin.Font.Style := [fsUnderline];
end;

procedure TfrmMHA_Main.lblResultsMouseEnter(Sender: TObject);
begin
  SetResults;
  lblResults.Font.Style := [fsUnderline];
end;

procedure TfrmMHA_Main.mnuResultsClick(Sender: TObject);
begin
  ShowResults;
end;

procedure TfrmMHA_Main.mnuTestsClick(Sender: TObject);
begin
  ShowTestsSelect;
end;

procedure TfrmMHA_Main.mnuASIClick(Sender: TObject);
begin
  ShowASI;
end;

procedure TfrmMHA_Main.mnuGAFClick(Sender: TObject);
begin
  // patch 108 llh -- the GAF is being removed in this patch
  //ShowGAF;
end;

procedure TfrmMHA_Main.mnuAboutClick(Sender: TObject);
begin
  WaitingDisplay;

  frmMHA_About := TfrmMHA_About.Create(nil);
  try
    frmMHA_About.ShowModal;
  finally
    frmMHA_About.Free;
  end; //try

  SetTests;

end;

procedure TfrmMHA_Main.mnuUploadResultsClick(Sender: TObject);
begin
  OpenFormName := ofUpload;
  WaitingDisplay;

  try
    frmMHA_UploadResults := TfrmMHA_UploadResults.Create(nil);
    frmMHA_UploadResults.ShowModal;
  finally
    frmMHA_UploadResults.Free;
  end;

  // In case all off line files are uploaded (and deleted) on Form.Show
  mnuUploadResults.Enabled :=
    LocalFiles.OffLineAdministrationFilesExist;

  Show;
  SetTests;
  OpenFormName := ofMain;

end;

procedure TfrmMHA_Main.mnuMetricClick(Sender: TObject);
begin
  VisitURL(MetricURL);
end;

function TfrmMHA_Main.EnterDataForDeadPatient: BOOLEAN;
begin
  Result := True;

  if Patient.IsDead then
    Result := (MsgDlgFrm(Patient.Name + ' died on ' + Patient.DOD +
      '.  Are you sure '
      + 'you want to enter new instrument data for this patient?', MHA_Caption,
      mtConfirmation, [mbYes, mbNo], 0) = mrYes);
end;

procedure TfrmMHA_Main.lblTestsMouseLeave(Sender: TObject);
begin
  lblInstAdmin.Font.Style := [];
end;

procedure TfrmMHA_Main.lblResultsMouseLeave(Sender: TObject);
begin
  lblResults.Font.Style := [];
end;

procedure TfrmMHA_Main.lblASIMouseLeave(Sender: TObject);
begin
  lblASI.Font.Style := [];
end;

procedure TfrmMHA_Main.lblGAFMouseLeave(Sender: TObject);
begin
  lblGAF.Font.Style := [];
end;

procedure TfrmMHA_Main.mnuOnlineSupportClick(Sender: TObject);
begin
  VisitURL(MHA_ONLINE_SUPPORT_URL);
end;

procedure TfrmMHA_Main.btnInstAdminClick(Sender: TObject);
begin
  ShowTestsSelect;
end;

procedure TfrmMHA_Main.btnResultsClick(Sender: TObject);
begin
  ShowResults;
end;

procedure TfrmMHA_Main.btnASIClick(Sender: TObject);
begin
  ShowASI;
end;

procedure TfrmMHA_Main.btnGAFClick(Sender: TObject);
begin
  // patch 108 llh - removed ability to enter GAF's, not sure what else needs to be done
  //                 also made the labels and button on the main menu not visible.
  //ShowGAF;
end;

procedure TfrmMHA_Main.FormKeyDown(Sender: TObject; var Key: Word;
  Shift: TShiftState);
var
  TempInt                     : Integer;
begin

  if (Shift = [ssALT]) and (Key = vk_F4) then
    Close;

  if gvUseScreenReader then
    Exit;

  TempInt := lvMenuMarker;

  if (Key = vk_Return) and not lvInProcess then
    begin
      case lvMenuMarker of
        0: ShowTestsSelect;
        1: ShowResults;
        2: ShowASI;
        //ver 70 patch 108, remove GAF from Option menu
        //3: ShowGAF;
      end;
      Exit;
    end;

  if Key in [73, 105] then {I, i}
    begin
      ShowTestsSelect;
      Exit;
    end;
  if Key in [82, 114] then {R, r}
    begin
      ShowResults;
      Exit;
    end;
  if Key in [71, 103] then {G, g}
    begin
      //ver 70 patch 108, remove GAF from Option menu
      //ShowGAF;
      Exit;
    end;
  if Key in [65, 97] then {A, a}
    begin
      ShowASI;
      Exit;
    end;

  if ((Key = vk_Down) or (Key = vk_Right) or (Key = vk_Tab)) then
    Inc(lvMenuMarker);

  if ((Key = vk_Up) or (Key = vk_Left) or
    ((Key = vk_Tab) and (shift = [ssShift]))) then
    Dec(lvMenuMarker);

  if TempInt <> lvMenuMarker then
    begin
      if lvMenuMarker > 3 then
        lvMenuMarker := 0;
      if lvMenuMarker < 0 then
        lvMenuMarker := 3;

      case lvMenuMarker of
        0: SetTests;
        1: SetResults;
        2: SetASI;
        3: SetGAF;
      end;
    end;

  lvInProcess := False;
end;


end.

