unit fMHA_Print;

interface

uses
  Windows,
  Messages,
  SysUtils,
  Classes,
  Graphics,
  Controls,
  Forms,
  fIA_dialogs,
  Printers,
  StdCtrls,
  ComCtrls,
  ExtCtrls,
  TeeProcs,
  TeEngine,
  Chart,
  Dialogs,
  Grids,
  BaseGrid,
  AdvGrid, AdvObj;

type
  TfrmMHA_Print = class(TForm)
    PrintDialog1: TPrintDialog;
    RichEdit1: TRichEdit;
    AdvStringGrid1: TAdvStringGrid;

    procedure SetMHAPrinter;
    procedure SetSpecResultsPrinter;
    procedure CheckFullPage(NewAmount: integer; PageTotal: Boolean);
    procedure LineOut(Col, Row: Integer; TextIn: string);
    procedure MakeNewPage;
    procedure BlankLine;

    procedure PrintGridSetup(Grid: TAdvStringGrid; CaptionIn: string);
  private
    { Private declarations }
  public
    { Public declarations }
    procedure PrintRichEdit(RichEdit: TRichEdit; lvHeaderIn: string);
    procedure PrintGrid(Grid: TAdvStringGrid; CaptionIn: string;
      PrintRect: TGridRect);
    procedure PrintChart(Chart: TChart; CaptionIn: string);
    procedure PrintTable(TableIn: TStringList; CaptionIn: string);

    // Special Results Printing
    procedure PrintSpecResultHeader(aHeader: string);
    procedure PrintSpecResultsChart(Chart: TChart);
    procedure PrintSpecResultsLine(aString: string);
    procedure PrintSpecResultsQALine(aString: string);
    procedure PrintSpecResultsGBLine(aString: string);
    procedure PrintSpecResultsEnd;
  end;

var
  frmMHA_Print                : TfrmMHA_Print;

implementation

{$R *.DFM}


var
  lvPixelsInInchX,
    lvPixelsInInchY,
    lvPageWidth,
    lvPageHeight,
    //lvPrintOffSetX,
    //lvPrintOffSetY,
    lvLinesPerPage,
    lvPageCount,
    lvPageTotal,
    lvBottomMargin,
    lvLeftMargin,
    lvLineHeight,
    lvCenterPage,
    lvAmountPrinted           : Integer;

  lvHeader                    : string;

  lvQALines                   : TStringList;

  lvPrintFont                 : TFont;

procedure TfrmMHA_Print.PrintSpecResultHeader(aHeader: string);
begin
  if not PrintDialog1.Execute then
    Exit;

  SetSpecResultsPrinter;
  lvQALines := TStringList.create;
  lvAmountPrinted := lvBottomMargin; // Half inch

  lvPageCount := 1;
  lvHeader := aHeader;

  with Printer do
    begin
      BeginDoc;
      Printer.Canvas.TextOut(lvLeftMargin, lvAmountPrinted, lvHeader);
      BlankLine;
    end;
end;

procedure TfrmMHA_Print.PrintSpecResultsEnd;
// Print last page's page-number and EndDoc.
begin
  Printer.Canvas.TextOut(lvCenterPage,
    lvPageHeight - lvBottomMargin - lvLineHeight,
    'Page  ' + InttoStr(lvPageCount));

  Printer.EndDoc;
  lvQALines.Free;
end;

procedure TfrmMHA_Print.SetSpecResultsPrinter;
begin
  {- Set font for regular (non-bold) -}
  Printer.Canvas.Font.Name := 'MS Sans Serif';
  Printer.Canvas.Font.Style := [];
  Printer.Canvas.Font.Pitch := fpDefault;
  Printer.Canvas.Font.Size := 10;

  // ---- Not used as user does not set the margins in this report.
  // PHYSICALOFFSETX For printing devices: the distance  from the left edge of
  // the physical page to the left edge of the printable area, in device units.
  // For example, a printer set to print at 600 dpi on 8.5"x11" paper, that
  // cannot print on the leftmost 0.25" of paper, has a horizontal physical
  // offset of 150 device units.
  //lvPrintOffSetX := GetDeviceCaps(Printer.Handle, PHYSICALOFFSETX);

  // ---- Not used as user does not set the margins in this report.
  // PHYSICALOFFSETY	For printing devices: the distance  from the top edge of the
  // physical page to the top edge of the printable area, in device units.
  // For example, a printer set to print at 600 dpi on 8.5"x11" paper, that
  // cannot print on the topmost 0.5" of paper, has a vertical physical offset
  // of 300 device units.
  //lvPrintOffSetY := GetDeviceCaps(Printer.Handle, PHYSICALOFFSETY);

  // PHYSICALHEIGHT	For printing devices: the height of the physical page, in
  // device units. For example, a printer set to print at 600 dpi on 8.5"x11"
  // paper has a physical height value of 6600 device units. Note that the
  // physical page is almost always greater than the printable area of the page,
  // and never smaller.
  lvPageHeight := GetDeviceCaps(Printer.Handle, PHYSICALHEIGHT);

  // PHYSICALWIDTH	For printing devices: the width of the physical page, in
  // device units. For example, a printer set to print at 600 dpi on 8.5"x11"
  // paper has a physical width value of 5100 device units. Note that the
  // physical page is almost always greater than the printable area of the page,
  // and never smaller.
  lvPageWidth := GetDeviceCaps(Printer.Handle, PHYSICALWIDTH);

  // Number of pixels per logical inch along the screen width.
  lvPixelsInInchX := GetDeviceCaps(Printer.Handle, LOGPIXELSX);

  // Number of pixels per logical inch along the screen height.
  lvPixelsInInchY := GetDeviceCaps(Printer.Handle, LOGPIXELSY);

  lvLinesPerPage := 70;

  // lvLineHeight := Printer.Canvas.TextHeight('X'); does not work with postscript printers
  // Size = 9 produces approx 9 lines per inch
  // Size = 10 give 7 lines per inch
  lvLineHeight := lvPageHeight div lvLinesPerPage;

  lvBottomMargin := lvPixelsInInchY div 2;
  lvLeftMargin := lvPixelsInInchX div 2; // half an inch
  lvCenterPage := lvPageWidth div 2;
end;


procedure TfrmMHA_Print.PrintSpecResultsChart(Chart: TChart);
var
  prtRect                     : TRect;
begin
  // Is there room for the chart on this page?
  CheckFullPage((lvPageHeight div 4) + (2 * lvLineHeight), False);

  BlankLine;
  BlankLine;

  with Chart do
    begin
      Foot.Font.Size := 10;
      prtRect.Top := lvAmountPrinted;
      prtRect.Left := lvLeftMargin;
      prtRect.Bottom := lvAmountPrinted + lvPageHeight div 4;
      prtRect.Right := lvPageWidth - 2 * lvLeftMargin;
      PrintProportional := false;
      PrintPartial(prtRect);
    end;

  lvAmountPrinted := lvAmountPrinted + (lvPageHeight div 4) + (2 * lvLineHeight);
  lvQALines.Clear;
end;

procedure TfrmMHA_Print.PrintSpecResultsQALine(aString: string);
// This will print the questions and answers after the group box header is
// printed. For now the strings are saved for later use.
begin
  lvQALines.Add(aString); // Wrap and clean up text later.
end;

procedure TfrmMHA_Print.PrintSpecResultsGBLine(aString: string);
var
  i                           : integer;
begin
  BlankLine;
  // Print Group Box header, e.g., "CAGE responses for 08/18/20XX"
  PrintSpecResultsLine(aString);
  BlankLine;

  // Next, print Questions and Answers
  for i := 0 to lvQALines.Count - 1 do
    PrintSpecResultsLine(lvQALines[i]);

  lvQALines.Clear; // get ready for the next use.
end;

procedure TfrmMHA_Print.PrintSpecResultsLine(aString: string);
// This will remove #13, #10 from aString and wrap if needed.
var
  i, j,
    FullPage                  : Integer;
  TempStr1,
    TempStr2                  : string;
begin
  // Remove any carriage returns and line feeds
  TempStr2 := '';
  TempStr1 := aString;

  if Length(TempStr1) > 0 then
    begin
      for j := 1 to Length(TempStr1) do
        if (TempStr1[j] = #13) or (TempStr1[j] = #10) then
          TempStr2 := TempStr2 + ' '
        else
          TempStr2 := TempStr2 + TempStr1[j];
    end
  else
    exit; // blank line

  TempStr1 := TempStr2;
  TempStr2 := '';

  // Wrap long strings
  i := Length(TempStr1);

  FullPage := lvPageWidth - (2 * lvLeftMargin);

  // Print line as is, no wrapping needed. Done.
  if Printer.Canvas.TextWidth(TempStr1) <= FullPage then
    begin
      CheckFullPage(lvLineHeight, False);
      Printer.Canvas.TextOut(lvLeftMargin, lvAmountPrinted, TempStr1);
      lvAmountPrinted := lvAmountPrinted + lvLineHeight;
      Exit;
    end;

  // Trim letters until the string fits within the printable space. The wrap
  // within a word is handled next.
  while Printer.Canvas.TextWidth(TempStr1) > FullPage do
    begin
      // Save the trimmed material
      TempStr2 := TempStr1[i] + TempStr2;

      // Remove the last letter
      Delete(TempStr1, i, 1);
      Dec(i);
    end; // while

  // Continue backing up until a space is found
  while aString[i] <> ' ' do
    begin
      // Save the trimmed material
      TempStr2 := TempStr1[i] + TempStr2;
      // Remove the last letter
      Delete(TempStr1, i, 1);
      Dec(i);
    end;

  CheckFullPage(lvLineHeight, False);
  Printer.Canvas.TextOut(lvLeftMargin, lvAmountPrinted, TempStr1);
  lvAmountPrinted := lvAmountPrinted + lvLineHeight;

  // Repeat
  if Length(TempStr2) > 1 then
    PrintSpecResultsLine(TempStr2);
end;


procedure TfrmMHA_Print.PrintChart(Chart: TChart; CaptionIn: string);
// This will print the chart in the center of the page.
begin
  if not PrintDialog1.Execute then
    Exit;

  with Chart do
    begin
      Foot.Text.Text := CaptionIn;
      Foot.Font.Size := 10;
      PrintLandscape;
    end;
end;


procedure TfrmMHA_Print.PrintGridSetup(Grid: TAdvStringGrid; CaptionIn: string);
begin
  lvPrintFont := TFont.Create;
  with lvPrintFont do
    begin
      Name := 'Courier New';
      Size := 10;
      Style := [];
      Color := clBlack;
    end;

  with Grid.PrintSettings do
    begin
      Borders := pbNoborder;
      Centered := True;
      ColumnSpacing := 30;
      Date := ppNone;
      DateFormat := 'mm/dd/yyyy';
      FitToPage := fpNever; //  (fpNever,fpAlways,fpGrow,fpShrink,fpCustom);
      Font.Assign(lvPrintFont);
      HeaderFont.Assign(lvPrintFont);
      FooterFont.Assign(lvPrintFont);
      FooterSize := 100; // Approx 1/2 inch
      HeaderSize := 200; // Approx 1 inch
      JobName := 'ASI table';
      LeftSize := 100; // Approx 1/2 inch

      NoAutoSize := False;
      Orientation := poPortrait; //  TPrinterOrientation = (poLandscape,poPortrait);
      PageNr := ppBottomCenter;
      PageNumSep := '/';

      PagePrefix := '';
      PageSuffix := '';
      RepeatFixedCols := True;
      RepeatFixedRows := True;
      RightSize := 100; // Approx 1/2 inch
      RowSpacing := 10;
      Time := ppNone;
      Title := ppTopCenter;
      TitleSpacing := 100; // Approx 1/2 inch
      TitleText := CaptionIn;

      UseFixedHeight := False;
      UseFixedWidth := False;
    end;
end;

procedure TfrmMHA_Print.PrintTable(TableIn: TStringList; CaptionIn: string);
var
  i                           : integer;
begin
  if not PrintDialog1.Execute then
    Exit;

  SetMHAPrinter;
  lvAmountPrinted := lvBottomMargin; // Half inch
  lvPageCount := 1;
  lvHeader := CaptionIn;

  with Printer do
    begin
      BeginDoc;

      Printer.Canvas.TextOut(lvLeftMargin, lvAmountPrinted, lvHeader);
      BlankLine;
      BlankLine;
      BlankLine;

      for i := 0 to TableIn.Count - 1 do
        LineOut(lvLeftMargin, lvAmountPrinted, TableIn[i]);

      EndDoc;
    end;

end;


procedure TfrmMHA_Print.PrintGrid(Grid: TAdvStringGrid; CaptionIn: string;
  PrintRect: TGridRect);

begin
  if not PrintDialog1.Execute then
    Exit;

  PrintGridSetup(Grid, CaptionIn);
  Grid.PrintRect(PrintRect);

  lvPrintFont.Free;
end;

procedure TfrmMHA_Print.SetMHAPrinter;
begin
  {- Set font for regular (non-bold) -}
  Printer.Canvas.Font.Name := 'Courier New';
  Printer.Canvas.Font.Style := [];
  Printer.Canvas.Font.Pitch := fpFixed;
  //lvLineHeight := Printer.Canvas.TextHeight('X'); does not work with postscript printers
  // Size = 9 produces approx 9 lines per inch
  // Size = 10 give 7 lines per inch

   // ---- Not used as user does not set the margins in this report.
  // PHYSICALOFFSETX For printing devices: the distance  from the left edge of
  // the physical page to the left edge of the printable area, in device units.
  // For example, a printer set to print at 600 dpi on 8.5"x11" paper, that
  // cannot print on the leftmost 0.25" of paper, has a horizontal physical
  // offset of 150 device units.
  //lvPrintOffSetX := GetDeviceCaps(Printer.Handle, PHYSICALOFFSETX);

  // ---- Not used as user does not set the margins in this report.
  // PHYSICALOFFSETY	For printing devices: the distance  from the top edge of the
  // physical page to the top edge of the printable area, in device units.
  // For example, a printer set to print at 600 dpi on 8.5"x11" paper, that
  // cannot print on the topmost 0.5" of paper, has a vertical physical offset
  // of 300 device units.
  //lvPrintOffSetY := GetDeviceCaps(Printer.Handle, PHYSICALOFFSETY);

  // PHYSICALHEIGHT	For printing devices: the height of the physical page, in
  // device units. For example, a printer set to print at 600 dpi on 8.5"x11"
  // paper has a physical height value of 6600 device units. Note that the
  // physical page is almost always greater than the printable area of the page,
  // and never smaller.
  lvPageHeight := GetDeviceCaps(Printer.Handle, PHYSICALHEIGHT);

  // PHYSICALWIDTH	For printing devices: the width of the physical page, in
  // device units. For example, a printer set to print at 600 dpi on 8.5"x11"
  // paper has a physical width value of 5100 device units. Note that the
  // physical page is almost always greater than the printable area of the page,
  // and never smaller.
  lvPageWidth := GetDeviceCaps(Printer.Handle, PHYSICALWIDTH);

  // Number of pixels per logical inch along the screen width.
  lvPixelsInInchX := GetDeviceCaps(Printer.Handle, LOGPIXELSX);

  // Number of pixels per logical inch along the screen height.
  lvPixelsInInchY := GetDeviceCaps(Printer.Handle, LOGPIXELSY);

  Printer.Canvas.Font.Size := 10;
  lvLinesPerPage := 70;
  lvLineHeight := lvPageHeight div lvLinesPerPage;
 
  lvBottomMargin := lvPixelsInInchY div 2;
  lvLeftMargin := lvPixelsInInchX div 2;
  lvCenterPage := (Printer.PageWidth div 2) - lvLeftMargin;
end;


procedure TfrmMHA_Print.CheckFullPage(NewAmount: integer; PageTotal: Boolean);
var
  TempStr                     : string;
begin
  if lvAmountPrinted + lvBottomMargin + NewAmount >= lvPageHeight then
    begin

      if PageTotal then
        TempStr := 'Page ' + IntToStr(lvPageCount) + ' of ' + IntToStr(lvPageTotal)
      else
        TempStr := 'Page ' + IntToStr(lvPageCount);

      Printer.Canvas.TextOut(lvCenterPage,
        lvPageHeight - lvBottomMargin - lvLineHeight, TempStr);

      MakeNewPage;
    end;
end;


procedure TfrmMHA_Print.BlankLine;
begin
  lvAmountPrinted := lvAmountPrinted + lvLineHeight;
end;


procedure TfrmMHA_Print.LineOut(Col, Row: Integer; TextIn: string);
begin
  if POS('***eop***', TextIn) > 0 then
    begin
      Printer.Canvas.TextOut(lvCenterPage,
        lvPageHeight - lvBottomMargin - lvLineHeight,
        'Page ' + IntToStr(lvPageCount) + ' of ' + IntToStr(lvPageTotal));
      MakeNewPage;
    end
  else
    begin
      Printer.Canvas.TextOut(Col, Row, TextIn);
      lvAmountPrinted := lvAmountPrinted + lvLineHeight;
      CheckFullPage(4 * lvLineHeight, True);
    end;
end;


procedure TfrmMHA_Print.MakeNewPage;
begin
  Inc(lvPageCount);
  Printer.NewPage;
  lvAmountPrinted := lvBottomMargin; {- Forced half inch -}
  Printer.Canvas.TextOut(lvLeftMargin, lvAmountPrinted, lvHeader);
  BlankLine;
  BlankLine;
  BlankLine;

end;

procedure TfrmMHA_Print.PrintRichEdit(RichEdit: TRichEdit; lvHeaderIn: string);
var
  i                           : integer;

  function CountFixedPages: integer;
  var
    Max, Marker, Found, loop  : integer;
  begin
    Found := 0;
    Marker := 0;
    Loop := 0;
    Max := RichEdit.GetTextLen;
    while Found <> -1 do
      begin
        Found := RichEdit.FindText('***eop***', Marker,
          Max, [stWholeWord]);
        Marker := Found + 1;
        Inc(loop);
      end;
    Result := loop;
  end;

begin
  if not PrintDialog1.Execute then
    Exit;

  SetMHAPrinter;
  lvAmountPrinted := lvBottomMargin; // Half inch

  lvPageTotal := CountFixedPages;

  if lvPageTotal = 1 then
    lvPageTotal := 1 + RichEdit.Lines.Count div (lvLinesPerPage - 14);

  lvPageCount := 1;
  lvHeader := lvHeaderIn;

  with Printer do
    begin
      BeginDoc;

      Printer.Canvas.TextOut(lvLeftMargin, lvAmountPrinted, lvHeader);
      BlankLine;
      BlankLine;
      BlankLine;

      for i := 0 to RichEdit.Lines.count - 1 do
        LineOut(lvLeftMargin, lvAmountPrinted, RichEdit.Lines[i]);

      Printer.Canvas.TextOut(lvCenterPage,
        lvPageHeight - lvBottomMargin - lvLineHeight,
        'Page ' + IntToStr(lvPageCount) + ' of ' + IntToStr(lvPageTotal));

      EndDoc;
    end;
end;

end.

