unit fMHA_Print_Blank;

interface

uses
  Windows,
  Messages,
  fIA_dialogs,
  SysUtils,
  System.Contnrs,
  System.UITypes,
  Variants,
  Classes,
  Graphics,
  Controls,
  Forms,
  Printers,
  JvExControls,
  JvComponent,
  JvXPCore,
  JvXPButtons,
  ExtCtrls,
  uIA_Display,
  StdCtrls,
  JvPageSetup,
  JvBaseDlg,
  JvWinDialogs,
  JvExStdCtrls,
  JvCombobox,
  JvColorCombo,
  Dialogs;

type
  TfrmPrint_Instrument = class(TForm)
    cbInstruments: TComboBox;
    lblInstructions: TLabel;
    pnlButtons: TPanel;
    btnCancel: TJvXPButton;
    btnPrint: TJvXPButton;
    pnlHeader: TPanel;
    lblHeader: TLabel;
    JvShellAboutDialog1: TJvShellAboutDialog;
    PrintDialog1: TPrintDialog;
    lblFontSize: TLabel;
    cbFontSize: TComboBox;
    btnSetPrinter: TJvXPButton;
    cbPrintFont: TJvFontComboBox;
    lblPrintFont: TLabel;
    lblInstrument: TLabel;
    lblRecommend: TLabel;
    PageSetupDialog: TPageSetupDialog;
    procedure FormCreate(Sender: TObject);
    procedure ConfigureForm;
    procedure PopulateWithInstruments;
    procedure btnCancelClick(Sender: TObject);
    procedure btnPrintClick(Sender: TObject);

    procedure PrintThisInstrument(aInstrumentName: string);
    procedure Print_String(Display: TDisplay; TextIn: string;
      PrintIt, PageControl: boolean);
    procedure Print_MultipleChoice(Display, AnswerDisplay: TDisplay;
      List: TStringList; PrintIt: boolean);
    procedure Print_Bold(aLeft: integer; StringIn: string; aAlign: TAlignment);
    procedure Print_TrackBar_Scale(Display, AnswerDisplay: TDisplay;
      aMin, aMax: Integer; PrintIt: boolean);
    procedure Print_TrackBar_Labels(Display, AnswerDisplay: TDisplay;
      List: TStringList; PrintIt: boolean);

    procedure PrintDone;
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure cbFontSizeChange(Sender: TObject);
    procedure btnSetPrinterClick(Sender: TObject);
    procedure cbPrintFontChange(Sender: TObject);
    procedure FormResize(Sender: TObject);
    procedure FormKeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
  private
    { Private declarations }

    procedure LoadStaticText(InstrumentName: string);
    function GetStaticText(Where: integer): string;

    procedure Increase_Y_Print(Amount: integer);
    procedure StartNewPage;
    procedure SetFont(FontIn: TFont);
    procedure Print_Blank_Line(HowMany: integer);
    procedure Print_A_Straight_Line(Start, Finish: Integer);
    procedure Print_Header;
    // This will wrap line
    procedure PrintLineWithCheckBoxWrap(X, Y: Integer; Text: string;
      Display: TDisplay; PrintIt: boolean);
    // No wrap
    procedure PrintLineWithCheckBoxNoWrap(X, Y: Integer;
      Text: string; PrintIt: boolean);
    function Print_WrapLine(LongLine: string; Display: TDisplay;
      PrintIt, PageControl: boolean): string;
    procedure Print_StaticText(Display: TDisplay; TextIn: string);
    function IsThereRoom(HeightIn: integer): boolean;

    function LeftPixelsToPrinterUnits(PixelsIn: Integer): Integer;
    function RightPixelsToPrinterUnits: Integer;
    function TopPixelsToPrinterUnits: Integer;
    function BottomPixelsToPrinterUnits: Integer;
    procedure VerifyPassedInstrument;

  public
    { Public declarations }
  end;

var
  frmPrint_Instrument         : TfrmPrint_Instrument;

implementation

{$R *.dfm}

uses
  uIA_Const,
  uMHA_Const,
  uMHA_Registry,
  uIA_GUIControl,
  uIA_Utils,
  uMHA_ResourceFile_OPS,
  uIA_AvailableInstrument,
  uIA_Instrument,
  uIA_Question,
  uIA_Introduction,
  uIA_SectionLabel,
  uIA_ThisUser,
  uMHA_Instrument_OPS,
  uMHA_DM;


var
  lvInstrument                : TInstrument;

  PrinterFont                 : TFont;

  PageCount,
    lvPageWidth, // In printer units: 600 dpi would have a PageWidth of 5100
    lvPageHeight,
    Center,
    PrintOffSetX,
    PrintOffSetY,
    PixelsInInchX,
    PixelsInInchY,
    DefaultHeight,
    ComponentHeight,
    Y_Print_Total             : Integer;

  lvInitialize                : Boolean;

  lvStaticText                : string;


procedure TfrmPrint_Instrument.FormCreate(Sender: TObject);
begin
  lvInitialize := True;

  // Keep Position out of ConfigureForm when ConfigureForm can be called
  // more than once with the form.
  Position := poDesigned;

  lvInstrument := TInstrument.Create;
  PrinterFont := TFont.Create;
  PrinterFont.Name := 'MS Sans Serif';

  Caption := 'Print Assistant';
  if gvNameOfInstrument = '' then
    lblHeader.Caption := '    Print a blank instrument '
  else
    lblHeader.Caption := '    Print a blank ' + gvNameOfInstrument;

  ConfigureForm;

  // Get settings for margins
  MHA_Registry.ReadRegistry;

  PrinterFont.Size := StrToIntDef(cbFontSize.Text, 10);
  DefaultHeight := (5 * Printer.Canvas.TextHeight('Yy')) div 4;

  // PHYSICALOFFSETX For printing devices: the distance  from the left edge of
  // the physical page to the left edge of the printable area, in device units.
  // For example, a printer set to print at 600 dpi on 8.5"x11" paper, that
  // cannot print on the leftmost 0.25" of paper, has a horizontal physical
  // offset of 150 device units.
  PrintOffSetX := GetDeviceCaps(Printer.Handle, PHYSICALOFFSETX);

  // PHYSICALOFFSETY	For printing devices: the distance  from the top edge of the
  // physical page to the top edge of the printable area, in device units.
  // For example, a printer set to print at 600 dpi on 8.5"x11" paper, that
  // cannot print on the topmost 0.5" of paper, has a vertical physical offset
  // of 300 device units.
  PrintOffSetY := GetDeviceCaps(Printer.Handle, PHYSICALOFFSETY);

  // PHYSICALHEIGHT	For printing devices: the height of the physical page, in
  // device units. For example, a printer set to print at 600 dpi on 8.5"x11"
  // paper has a physical height value of 6600 device units. Note that the
  // physical page is almost always greater than the printable area of the page,
  // and never smaller.
  lvPageHeight := GetDeviceCaps(Printer.Handle, PHYSICALHEIGHT);

  // PHYSICALWIDTH	For printing devices: the width of the physical page, in
  // device units. For example, a printer set to print at 600 dpi on 8.5"x11"
  // paper has a physical width value of 5100 device units. Note that the
  // physical page is almost always greater than the printable area of the page,
  // and never smaller.
  lvPageWidth := GetDeviceCaps(Printer.Handle, PHYSICALWIDTH);

  // Number of pixels per logical inch along the screen width.
  PixelsInInchX := GetDeviceCaps(Printer.Handle, LOGPIXELSX);

  // Number of pixels per logical inch along the screen height.
  PixelsInInchY := GetDeviceCaps(Printer.Handle, LOGPIXELSY);

  Center := lvPageWidth div 2;
  PopulateWithInstruments;

  Screen.Cursor := crDefault;
  lvInitialize := False;
  ActiveControl := cbInstruments;
  ShowModal;
end;


procedure TfrmPrint_Instrument.FormClose(Sender: TObject;
  var Action: TCloseAction);
begin
  if lvInstrument <> nil then
    lvInstrument.Free;
  PrinterFont.Free;
end;


procedure TfrmPrint_Instrument.ConfigureForm;
var
  StandardHeight,
    TempHeight                : Integer;
begin
  // See also FormResize for left positions & widths.
  with GUI_Control do
    begin
      Scaled := True;
      AutoScroll := False;
      AutoSize := False;
      PixelsPerInch := 120;

      KeyPreview := True;
      HorzScrollBar.Range := 0; // Ensures that buttons will move when re-sized
      VertScrollBar.Range := 0;
      Font.Assign(gvFont);

      Width := GetWidth(6 * ButtonWidthNormal);
      StandardHeight := 3 * GUI_Control.TextHeight('X', gvFont) div 2;
      Height := GetHeight(15 * StandardHeight + pnlHeaderHeight + (3 * ButtonHeight div 2));
      Left := GetLeft(Width);
      Top := GetTop(Height);
      if CheckWidth(Width) or CheckHeight(Height) then
        WindowState := wsMaximized;
      Refresh; // resets alClient dimensions

      pnlHeader.Height := pnlHeaderHeight;
      lblHeader.Font.Size := lblHeaderFontSize;
      pnlButtons.Height := 3 * ButtonHeight div 2;

      btnCancel.Width := ButtonWidthNormal;
      btnCancel.Top := ButtonHeight div 4;
      btnCancel.Height := ButtonHeight;

      btnPrint.Top := btnCancel.Top;
      btnPrint.Height := ButtonHeight;
      btnPrint.Width := ButtonWidthNormal;

      btnSetPrinter.Top := btnCancel.Top;
      btnSetPrinter.Height := ButtonHeight;
      btnSetPrinter.Width := ButtonWidthNormal;
      btnSetPrinter.Left := (pnlButtons.Width - btnSetPrinter.Width) div 2;

      lblPrintFont.AutoSize := False;
      lblPrintFont.Alignment := taRightJustify;

      TempHeight := (StandardHeight div 2) + pnlHeader.Height;

      lblInstructions.Top := TempHeight;
      lblInstructions.Height := StandardHeight * 3;
      TempHeight := TempHeight + StandardHeight + lblInstructions.Height;

      lblInstrument.Top := TempHeight;
      lblInstrument.Height := StandardHeight;
      cbInstruments.Top := lblInstrument.Top;
      cbInstruments.Height := StandardHeight;
      TempHeight := TempHeight + StandardHeight + lblInstrument.Height;

      lblPrintFont.Top := TempHeight;
      lblPrintFont.Height := StandardHeight;
      cbPrintFont.Top := lblPrintFont.Top;
      cbPrintFont.Height := StandardHeight;
      TempHeight := TempHeight + StandardHeight + lblPrintFont.Height;

      lblFontSize.Top := TempHeight;
      lblFontSize.Height := StandardHeight;
      cbFontSize.Top := lblFontSize.Top;
      cbFontSize.Height := StandardHeight;
      TempHeight := TempHeight + StandardHeight + lblFontSize.Height;

      lblRecommend.Top := TempHeight;
      lblRecommend.Height := StandardHeight;

      // Default values are the recommended ones.
      cbFontSize.ItemIndex := 1; // size 10
      cbPrintFont.FontName := 'MS Sans Serif';

      if gvUseScreenReader then
        begin
          Left := 0;
          Width := Screen.Width;
          //btnPrint.caption := btnPrint.Caption + ' button';
          //btnCancel.caption := btnCancel.Caption + ' button';
          //btnSetPrinter.caption := btnSetPrinter.Caption + ' button';
        end;

    end;
end;


procedure TfrmPrint_Instrument.PopulateWithInstruments;
// This list will be all the instruments this user has access to administer
// and/or view the results.
var
  i, j                        : Integer;
  reply                       : string;
begin
  cbinstruments.Clear;

  // InstrumentResults does not create ListOfInstruments
  if Instrument_OPS.ListOfInstruments.Count = 0 then
    begin
      // This list is based on the privledges of the user and not those of the Ordered-By
      // as in the Instrument Administrator's list of available instruments.
      reply := Instrument_OPS.LoadListOfAvailableInstruments(ThisUser.AccessKeys);
      if reply <> OK then
        begin
          ShowMessage(' Sorry, there is an error loading the instruments: ' +
            #13 + reply);
          Exit;
        end;
    end;

  for i := 0 to Instrument_OPS.ListOfInstruments.Count - 1 do
  begin
    if (TAvailableInstrument(Instrument_OPS.ListOfInstruments[i]).R_Privilege <> '')  then
        begin
          for j := 1 to NumberofPieces(ThisUser.AccessKeys, U) do
            begin
              if POS(Piece(ThisUser.AccessKeys, U, j), TAvailableInstrument(Instrument_OPS.ListOfInstruments[i]).R_Privilege) > 0 then
                cbInstruments.Items.Add(TAvailableInstrument(Instrument_OPS.ListOfInstruments[i]).AvailInstrumentName);
            end; // for j
        end
      else
        // No restrictions on this instrument
        cbInstruments.Items.Add(TAvailableInstrument(Instrument_OPS.ListOfInstruments[i]).AvailInstrumentName);
    end;

(*
    begin
      if not TAvailableInstrument(Instrument_OPS.ListOfInstruments[i]).CanBeAdministered then
        Continue;
      if (TAvailableInstrument(Instrument_OPS.ListOfInstruments[i]).IsOperational) = 'O' then
        Continue;
      if not TAvailableInstrument(Instrument_OPS.ListOfInstruments[i]).CopyrightStatusIsOK then
        Continue;

      if TAvailableInstrument(Instrument_OPS.ListOfInstruments[i]).RequiresLicense then
        begin
          if not TAvailableInstrument(Instrument_OPS.ListOfInstruments[i]).LicenseIsCurrent then
            Continue;
        end;

      // Allow users with the proper key to print the instrument
      if (TAvailableInstrument(Instrument_OPS.ListOfInstruments[i]).R_Privilege <> '') or
        (TAvailableInstrument(Instrument_OPS.ListOfInstruments[i]).A_Privilege <> '') then
        begin
          for j := 1 to NumberofPieces(ThisUser.AccessKeys) do
            begin
              if POS(Piece(ThisUser.AccessKeys, U, j), TAvailableInstrument(Instrument_OPS.ListOfInstruments[i]).R_Privilege) > 0 then
                cbInstruments.Items.Add(TAvailableInstrument(Instrument_OPS.ListOfInstruments[i]).AvailInstrumentName);
              if POS(Piece(ThisUser.AccessKeys, U, j), TAvailableInstrument(Instrument_OPS.ListOfInstruments[i]).A_Privilege) > 0 then
                cbInstruments.Items.Add(TAvailableInstrument(Instrument_OPS.ListOfInstruments[i]).AvailInstrumentName);
            end; // for j
        end
      else
        // No restrictions on this instrument
        cbInstruments.Items.Add(TAvailableInstrument(Instrument_OPS.ListOfInstruments[i]).AvailInstrumentName);
    end; // for i
*)

  if gvNameOfInstrument = '' then
    cbInstruments.ItemIndex := 0
  else
    VerifyPassedInstrument;
end;


procedure TfrmPrint_Instrument.VerifyPassedInstrument;
var
  i                           : Integer;
  OK                          : Boolean;
begin
  OK := False;

  for i := 0 to cbInstruments.Items.Count - 1 do
    begin
      if UpperCase(gvNameOfInstrument) = UpperCase(cbInstruments.Items[i]) then
        begin
          cbInstruments.Text := gvNameOfInstrument;
          OK := True;
          break;
        end;
    end;

  if not OK then
    ShowMessage('Sorry, the instrument that you wish to print can not be ' +
      'printed. This may be due to a copyright issue, or the instrument''s ' +
      'license has expired.');
end;

procedure TfrmPrint_Instrument.btnCancelClick(Sender: TObject);
begin
  Close;
end;

procedure TfrmPrint_Instrument.btnPrintClick(Sender: TObject);
begin
  PageCount := 1;
  Y_Print_Total := TopPixelsToPrinterUnits;

  if cbInstruments.Text = '' then
    begin
      ShowMessage('No instrument selected.');
      Exit;
    end;

  if PrintDialog1.Execute then
    try
      lblHeader.Caption := '    Printing... ';
      cbInstruments.Enabled := False;
      btnPrint.Enabled := False;

      PrintThisInstrument(cbInstruments.Text);
    except
      on E: exception do
        begin
          ShowMessage('Printing error: ' + E.message);
          exit;
        end;
    end; {- try -}

  lblHeader.Caption := '    Print a blank instrument ';
  cbInstruments.Enabled := True;
  btnPrint.Enabled := True;
end;

function TfrmPrint_Instrument.IsThereRoom(HeightIn: integer): boolean; // Add new line(s) to print total: check for new page.
// The amount passed is the total height for the component so that
// questions are not removed from answers and multiple-choice answers
// are all on the same page.
begin
  result := true;

  if HeightIn + Y_Print_Total +
    (2 * DefaultHeight) + // room for Page numbers
    BottomPixelsToPrinterUnits >= lvPageHeight then
    Result := False;
end;

procedure TfrmPrint_Instrument.PrintThisInstrument(aInstrumentName: string);
var
  i, j                        : Integer;
  TempStr                     : string;
  TempStrList                 : TStringList;

  procedure Build_Items(Designator: TChoiceIdentifier;
    var StringListIn: TStringList);
  // This will add the designator to the text of the item; it could absent
  // or start with a "0." or "1."
  var
    i                         : Integer;
    TempSL                    : TStringList;
  begin
    if StringListIn.Count = 0 then
      Exit;

    TempSL := TStringList.Create;

    for i := 0 to StringListIn.Count - 1 do
      begin
        case Designator of
          ciNone: TempSL.Add(Piece(StringListIn[i], U, 3));
          ciZeroBased: TempSL.Add(inttostr(i) + '. ' + Piece(StringListIn[i], U, 3));
          ciOneBased: TempSL.Add(inttostr(i + 1) + '. ' + Piece(StringListIn[i], U, 3));
        end; //case Question.ChoiceIdentifier
      end;

    StringListIn.Clear;
    StringListIn.AddStrings(TempSL);
    TempSL.Free;
  end;

begin
  Printer.BeginDoc;

  lvInstrument.InstrumentName := aInstrumentName;

  // Look for the instrument on the local directory
  if not ResourceFile_OPS.ReadResourceFile(lvInstrument) then
    begin
      // Get instrument from Vista: slower
      Instrument_OPS.Load(lvInstrument);
      // Make a copy on the local drive for next time
      ResourceFile_OPS.WriteResourceFile(lvInstrument);
    end;

  TempStrList := TStringList.Create;

   // Build static text from Vista lookup: handled by DLL
  LoadStaticText(lvInstrument.InstrumentName);

  if lvInstrument.PrintTitle = '' then
    Print_Bold(0, lvInstrument.InstrumentName, taCenter)
  else
    Print_Bold(0, lvInstrument.PrintTitle, taCenter);
  Print_Blank_Line(4);

  Print_Header; // Name, date, etc.

  for i := 0 to lvInstrument.ListOfQuestions.Count - 1 do
    begin
      with TQuestion(lvInstrument.ListOfQuestions[i]) do
        begin

          // Check for a section label
          for j := 0 to lvInstrument.ListOfSections.Count - 1 do
            if (TSectionLabel(lvInstrument.ListOfSections.Items[j]).QuestionID = QuestionID) then
              begin
                Print_Blank_Line(2);
                Print_Bold(
                  LeftPixelsToPrinterUnits(
                  TSectionLabel(lvInstrument.ListOfSections.Items[j]).Display.DisplayLeft),
                  TSectionLabel(lvInstrument.ListOfSections.Items[j]).SectionCaption,
                  TSectionLabel(lvInstrument.ListOfSections.Items[j]).Display.DisplayAlignment);
                Print_Blank_Line(1);
              end;

          // Build the introduction if there is one.
          if TIntroduction(lvInstrument.ListOfIntroductions[i]).iText <> '' then
            if TIntroduction(lvInstrument.ListOfIntroductions[i]).IsNew then
              Print_String(TIntroduction(lvInstrument.ListOfIntroductions[i]).Display,
                TIntroduction(lvInstrument.ListOfIntroductions[i]).iText, TRUE, TRUE {Page Control});

          if (lvStaticText <> '') then
            begin
              TempStr := GetStaticText(i);
              if TempStr <> '' then
                begin
                  if lvInstrument.ListOfIntroductions.Count > 0 then
                    Print_StaticText(
                      TIntroduction(lvInstrument.ListOfIntroductions[0]).Display,
                      TempStr)
                  else
                    Print_StaticText(Display, TempStr);
                end;
            end;

          // Treat the component as a complete unit so that question text is
          // not on one page while the answers are on another
          ComponentHeight := 0;

          // Get the height of the component; wrapped strings are handled
          Print_String(Display, qCaption, FALSE {No print}, FALSE {Page Control});

          if (ResponseType = rtMultiChoice) and
            ((AnswerDisplay.DisplayComponent = acRadioGroup) or
            (AnswerDisplay.DisplayComponent = acComboBox) or
            (AnswerDisplay.DisplayComponent = acListBox)) then
            begin
              TempStrList.Clear;
              TempStrList.AddStrings(MultiChoice.Choices);
              Build_Items(ChoiceIdentifier, TempStrList);

              Print_MultipleChoice(Display, AnswerDisplay, TempStrList, FALSE {No print});

              if not IsThereRoom(ComponentHeight) then
                StartNewPage;

              Print_String(Display, qCaption, TRUE, TRUE {Page Control});
              Print_MultipleChoice(Display, AnswerDisplay, TempStrList, TRUE {Print});
            end;

          if ResponseType = rtCheckList then
            begin
              TempStrList.Clear;
              TempStrList.AddStrings(MultiChoice.Choices);
              Build_Items(ChoiceIdentifier, TempStrList);
              Print_MultipleChoice(Display, AnswerDisplay, TempStrList, FALSE {No print});

              if not IsThereRoom(ComponentHeight) then
                StartNewPage;

              Print_String(Display, qCaption, TRUE, TRUE {Page Control});
              Print_MultipleChoice(Display, AnswerDisplay, TempStrList, TRUE {Print});
            end;

          if ResponseType = rtTrackBar then
            begin
              Print_TrackBar_Scale(Display, AnswerDisplay, qMin, qMax, FALSE);

              if MultiChoice.Choices.Count > 0 then
                begin
                  TempStrList.Clear;
                  TempStrList.AddStrings(MultiChoice.Choices);
                  Build_Items(ChoiceIdentifier, TempStrList);
                  Print_TrackBar_Labels(Display, AnswerDisplay, TempStrList, FALSE {No print});
                end;

              if not IsThereRoom(ComponentHeight) then
                StartNewPage;

              Print_String(Display, qCaption, TRUE {Print}, TRUE {Page Control});
              Increase_Y_Print(-DefaultHeight div 2);
              Print_TrackBar_Scale(Display, AnswerDisplay, qMin, qMax, TRUE {Print});
              Increase_Y_Print(-DefaultHeight);

              if MultiChoice.Choices.Count > 0 then
                Print_TrackBar_Labels(Display, AnswerDisplay, TempStrList, True {Print});
            end;

          if (ResponseType = rtString) or
            (ResponseType = rtDate) or
            (ResponseType = rtMaskEdit) or
            (ResponseType = rtInteger) or
            (ResponseType = rtVista) or
            (ResponseType = rtCurrency) then
            begin
              if Display.DisplaySameLine then
                begin
                  if not IsThereRoom(ComponentHeight) then
                    StartNewPage;
                  Print_String(Display, qCaption, TRUE, TRUE {Page Control});
                  Print_A_Straight_Line(AnswerDisplay.DisplayLeft, AnswerDisplay.DisplayLeft + 60);
                end
              else
                begin
                  ComponentHeight := ComponentHeight + DefaultHeight;
                  if not IsThereRoom(ComponentHeight) then
                    StartNewPage;
                  Print_String(Display, qCaption, TRUE, TRUE {Page Control});
                  Print_A_Straight_Line(Display.DisplayLeft, Display.DisplayLeft + 60);
                  Increase_Y_Print(DefaultHeight);
                end;
            end;

          if ResponseType = rtMemo then
            begin
              ComponentHeight := ComponentHeight + (4 * DefaultHeight);
              if not IsThereRoom(ComponentHeight) then
                StartNewPage;

              Print_String(Display, qCaption, TRUE, TRUE {Page Control});
              Print_A_Straight_Line(0, 60);
              Increase_Y_Print(DefaultHeight);
              Print_A_Straight_Line(0, 60);
              Increase_Y_Print(DefaultHeight);
              Print_A_Straight_Line(0, 60);
              Increase_Y_Print(DefaultHeight);
            end;
        end; // with TQuestion(Instrument.ListOfQuestions[i])
    end; // for i

  TempStrList.Free;

  PrintDone;
  Printer.EndDoc;
end;

procedure TfrmPrint_Instrument.PrintDone;
// Print the last page number
begin
  while Y_Print_Total + (2 * DefaultHeight) + // room for Page numbers
    BottomPixelsToPrinterUnits <= lvPageHeight do
    Y_Print_Total := Y_Print_Total + DefaultHeight;

  Printer.Canvas.TextOut(Center - (Printer.Canvas.TextWidth('Page XX') div 2),
    Y_Print_Total,
    'Page ' + InttoStr(PageCount));
end;

function TfrmPrint_Instrument.GetStaticText(Where: integer): string;
var
  Count,
    i                         : Integer;
  TempStr                     : string;
begin
  result := '';
  if lvStaticText = '' then
    Exit;
  Count := NumberOfPieces(lvStaticText, U);

  if Count = 0 then
    Exit;

  for i := 1 to Count do
    begin
      TempStr := Piece(lvStaticText, U, i);
      if IntToStr(Where) = Piece(TempStr, EQ, 1) then
        result := Piece(TempStr, EQ, 2);
    end;

end;

procedure TfrmPrint_Instrument.LoadStaticText(InstrumentName: string);
var
  ResultString                : string;
  dllHandle                   : THandle;
begin
  // patch 105 added gvAuxFile which = proper C:\Program Files dir for the YS_MHA_AUX file llh
  dllHandle := LoadLibrary(PChar(gvAuxFile));

  if dllHandle = 0 then
    begin
      // patch 105 added gvAuxFile, removed C:\Program Files llh
      ShowMessage('Could not find DLL used for scoring instruments.' +
      'The file YS_MHA_AUX.DLL is usually located in ' + gvAuxFile);
    end
  else
    begin
      try
        @StaticLookupDLL := GetProcAddress(dllHandle, 'StaticLookup');

        if @StaticLookupDLL = nil then
          // function not found
          Exit;

        if Assigned(StaticLookupDLL) then
          StaticLookupDLL(
            InstrumentName,
            '', //PatientIEN
            '', // OrderedByDUZ
            ResultString);

        lvStaticText := ResultString;
        // Format is:
        // #=Text^#=text

      finally
        FreeLibrary(dllHandle);
      end; {try..finally}
    end;
end;

function TfrmPrint_Instrument.LeftPixelsToPrinterUnits(PixelsIn: Integer): Integer;
// PixelsIn is in pixels (96 per inch usually), converted to printer units (DPI, 600
// per inch is typical).
// There are three factors to left margin:
//    1. Printer offset (minimum margin set by printer limitation)
//    2. Margin set in Page Setup by user
//    3. Left setting provided by the component (PixelsIn)
begin
  result := Round(((gvLeftMargin + PixelsIn) / PixelsPerInch) * PixelsInInchX) -
    PrintOffSetX;

 // Avoid negative margins
  if Result < 1 then
    Result := PrintOffSetX;
end;

function TfrmPrint_Instrument.RightPixelsToPrinterUnits: Integer;
// PixelsIn is in pixels (96 per inch usually), converted to printer units (DPI, 600
// per inch is typical). The printer offset is factored in.
begin
  result := Round((gvRightMargin / PixelsPerInch) * PixelsInInchX) +
    PrintOffSetX;

  // Avoid negative margins
  if Result < 1 then
    Result := PrintOffSetX;
end;

function TfrmPrint_Instrument.TopPixelsToPrinterUnits: Integer;
// PixelsIn is in pixels (96 per inch usually), converted to printer units (DPI, 600
// per inch is typical). The printer offset is factored in.
begin
  result := Round((gvTopMargin / PixelsPerInch) * PixelsInInchY) -
    PrintOffSetY;

  // Avoid negative margins
  if Result < 1 then
    Result := PrintOffSetY;
end;

function TfrmPrint_Instrument.BottomPixelsToPrinterUnits: Integer;
// PixelsIn is in pixels (96 per inch usually), converted to printer units (DPI, 600
// per inch is typical). The printer offset is factored in.
begin
  result := Round((gvBottomMargin / PixelsPerInch) * PixelsInInchY) +
    PrintOffSetY;

  // Avoid negative margins
  if Result < 1 then
    Result := PrintOffSetY;
end;

procedure TfrmPrint_Instrument.StartNewPage;
var
  Print_Y                     : integer;
begin
  Printer.Canvas.Font.Size := StrToIntDef(cbFontSize.Text, 10);
  Printer.Canvas.Font.Style := [];
  SetFont(PrinterFont);

  // Print page numbers at bottom of page
  Print_Y := lvPageHeight - DefaultHeight - // room for Page numbers
    BottomPixelsToPrinterUnits;
  Printer.Canvas.TextOut(Center - (Printer.Canvas.TextWidth('Page XX') div 2),
    Print_Y,
    'Page ' + InttoStr(PageCount));
  Printer.NewPage;
  Y_Print_Total := TopPixelsToPrinterUnits;
  Inc(PageCount);
end;


procedure TfrmPrint_Instrument.Increase_Y_Print(Amount: integer);
begin
  Y_Print_Total := Y_Print_Total + Amount;
end;

procedure TfrmPrint_Instrument.Print_Bold(aLeft: integer; StringIn: string; aAlign: TAlignment);
// Headers and Section labels will use this.
begin

  Printer.Canvas.Font.Size := StrToIntDef(cbFontSize.Text, 10) + 2;
  Printer.Canvas.Font.Style := [fsBold];

  if not IsThereRoom((5 * Printer.Canvas.TextHeight('Yy')) div 4) then
    begin
      StartNewPage;

      // Reset the font
      Printer.Canvas.Font.Size := StrToIntDef(cbFontSize.Text, 10) + 2;
      Printer.Canvas.Font.Style := [fsBold];
    end;

  if aAlign = taCenter then
    Printer.Canvas.TextOut(Center - (Printer.Canvas.TextWidth(StringIn) div 2),
      Y_Print_Total, StringIn);

  if aAlign = taLeftJustify then
    Printer.Canvas.TextOut(aLeft, Y_Print_Total, StringIn);

  if aAlign = taRightJustify then
    Printer.Canvas.TextOut(lvPageWidth - Printer.Canvas.TextWidth(StringIn),
      Y_Print_Total, StringIn);

  Increase_Y_Print((5 * Printer.Canvas.TextHeight('Yy')) div 4);

end;

procedure TfrmPrint_Instrument.Print_Header;

  function FixedLength(StringIn: string): string;
  var
    Fixed,
      ii                      : Integer;
  begin
    ii := Printer.Canvas.TextWidth(StringIn);
    Result := StringIn;
    Fixed := (lvPageWidth div 2) - LeftPixelsToPrinterUnits(0) -
      RightPixelsToPrinterUnits div 2;

    while ii <= Fixed do
      begin
        Result := Result + '_';
        ii := Printer.Canvas.TextWidth(Result);
      end;
  end;

begin
  Printer.Canvas.Font.Size := StrToIntDef(cbFontSize.Text, 10);
  Printer.Canvas.Font.Style := [];
  SetFont(PrinterFont); // Needed to reset size after title changes it.

  Printer.Canvas.TextOut(LeftPixelsToPrinterUnits(0), Y_Print_Total,
    FixedLength('Veteran''s Name: '));
  Printer.Canvas.TextOut(Center, Y_Print_Total,
    FixedLength('Date: '));
  Increase_Y_Print(DefaultHeight);
  Print_Blank_Line(2);

  Printer.Canvas.TextOut(LeftPixelsToPrinterUnits(0), Y_Print_Total,
    FixedLength('Veteran''s DOB: '));
  Printer.Canvas.TextOut(Center, Y_Print_Total,
    FixedLength('Last four digits of SSN: '));
  Increase_Y_Print(DefaultHeight);
  Print_Blank_Line(2);

  Printer.Canvas.TextOut(LeftPixelsToPrinterUnits(0), Y_Print_Total,
    FixedLength('Staff: '));
  Printer.Canvas.TextOut(Center, Y_Print_Total,
    FixedLength('Location: '));
  Increase_Y_Print(DefaultHeight);
  Print_Blank_Line(2);
end;


procedure TfrmPrint_Instrument.Print_A_Straight_Line(Start, Finish: Integer);
var
  i                           : integer;
  TempStr                     : string;
begin
  Print_Blank_Line(2);

  Printer.Canvas.Font.Size := StrToIntDef(cbFontSize.Text, 10);
  Printer.Canvas.Font.Style := [];
  SetFont(PrinterFont); // Needed to reset size after title changes it.

  TempStr := '';
  for i := Start to Finish do
    TempStr := TempStr + '_';

  Printer.Canvas.TextOut(LeftPixelsToPrinterUnits(Start),
    Y_Print_Total, TempStr);
end;


procedure TfrmPrint_Instrument.PrintLineWithCheckBoxWrap(X, Y: Integer; Text: string;
  Display: TDisplay; PrintIt: boolean);
  // Multiple-choice answers
  // This prints a check-box followed by text
var
  FullPage,
    Indent,
    TempY,
    Space                     : integer;

  FontName                    : string;
  FontStyle                   : TFontStyles;

  procedure WrapMultiChoiceLine(LineIn: string; Display: TDisplay);
  var
    i                         : integer;
    TempStr1, TempStr2        : string;
  begin
    TempStr1 := LineIn;
    TempStr2 := '';
    i := Length(TempStr1);

    if Printer.Canvas.TextWidth(TempStr1) <= FullPage then
      begin
        if PrintIt then
          begin
            Printer.Canvas.TextOut(LeftPixelsToPrinterUnits(Indent) + Space,
              Y_Print_Total, TempStr1);
            Increase_Y_Print(DefaultHeight);
          end
        else
          ComponentHeight := ComponentHeight + DefaultHeight;
        exit;
      end;

    // Trim letters until the string fits within the printable space. The wrap
    // within a word is handled next.
    while Printer.Canvas.TextWidth(TempStr1) > FullPage do
      begin
        // Save the trimmed material
        TempStr2 := TempStr1[i] + TempStr2;

        // Remove the last letter
        Delete(TempStr1, i, 1);
        Dec(i);
      end; // while

    // Continue backing up until a space is found
    while TempStr1[i] <> ' ' do
      begin
        // Save the trimmed material
        TempStr2 := TempStr1[i] + TempStr2;
        // Remove the last letter
        Delete(TempStr1, i, 1);
        Dec(i);
      end;

    if PrintIt then
      begin
        Printer.Canvas.TextOut(LeftPixelsToPrinterUnits(Indent) + Space,
          Y_Print_Total, TempStr1);
        Increase_Y_Print(DefaultHeight);
      end
    else
      ComponentHeight := ComponentHeight + DefaultHeight;

    // Repeat
    if Length(TempStr2) > 1 then
      WrapMultiChoiceLine(TempStr2, Display);
  end;

begin
  Indent := X;
  TempY := Y; // This is Y_Print_Total

  FullPage := lvPageWidth - LeftPixelsToPrinterUnits(Display.DisplayLeft) -
    RightPixelsToPrinterUnits;

  // Hold these values
  FontName := Printer.Canvas.Font.Name;
  FontStyle := Printer.Canvas.Font.Style;

  Printer.Canvas.Font.Name := 'WingDings';
  Printer.Canvas.Font.Style := [];

  if PrintIt then
    Printer.Canvas.TextOut(LeftPixelsToPrinterUnits(Indent),
      TempY, Char(111)); // Print small empty box

  Space := Printer.Canvas.TextWidth(Char(111) + Char(127));

  Printer.Canvas.Font.Name := FontName;
  Printer.Canvas.Font.Style := FontStyle;

  // Does this line need to be wrapped?
  if Indent + Space + Printer.Canvas.TextWidth(Text) > FullPage then
    begin
      WrapMultiChoiceLine(Text, Display);
    end
  else
    begin
      if PrintIt then
        begin
          Printer.Canvas.TextOut(LeftPixelsToPrinterUnits(Indent) + Space,
            TempY, Text);
          Increase_Y_Print(DefaultHeight);
        end
      else
        ComponentHeight := ComponentHeight + DefaultHeight;
    end;
end;

procedure TfrmPrint_Instrument.PrintLineWithCheckBoxNoWrap(X, Y: Integer;
  Text: string; PrintIt: boolean);
// Used to print columns of multiple-choice items on same line
// Units passed for X & Y are in printer units.
var
  Indent,
    TempY,
    Space                     : integer;
  FontName                    : string;
  FontStyle                   : TFontStyles;
begin
  Indent := X;
  TempY := Y; // This is Y_Print_Total, not height of this line

  // Hold these values
  FontName := Printer.Canvas.Font.Name;
  FontStyle := Printer.Canvas.Font.Style;

  Printer.Canvas.Font.Name := 'WingDings';
  Printer.Canvas.Font.Style := [];

  if PrintIt then
    Printer.Canvas.TextOut(Indent, TempY, Char(111)); // Print little check box

  Space := Printer.Canvas.TextWidth(Char(111) + Char(127));

  Printer.Canvas.Font.Name := FontName;
  Printer.Canvas.Font.Style := FontStyle;

  if PrintIt then
    Printer.Canvas.TextOut(Indent + Space, TempY, Text);
// Do not increase componentHeight or Y_Print
end;


procedure TfrmPrint_Instrument.Print_Blank_Line(HowMany: integer);
var
  i                           : integer;
begin
  SetFont(PrinterFont);

  for i := 1 to HowMany do
    begin
      Printer.Canvas.TextOut(0, Y_Print_Total, '  ');
      Increase_Y_Print(DefaultHeight div 2);
    end;
end;


procedure TfrmPrint_Instrument.SetFont(FontIn: TFont);
begin
  // Set font for printer field contents
  Printer.Canvas.Font.Assign(FontIn);
end;


function TfrmPrint_Instrument.Print_WrapLine(LongLine: string;
  Display: TDisplay; PrintIt, PageControl: boolean): string;
var
  i,
    FullPage                  : Integer;
  TempStr1,
    TempStr2                  : string;
begin
  Result := '';
  TempStr1 := LongLine;
  TempStr2 := '';
  i := Length(TempStr1);

  FullPage := lvPageWidth - LeftPixelsToPrinterUnits(Display.DisplayLeft) -
    RightPixelsToPrinterUnits;

  if (Printer.Canvas.TextWidth(TempStr1) <= FullPage) then
    begin
      if PrintIt then
        begin
          if Display.DisplayAlignment = taLeftJustify then
            Printer.Canvas.TextOut(LeftPixelsToPrinterUnits(Display.DisplayLeft),
              Y_Print_Total, TempStr1);

          if Display.DisplayAlignment = taRightJustify then
            Printer.Canvas.TextOut(lvPageWidth - Printer.Canvas.TextWidth(TempStr1),
              Y_Print_Total, TempStr1);

          if Display.DisplayAlignment = taCenter then
            Printer.Canvas.TextOut(Center - (Printer.Canvas.TextWidth(TempStr1) div 2),
              Y_Print_Total, TempStr1);

          Increase_Y_Print(DefaultHeight);

          if PageControl then
            if not IsThereRoom(DefaultHeight) then
              StartNewPage;
        end;
      Exit;
    end;

  // Trim letters until the string fits within the printable space. The wrap
  // within a word is handled next.
  while Printer.Canvas.TextWidth(TempStr1) > FullPage do
    begin
      // Save the trimmed material
      TempStr2 := TempStr1[i] + TempStr2;

      // Remove the last letter
      Delete(TempStr1, i, 1);
      Dec(i);
    end; // while

  // Continue backing up until a space is found
  while LongLine[i] <> ' ' do
    begin
      // Save the trimmed material
      TempStr2 := TempStr1[i] + TempStr2;
      // Remove the last letter
      Delete(TempStr1, i, 1);
      Dec(i);
    end;

  if PrintIt then
    begin
      if Display.DisplayAlignment = taLeftJustify then
        Printer.Canvas.TextOut(LeftPixelsToPrinterUnits(Display.DisplayLeft),
          Y_Print_Total, TempStr1);

      if Display.DisplayAlignment = taRightJustify then
        Printer.Canvas.TextOut(lvPageWidth - Printer.Canvas.TextWidth(TempStr1),
          Y_Print_Total, TempStr1);

      if Display.DisplayAlignment = taCenter then
        Printer.Canvas.TextOut(Center - (Printer.Canvas.TextWidth(TempStr1) div 2),
          Y_Print_Total, TempStr1);

      Increase_Y_Print(DefaultHeight);

      if PageControl then
        if not IsThereRoom(DefaultHeight) then
          StartNewPage;
    end;

  // Repeat

  if Length(TempStr2) > 1 then
    begin
      ComponentHeight := ComponentHeight + DefaultHeight;
      Result := TempStr2;
    end;
end;

procedure TfrmPrint_Instrument.Print_StaticText(Display: TDisplay; TextIn: string);
// Static Text is boilerplate that is added to the instrument. It may be a short
// sentence, a paragraph, or several long paragraphs.
// This routine will look for carriage returns which separate paragraphs and
// wrap strings to fit within the margins.
// The Global Deterioration Scale (GDS) has a page or more of static text.
var
  i, Count, FullPage          : Integer;
  TempStr, TempStr1, TempStr2 : string;


begin
  TempStr := '';

  Print_Blank_Line(2); // 1 is (DefaultHeight div 2)

  Printer.Canvas.Font.Size := StrToIntDef(cbFontSize.Text, 10);
  Printer.Canvas.Font.Style := [];
  SetFont(PrinterFont); // Needed to reset size after title changes it.

  FullPage := lvPageWidth - LeftPixelsToPrinterUnits(Display.DisplayLeft) -
    RightPixelsToPrinterUnits;

  // Static Text may contain carriage returns and tabs.
  Count := NumberofPieces(TextIn, #13);

  if Count > 0 then
    begin
      TempStr := TextIn;
      for i := 1 to Count do
        begin
          TempStr1 := Piece(TempStr, #13, i);

          TempStr2 := Print_WrapLine(TempStr1, Display, True, True {Page Control});

          while Tempstr2 <> '' do
            TempStr2 := Print_WrapLine(TempStr2, Display, True, True {Page Control});
        end;
      Exit;
    end;

  // Long string: Check for the need to wrap
  if Printer.Canvas.TextWidth(TextIn) > FullPage then
    begin
      // ComponentHeight will be added to by this call
      TempStr := Print_WrapLine(TextIn, Display, True, True {Page Control});

      while Tempstr <> '' do
        TempStr := Print_WrapLine(TempStr, Display, True, True {Page Control});

      Exit;
    end;


  // Short String
  if Display.DisplayAlignment = taLeftJustify then
    Printer.Canvas.TextOut(LeftPixelsToPrinterUnits(Display.DisplayLeft), Y_Print_Total, TextIn);

  if Display.DisplayAlignment = taRightJustify then
    Printer.Canvas.TextOut(lvPageWidth - Printer.Canvas.TextWidth(TextIn),
      Y_Print_Total, TextIn);

  if Display.DisplayAlignment = taCenter then
    Printer.Canvas.TextOut(Center - (Printer.Canvas.TextWidth(TextIn) div 2),
      Y_Print_Total, TextIn);

  Increase_Y_Print(DefaultHeight);

  if not IsThereRoom(DefaultHeight) then
    StartNewPage;
end;


procedure TfrmPrint_Instrument.Print_String(Display: TDisplay; TextIn: string;
  PrintIt, PageControl: boolean);
// When PrintIt is false, we are checking the overall size of the component so
// that radio groups, for example, are not split between two pages.
var
  FullPage                    : Integer;
  TempStr                     : string;
begin
  TempStr := '';

  if PrintIt then
    Print_Blank_Line(2) // 1 is (DefaultHeight div 2)
  else
    ComponentHeight := ComponentHeight + DefaultHeight;

  Printer.Canvas.Font.Size := StrToIntDef(cbFontSize.Text, 10);
  Printer.Canvas.Font.Style := [];
  SetFont(PrinterFont); // Needed to reset size after title changes it.

  FullPage := lvPageWidth - LeftPixelsToPrinterUnits(Display.DisplayLeft) -
    RightPixelsToPrinterUnits;

  // Check for the need to wrap
  if Printer.Canvas.TextWidth(TextIn) > FullPage then
    begin
      // ComponentHeight will be added to by this call
      TempStr := Print_WrapLine(TextIn, Display, PrintIt, PageControl);

      while Tempstr <> '' do
        TempStr := Print_WrapLine(TempStr, Display, PrintIt, PageControl);
      Exit;
    end;

  if PrintIt then
    begin
      if Display.DisplayAlignment = taLeftJustify then
        Printer.Canvas.TextOut(LeftPixelsToPrinterUnits(Display.DisplayLeft), Y_Print_Total, TextIn);

      if Display.DisplayAlignment = taRightJustify then
        Printer.Canvas.TextOut(lvPageWidth - Printer.Canvas.TextWidth(TextIn),
          Y_Print_Total, TextIn);

      if Display.DisplayAlignment = taCenter then
        Printer.Canvas.TextOut(Center - (Printer.Canvas.TextWidth(TextIn) div 2),
          Y_Print_Total, TextIn);

      Increase_Y_Print(DefaultHeight);

      if PageControl then
        if not IsThereRoom(DefaultHeight) then
          StartNewPage;
    end
  else
    ComponentHeight := ComponentHeight + DefaultHeight;
end;


procedure TfrmPrint_Instrument.Print_MultipleChoice(Display,
  AnswerDisplay: TDisplay; List: TStringList; PrintIt: boolean);
var
  ColWidth,
    r, c,
    Indent,
    Indent1,
    i                         : integer;

begin

  Printer.Canvas.Font.Size := StrToIntDef(cbFontSize.Text, 10);
  Printer.Canvas.Font.Style := [];
  SetFont(PrinterFont); // Needed to reset size after title changes it.

  Indent := Display.DisplayLeft + AnswerDisplay.DisplayLeft;

  if Indent = 0 then
    Indent := 20;

  if AnswerDisplay.DisplayColumns <= 1 then
    begin
      for i := 0 to List.Count - 1 do
        begin
          // Wrap if needed.
          PrintLineWithCheckBoxWrap(Indent, Y_Print_Total, List[i], Display, PrintIt);
        end;
      exit;
    end;

  // i is number of rows: add a row when needed
  i := List.Count div AnswerDisplay.DisplayColumns;
  if List.Count mod AnswerDisplay.DisplayColumns > 0 then
    Inc(i);

  Indent := LeftPixelsToPrinterUnits(Indent);
  ColWidth := (lvPageWidth - Indent -
    (RightPixelsToPrinterUnits div 2)) div AnswerDisplay.DisplayColumns;

  for r := 0 to i - 1 do
    begin
      for c := 0 to AnswerDisplay.DisplayColumns - 1 do
        begin
          if r + (c * i) < List.Count then
            begin
              if c = 0 then
                Indent1 := Indent
              else
                Indent1 := Indent + (c * ColWidth);
              // No wrap
              PrintLineWithCheckBoxNoWrap(Indent1, Y_Print_Total, List[r + (c * i)],
                PrintIt);
            end;
        end; // for c

      if PrintIt then
        Increase_Y_Print(DefaultHeight)
      else
        ComponentHeight := ComponentHeight + DefaultHeight;
    end; // for r
end;


procedure TfrmPrint_Instrument.Print_TrackBar_Scale(Display, AnswerDisplay: TDisplay;
  aMin, aMax: Integer; PrintIt: boolean);
var
  i,
    vWidth                    : integer;
  Space,
    Indent,
    TempStr                   : string;
begin
  if PrintIt then
  else
    begin
      ComponentHeight := ComponentHeight + DefaultHeight;
      exit;
    end;

  Printer.Canvas.Font.Size := StrToIntDef(cbFontSize.Text, 10);
  Printer.Canvas.Font.Style := [];
  SetFont(PrinterFont); // Needed to reset size after title changes it.

  vWidth := (lvPageWidth -
    LeftPixelsToPrinterUnits(Display.DisplayLeft + (2 * AnswerDisplay.DisplayLeft)) -
    (RightPixelsToPrinterUnits));

  Indent := '';
  for i := 1 to AnswerDisplay.DisplayLeft div 2 do
    Indent := Indent + ' ';

  while Printer.Canvas.TextWidth(Indent + TempStr) <= vWidth do
    begin
      Space := Space + ' ';
      TempStr := '';
      for i := aMin to aMax - 1 do
        TempStr := TempStr + IntToStr(i) + Space;
      TempStr := TempStr + Inttostr(aMax);
    end;

  TempStr := Indent + TempStr;
  Print_String(Display, TempStr, TRUE, FALSE {Page Control});
end;


procedure TfrmPrint_Instrument.Print_TrackBar_Labels(Display, AnswerDisplay: TDisplay;
  List: TStringList; PrintIt: boolean);
var
  i,
    vWidth                    : Integer;
  Indent,
    TempStr1,
    TempStr                   : string;
begin
  if (List.Count <= 1) or (List.Count > 3) then
    Exit;

  Printer.Canvas.Font.Size := StrToIntDef(cbFontSize.Text, 10);
  Printer.Canvas.Font.Style := [];
  SetFont(PrinterFont); // Needed to reset size after title changes it.

  vWidth := (lvPageWidth -
    LeftPixelsToPrinterUnits(Display.DisplayLeft + (2 * AnswerDisplay.DisplayLeft)) -
    RightPixelsToPrinterUnits);

  for i := 1 to AnswerDisplay.DisplayLeft div 2 do
    Indent := Indent + ' ';

  TempStr1 := '';

  if List.Count = 2 then
    begin
      TempStr := List[0] + ' ' + List[1];
      while Printer.Canvas.TextWidth(TempStr) <= vWidth do
        begin
          TempStr1 := TempStr1 + ' ';
          TempStr := Indent + List[0] + TempStr1 + List[1];
        end;
      TempStr := Indent + List[0] + TempStr1 + '     ' + List[1];
    end;

  if List.Count = 3 then
    begin
      TempStr := Indent + List[0] + ' ' + List[1] + ' ' + List[2];
      while Printer.Canvas.TextWidth(TempStr) <= vWidth do
        begin
          TempStr1 := TempStr1 + ' ';
          TempStr := Indent + List[0] + TempStr1 + List[1] + TempStr1 + List[2];
        end;
      TempStr := Indent + List[0] + TempStr1 + List[1] + TempStr1 + '     ' + List[2];
    end;

  if PrintIt then
    Print_String(Display, TempStr, TRUE, TRUE {Page Control})
  else
    ComponentHeight := ComponentHeight + DefaultHeight;
end;


procedure TfrmPrint_Instrument.cbFontSizeChange(Sender: TObject);
begin
  PrinterFont.Size := StrToIntDef(cbFontSize.Text, 10);
  SetFont(PrinterFont);
  DefaultHeight := (5 * Printer.Canvas.TextHeight('Yy')) div 4;

   // Larger fonts need a little adjustment.
  if (cbFontSize.Text = '12') then
    DefaultHeight := (3 * Printer.Canvas.TextHeight('Yy')) div 2;

  if (cbFontSize.Text = '14') then
    DefaultHeight := (7 * Printer.Canvas.TextHeight('Yy')) div 4;
end;

procedure TfrmPrint_Instrument.btnSetPrinterClick(Sender: TObject);
begin
  with PageSetupDialog do
    begin
      MHA_Registry.ReadRegistry;

      MinMarginLeft := PrintOffSetX;
      MinMarginRight := PrintOffSetX;
      MinMarginTop := PrintOffSetY;
      MinMarginBottom := PrintOffSetY;

      // Convert pixels to thousands of an inch. 96 pxels per inch
      MarginLeft := Round((gvLeftMargin / PixelsPerInch) * 1000);
      MarginRight := Round((gvRightMargin / PixelsPerInch) * 1000);
      MarginTop := Round((gvTopMargin / PixelsPerInch) * 1000);
      MarginBottom := Round((gvBottomMargin / PixelsPerInch) * 1000);

      if not Execute then
        Exit;

      // MarginLeft is thousands of inch, convert to pixels
      // (1000 is one inch, 1250 is 1.25 inch, etc.)
      gvLeftMargin := Round(MarginLeft / 1000 * PixelsPerInch);
      gvRightMargin := Round(MarginRight / 1000 * PixelsPerInch);
      gvTopMargin := Round(MarginTop / 1000 * PixelsPerInch);
      gvBottomMargin := Round(MarginBottom / 1000 * PixelsPerInch);
      MHA_Registry.SaveRegistry;

      lvPageWidth := PageWidth;
      lvPageHeight := PageHeight;

      PrintOffSetX := GetDeviceCaps(Printer.Handle, PHYSICALOFFSETX);
      PrintOffSetY := GetDeviceCaps(Printer.Handle, PHYSICALOFFSETY);
      lvPageHeight := GetDeviceCaps(Printer.Handle, PHYSICALHEIGHT);
      lvPageWidth := GetDeviceCaps(Printer.Handle, PHYSICALWIDTH);
      PixelsInInchX := GetDeviceCaps(Printer.Handle, LOGPIXELSX);
      PixelsInInchY := GetDeviceCaps(Printer.Handle, LOGPIXELSY);
    end;
end;

procedure TfrmPrint_Instrument.cbPrintFontChange(Sender: TObject);
begin
  PrinterFont.Name := cbPrintFont.FontName;
  DefaultHeight := (5 * Printer.Canvas.TextHeight('Yy')) div 4;
  SetFont(PrinterFont);
end;

procedure TfrmPrint_Instrument.FormResize(Sender: TObject);
var
  Space                       : Integer;
begin
  Space := lblInstrument.Width div 8;

  lblInstructions.Left := (ClientWidth - lblInstructions.Width) div 2;
  lblInstrument.Left := 2 * Space;
  cbInstruments.Left := 11 * Space;

  cbPrintFont.Left := cbInstruments.Left;

  lblPrintFont.Width := lblInstrument.Width;
  lblPrintFont.Left := lblInstrument.Left;

  cbInstruments.Width := ClientWidth - cbInstruments.Left - (4 * Space);
  cbPrintFont.Width := cbInstruments.Width;

  cbFontSize.Width := cbPrintFont.Width div 4;
  cbFontSize.Left := ClientWidth - cbFontSize.Width - (4 * Space);

  lblFontSize.Left := cbFontSize.Left - lblFontSize.Width - Space;

  lblRecommend.Left := (ClientWidth - lblRecommend.Width) div 2;

  btnCancel.Left := btnCancel.Width div 3;
  btnPrint.Left := pnlButtons.Width - btnPrint.Width - btnCancel.Left;
  btnSetPrinter.Left := (pnlButtons.Width - btnSetPrinter.Width) div 2;

  if lvInitialize then
    Exit;

  // Reset these, used in placement of dialogs
  gvLeft := Left;
  gvTop := Top;
  gvHeight := Height;
  gvWidth := Width;
end;

procedure TfrmPrint_Instrument.FormKeyDown(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
  if Key = vk_Escape then
    begin
      Close;
      Key := 0;
    end;
end;

end.

