unit fMHA_SelectPatient;
// Off-line patient selection form

interface

uses
  Windows,
  Messages,
  SysUtils,
  System.Contnrs,
  Classes,
  Graphics,
  Controls,
  Forms,
  StdCtrls,
  ExtCtrls,
  ORCtrls,
  ComCtrls,
  Mask,
  RzLabel,
  RzPanel,
  Fmcntrls,
  JvExControls,
  JvComponent,
  JvXPCore,
  JvXPButtons;

type
  TfrmMHA_SelectPatient = class(TForm)
    pnlButton: TPanel;
    pnlSelect: TPanel;
    pnlHeader: TRzPanel;
    lblHeader: TRzLabel;
    pnlHeaderSpacer: TPanel;
    pnlDetails: TPanel;
    grpbxPatient: TGroupBox;
    lblFName: TLabel;
    lblLName: TLabel;
    lblSSN: TLabel;
    lblDOB: TLabel;
    edFName: TEdit;
    edLName: TEdit;
    edSSN: TMaskEdit;
    dtpkrDOB: TDateTimePicker;
    grpSelect: TGroupBox;
    cbxPtName: TComboBox;
    pnlSex: TPanel;
    rbMale: TRadioButton;
    rbFemale: TRadioButton;
    lblSex: TLabel;
    btnNew: TJvXPButton;
    btnDelete: TJvXPButton;
    btnOK: TJvXPButton;
    btnCancel: TJvXPButton;

    procedure FormCreate(Sender: TObject);
    procedure FormShow(Sender: TObject);

    procedure ConfigureButtons;
    procedure ConfigureForm;
    procedure btnOKClick(Sender: TObject);
    //procedure FormCloseQuery(Sender: TObject; var CanClose: Boolean);
    procedure cbxPtNameClick(Sender: TObject);
    procedure btnCancelClick(Sender: TObject);
    procedure btnNewClick(Sender: TObject);
    procedure btnDeleteClick(Sender: TObject);
    procedure edFNameChange(Sender: TObject);
    procedure edLNameExit(Sender: TObject);

    procedure FormKeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure edSSNExit(Sender: TObject);
    procedure FormResize(Sender: TObject);
    procedure FormKeyPress(Sender: TObject; var Key: Char);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);

  private
    FCloseOut: Boolean;
    procedure DisplayPatientProperties;
    function BuildName: string;
    function NameExists(aName: string): Boolean;
    function SSNExists(aSSN: string): Boolean;
    procedure DeletePatientAndFiles;
    procedure PopulatePtCombo;
    procedure EnableButtons;
    { Private declarations }
  public
    { Public declarations }
    property CloseOut: Boolean read FCloseOut write FCloseOut;
  end;

var
  frmMHA_SelectPatient        : TfrmMHA_SelectPatient;

implementation

uses
  uIA_Const,
  uMHA_Const,
  uMHA_Registry,
  uIA_GUIControl,
  fIA_dialogs, Dialogs,
  uIA_Utils,
  uMHA_OffLine_OPS,
  uMHA_LocalFiles,
  uIA_Patient,
  fMHA_Main,
  fMHA_InstrumentSelect;

{$R *.DFM}

var
  lvCurrentPt                 : Integer;
  lvInitialize                : Boolean;

procedure TfrmMHA_SelectPatient.FormCreate(Sender: TObject);
begin
  lvInitialize := True;
  // Keep Position out of ConfigureForm when ConfigureForm can be called
  // more than once with the form.
  Position := poDesigned;

  FCloseOut := True;
  MHA_Registry.ReadRegistry;
  ConfigureForm;
  ConfigureButtons;

  frmMHA_Main.Hide;
end;

procedure TfrmMHA_SelectPatient.FormShow(Sender: TObject);
begin

  OffLine_OPS.LoadOffLinePatients;
  lvCurrentPt := -1;

  PopulatePtCombo;

  if gvUseScreenReader then
    begin
      Caption := 'Select or create a patient for off-line administrations.';
      grpbxPatient.Caption := '';
      ActiveControl := cbxPtName;
    end;

  if cbxPtName.Items.Count = 0 then
    btnNewClick(sender)
  else
    cbxPtNameClick(Sender); // sets lvCurrentPt

  lvInitialize := False;
  Screen.Cursor := crDefault;

  if gvUseScreenReader then
    ActiveControl := grpSelect
  else
    ActiveControl := cbxPtName;
end;

procedure TfrmMHA_SelectPatient.PopulatePtCombo;
var
  i                           : integer;
begin
  cbxPtName.Items.Clear;
  cbxPtName.Text := '';

  if OffLine_OPS.ListOfOffLinePatients.Count = 0 then
    Exit;

  for i := 0 to OffLine_OPS.ListOfOffLinePatients.Count - 1 do
    cbxPtName.Items.Add(TPatient(OffLine_OPS.ListOfOffLinePatients[i]).Name);

  lvCurrentPt := 0;
  DisplayPatientProperties;
end;

procedure TfrmMHA_SelectPatient.ConfigureButtons;
begin
  pnlButton.Height := 3 * GUI_Control.ButtonHeight div 2;

  btnNew.Height := GUI_Control.ButtonHeight;
  btnDelete.Height := btnNew.Height;
  btnCancel.Height := btnNew.Height;
  btnOK.Height := btnNew.Height;

  btnNew.Width := GUI_Control.ButtonWidthNormal;
  btnDelete.Width := btnNew.Width;
  btnCancel.Width := btnNew.Width;
  btnOK.Width := btnNew.Width;

  btnNew.Top := (pnlButton.Height - btnNew.Height) div 2;
  btnDelete.Top := btnNew.Top;
  btnCancel.Top := btnNew.Top;
  btnOK.Top := btnNew.Top;

  btnNew.Left := (pnlButton.Width - (4 * btnNew.Width)) div 5;
  btnDelete.Left := (2 * btnNew.Left) + btnNew.Width;
  btnCancel.Left := (3 * btnNew.Left) + (2 * btnNew.Width);
  btnOK.Left := (4 * btnNew.Left) + (3 * btnNew.Width);

  {if gvUseScreenReader then
    begin
      btnOK.caption := btnOK.Caption + ' button';
      btnCancel.caption := btnCancel.Caption + ' button';
      btnDelete.caption := btnDelete.Caption + ' button';
      btnNew.caption := btnNew.Caption + ' button';
    end;
  }
end;

procedure TfrmMHA_SelectPatient.ConfigureForm;
var
  space                       : integer;
begin
  with GUI_Control do
    begin
      Scaled := True;
      AutoScroll := False;
      AutoSize := False;

      PixelsPerInch := 120;
      KeyPreview := True;
      HorzScrollBar.Range := 0; // Ensures that buttons will move when re-sized
      VertScrollBar.Range := 0;
      Font.Assign(gvFont);

      Height := GetHeight(16 * ButtonHeight);
      Width := GetWidth(5 * ButtonWidthNormal);
      Left := GetLeft(Width);
      Top := GetTop(Height);
      if CheckWidth(Width) or CheckHeight(Height) then
        WindowState := wsMaximized;
      Refresh; // resets alClient dimensions

      Caption := MHA_Caption;

      space := TextHeight('Yy', gvFont);
      pnlHeader.Height := pnlHeaderHeight;
      lblHeader.Font.Size := lblHeaderFontSize;

      pnlSelect.Height := 2 * ButtonHeight;
      pnlSelect.Font.Assign(gvFont);
      pnlDetails.Font.Assign(gvFont);

      cbxPtName.Top := (2 * (pnlSelect.Height - cbxPtName.Height)) div 3;

      rbMale.Height := TextHeight('Yy', gvFont) + 2;
      rbFemale.Height := TextHeight('Yy', gvFont) + 2;

      lblFName.Top := ButtonHeight + space;
      edFName.Top := lblFName.Top;
      lblLName.Top := lblFName.Top + lblFName.Height + space;
      edLName.Top := lblLName.Top;
      lblSSN.Top := lblLName.Top + lblLName.Height + space;
      edSSN.Top := lblSSN.Top;
      lblDOB.Top := lblSSN.Top + lblSSN.Height + space;
      dtpkrDOB.Top := lblDOB.Top;

      pnlSex.Top := lblDOB.Top + lblDOB.Height + space;
      rbMale.Top := 5;
      rbFemale.Top := 5;
      lblSex.Top := 5;

      //lblDFN.Left := 2 * Width div 3;
      //txtDFN.Left := lblDFN.Left + lblDFN.Width + space;
      pnlSex.Left := 0;

      edFName.Width := TextWidth('XXXXXXXXXXXXXXX',
        gvFont);
      edLName.Width := edFName.Width;
      edSSN.Width := edFName.Width;
      dtpkrDOB.Width := edFName.Width;
      pnlSex.Width := grpbxPatient.Width;
      rbMale.Width := TextWidth(rbFemale.caption + 'XXX',
        gvFont);
      rbFemale.Width := rbMale.Width;
      cbxPtName.Width := 3 * edFName.Width div 2;

      edFName.Left := Width div 2;
      lblFName.Left := edFName.Left - TextWidth(lblFName.Caption + 'XX',
        pnlSelect.Font);
      edLName.Left := edFName.Left;
      lblLName.Left := edFName.Left - TextWidth(lblLName.Caption + 'XX',
        pnlSelect.Font);
      edSSN.Left := edFName.Left;
      lblSSN.Left := edFName.Left - TextWidth(lblSSN.Caption + 'XX',
        pnlSelect.Font);
      dtpkrDOB.Left := edFName.Left;
      lblDOB.Left := edFName.Left - TextWidth(lblDOB.Caption + 'XX',
        pnlSelect.Font);
      rbMale.Left := edFName.Left;
      rbFemale.Left := rbMale.Left + rbMale.Width + 10;
      lblSex.Left := edFName.Left - TextWidth(lblSex.Caption + 'XX',
        pnlSelect.Font);

      edFName.Height := 3 * TextHeight('Yy', pnlSelect.Font) div 2;
      lblFName.Height := edFName.Height;
      edLName.Height := edFName.Height;
      lblLName.Height := edFName.Height;
      edSSN.Height := edFName.Height;
      lblSSN.Height := edFName.Height;
      dtpkrDOB.Height := edFName.Height;
      lblDOB.Height := edFName.Height;
      pnlSex.Height := edFName.Height + 5;
      rbMale.Height := edFName.Height;
      rbFemale.Height := edFName.Height;
      lblSex.Height := edFName.Height;
    end;
end;

procedure TfrmMHA_SelectPatient.DisplayPatientProperties;
begin
  if lvCurrentPt = -1 then
    begin
      cbxPtName.Text := '';
      edFName.Text := '';
      edLName.Text := '';
      edSSN.Text := '';
      dtpkrDOB.Date := StrToDate('01/01/1900');
      rbMale.Checked := False;
      rbFemale.Checked := False;
    end
  else
    begin
      cbxPtName.Text := TPatient(OffLine_OPS.ListOfOffLinePatients[lvCurrentPt]).Name;
      edFName.Text := Trim(Piece(TPatient(OffLine_OPS.ListOfOffLinePatients[lvCurrentPt]).Name, ',', 2));
      edLName.Text := Trim(Piece(TPatient(OffLine_OPS.ListOfOffLinePatients[lvCurrentPt]).Name, ',', 1));
      edSSN.Text := TPatient(OffLine_OPS.ListOfOffLinePatients[lvCurrentPt]).SSN;
      dtpkrDOB.Date := StrToDate(TPatient(OffLine_OPS.ListOfOffLinePatients[lvCurrentPt]).DOB);

      if (TPatient(OffLine_OPS.ListOfOffLinePatients[lvCurrentPt]).Sex = 'M') then
        rbMale.Checked := True;
      if (TPatient(OffLine_OPS.ListOfOffLinePatients[lvCurrentPt]).Sex = 'F') then
        rbFemale.Checked := True;
    end;
end;

procedure TfrmMHA_SelectPatient.btnOKClick(Sender: TObject);
var
  TempSSN                     : string;
begin

  if edLName.Text = '' then
    begin
      MsgDlgFrm('Please provide a last name before continuing.',
        MHA_Caption, mtInformation, [mbOK], 0);
      Exit;
    end;

  if edFName.Text = '' then
    begin
      MsgDlgFrm('Please provide a first name before continuing.',
        MHA_Caption, mtInformation, [mbOK], 0);
      Exit;
    end;

  if edSSN.Text = '' then
    begin
      if MsgDlgFrm('The SSN is blank. If at all possible, please provide it as this ' +
        'helps to identify this patient in Vista when these results are uploaded later.' + #13 +
        'Do you wish to continue anyway?',
        MHA_Caption, mtConfirmation, [mbYes, mbNo], 0) =
        mrNo then
        Exit;
    end;

  if dtpkrDOB.Date = StrToDate('01/01/1900') then
    begin
      if MsgDlgFrm('The Date of Birth is blank. If at all possible, please provide it as this ' +
        'helps to identify this patient in Vista when these results are uploaded later.' + #13 +
        'Do you wish to continue anyway?',
        MHA_Caption, mtConfirmation, [mbYes, mbNo], 0) =
        mrNo then
        Exit;
    end;

  if (not (rbMale.Checked) and (not rbFemale.Checked)) then
    begin
      if MsgDlgFrm('The patient''s gender is not checked. If at all possible, please check one ' +
        'of these as it helps to identify this patient in Vista when these results are uploaded later.' + #13 +
        'Do you wish to continue anyway?',
        MHA_Caption, mtConfirmation, [mbYes, mbNo], 0) =
        mrNo then
        Exit;
    end;

  TempSSN := edSSN.Text;
  if Length(TempSSN) > 6 then
    begin
      Insert('-', TempSSN, 4);
      Insert('-', TempSSN, 7);
    end;

  with TPatient(OffLine_OPS.ListOfOffLinePatients[lvCurrentPt]) do
    begin
      Name := BuildName;
      SSN := edSSN.Text;
      FormattedSSN := TempSSN;
      if length(FormattedSSN) = 11 then
        FormattedSSN := 'xxx-xx-' + Copy(FormattedSSN, 8, 4)
      else
        FormattedSSN := 'xxx-xx-xxxx';
      DOB := DateToStr(dtpkrDOB.Date);
      Sex := 'U';
      if rbMale.Checked then
        Sex := 'M';
      if rbFemale.Checked then
        Sex := 'F';
    end;

   // Patient: This is used by InstrumentSelect and, later, the administration of the
   // instrument.
  with Patient do
    begin
      Name := BuildName;
      SSN := edSSN.Text;
      FormattedSSN := TempSSN;
      if length(FormattedSSN) = 11 then
        FormattedSSN := 'xxx-xx-' + Copy(FormattedSSN, 8, 4)
      else
        FormattedSSN := 'xxx-xx-xxxx';
      DOB := DateToStr(dtpkrDOB.Date);
      Sex := 'U';
      if rbMale.Checked then
        Sex := 'M';
      if rbFemale.Checked then
        Sex := 'F';
      DFN := TPatient(OffLine_OPS.ListOfOffLinePatients[lvCurrentPt]).DFN;
      SetCaptionInfo;
    end;

  OffLine_OPS.SaveOffLinePatients;

  FCloseOut := False;

  try
    frmMHA_InstrumentSelect := TfrmMHA_InstrumentSelect.Create(nil);
    frmMHA_InstrumentSelect.ShowModal;
  finally
    frmMHA_InstrumentSelect.Free;
  end; //try

  // This updates the list of patients, if a new one was added, and gives a
  // click to the NextOffLineDFN.
  OffLine_OPS.LoadOffLinePatients;
  PopulatePtCombo;
  btnNew.Enabled := True;
  Show;
end;

procedure TfrmMHA_SelectPatient.EnableButtons;
begin
  btnOK.Enabled := not (edFName.Text = '')
    and not (edLName.Text = '');
    //and not (edSSN.Text = '')
    //and ((rbMale.Checked) or (rbFemale.Checked))
    //and not (dtpkrDOB.Date = StrToDate('01/01/1900'));

  btnDelete.Enabled := not (edFName.Text = '')
    and not (edLName.Text = '');
end;

(*
procedure TfrmMHA_SelectPatient.FormCloseQuery(Sender: TObject;
  var CanClose: Boolean);
begin

  if not Patient.IsSelected then
    if MsgDlgFrm('No patient is selected.  Are you sure you want to close?',
      MHA_Caption, mtConfirmation, [mbYes, mbNo], 0) =
      mrYes then
      begin
        CanClose := True;
        halt;
      end
    else
      CanClose := false;

end;*)

procedure TfrmMHA_SelectPatient.cbxPtNameClick(Sender: TObject);
var
  i                           : Integer;
begin
// The combo index is not the same as lvCurrentPt.
  if OffLine_OPS.ListOfOffLinePatients.Count = 0 then
    lvCurrentPt := -1
  else
    begin
      for i := 0 to OffLine_OPS.ListOfOffLinePatients.Count - 1 do
        if cbxPtName.Text = TPatient(OffLine_OPS.ListOfOffLinePatients[i]).Name then
          begin
            lvCurrentPt := i;
            break;
          end;
    end;

  DisplayPatientProperties;
end;

procedure TfrmMHA_SelectPatient.btnCancelClick(Sender: TObject);
begin
  Patient.Clear;
  DisplayPatientProperties;
  Close;
end;

procedure TfrmMHA_SelectPatient.btnNewClick(Sender: TObject);
var
  aPatient                    : TPatient;
begin
  aPatient := TPatient.Create;
  aPatient.Clear;

  with aPatient do
    begin
      Name := '';
      SSN := '';
      DOB := '01/01/1900';
      Sex := 'M';
      DFN := IntToStr(OffLine_OPS.NextOffLineDFN);
    end;

  OffLine_OPS.ListOfOffLinePatients.Add(aPatient);
  lvCurrentPt := OffLine_OPS.ListOfOffLinePatients.Count - 1;

  DisplayPatientProperties;
  btnNew.Enabled := False;
  edFName.SetFocus;
end;

function TfrmMHA_SelectPatient.BuildName: string;
begin
  Result := Trim(edLName.Text) + ', ' + Trim(edFName.Text);
end;

procedure TfrmMHA_SelectPatient.btnDeleteClick(Sender: TObject);
var
  tmpName, ShowName           : string;
begin
  tmpName := BuildName;
  ShowName := Trim(Piece(tmpName, ',', 2)) + ' ' + Trim(Piece(tmpName, ',', 1));
  ShowName := MixedCase(ShowName);
  if MsgDlgFrm('Are you sure you want to delete ' + ShowName + ' from '
    + 'the off-line patient file?' + CRLF + CRLF +
    'This action cannot be undone!',
    MHA_Caption, mtWarning, [mbYes, mbNo], 0) = mrNo then
    exit;

  DeletePatientAndFiles;

end;

procedure TfrmMHA_SelectPatient.DeletePatientAndFiles;
begin
  // Remove *.adm, *.ans for this off-line patient
  LocalFiles.RemoveOffLineFilesForPt
    (TPatient(OffLine_OPS.ListOfOffLinePatients[lvCurrentPt]).DFN);

  // Remove patient from list of off-line patients
  OffLine_OPS.ListOfOffLinePatients.Delete(lvCurrentPt);

  // Save the shortened list
  OffLine_OPS.SaveOffLinePatients;

  // Remove name from combo box
  PopulatePtCombo; // This will also display the first Pt when done.
end;

procedure TfrmMHA_SelectPatient.edFNameChange(Sender: TObject);
begin
  EnableButtons;
end;

procedure TfrmMHA_SelectPatient.edLNameExit(Sender: TObject);
begin
  if NameExists(BuildName) then
    MsgDlgFrm('This name already has off-line administrations of ' +
      'instruments associated with him/her. Look in ' +
      'the combo box, "' + trim(grpSelect.Caption) + '" to confirm this.',
      MHA_Caption, mtInformation, [mbOk], 0);
end;

function TfrmMHA_SelectPatient.NameExists(aName: string): Boolean;
// Scan all the off-line patients, skip the current one.
var
  i                           : Integer;
begin
  result := False;

  if OffLine_OPS.ListOfOffLinePatients.Count < 1 then
    Exit;

  if lvCurrentPt > 1 then
    for i := 0 to lvCurrentPt - 1 do
      if aName = TPatient(OffLine_OPS.ListOfOffLinePatients[i]).Name then
        begin
          result := True;
          Exit;
        end;

  if lvCurrentPt + 1 < OffLine_OPS.ListOfOffLinePatients.Count - 1 then
    for i := lvCurrentPt + 1 to OffLine_OPS.ListOfOffLinePatients.Count - 1 do
      if aName = TPatient(OffLine_OPS.ListOfOffLinePatients[i]).Name then
        result := True;

end;


procedure TfrmMHA_SelectPatient.edSSNExit(Sender: TObject);
begin
  if SSNExists(edSSN.text) then
    ActiveControl := cbxPtName;
end;

function TfrmMHA_SelectPatient.SSNExists(aSSN: string): Boolean;
// Scan all the off-line patients, skip the current one.
var
  i                           : Integer;
begin
  result := False;

  // The two dashes are ignored
  if Length(aSSN) < 9 then
    Exit;

  if OffLine_OPS.ListOfOffLinePatients.Count < 1 then
    Exit;

  if lvCurrentPt > 1 then
    for i := 0 to lvCurrentPt - 1 do
      if aSSN = TPatient(OffLine_OPS.ListOfOffLinePatients[i]).SSN then
        begin
          result := True;
          MsgDlgFrm('This Social Security Number is in use by another patient, ' +
            TPatient(OffLine_OPS.ListOfOffLinePatients[i]).Name +
            '. This name will appear in ' +
            'the combo box, "' + trim(grpSelect.Caption) + '."', MHA_Caption, mtInformation,
            [mbOk], 0);
          Exit;
        end;

  if lvCurrentPt + 1 < OffLine_OPS.ListOfOffLinePatients.Count - 1 then
    for i := lvCurrentPt + 1 to OffLine_OPS.ListOfOffLinePatients.Count - 1 do
      if aSSN = TPatient(OffLine_OPS.ListOfOffLinePatients[i]).SSN then
        begin
          result := True;
          MsgDlgFrm('This Social Security Number is in use by another patient, ' +
            TPatient(OffLine_OPS.ListOfOffLinePatients[i]).Name +
            '. This name will appear in ' +
            'the combo box, "' + trim(grpSelect.Caption) + '."', MHA_Caption, mtInformation,
            [mbOk], 0);
        end;
end;

procedure TfrmMHA_SelectPatient.FormKeyDown(Sender: TObject;
  var Key: Word; Shift: TShiftState);
begin
  if (Shift = [ssALT]) and (Key = vk_F4) then
    Close;

  if Key = vk_Escape then
    begin
      Close;
      Key := 0;
    end;
end;

procedure TfrmMHA_SelectPatient.FormKeyPress(Sender: TObject;
  var Key: Char);
begin
    // Note: use this in KeyPress to avoid a beep
  if Key = #13 then {- Go to next field -}
    begin
      if ActiveControl.ClassName = 'TMemo' then
        Exit;
      Perform(WM_NEXTDLGCTL, 0, 0);
      Key := #0; {- requires "Message" in "uses" -}
    end;
end;


procedure TfrmMHA_SelectPatient.FormResize(Sender: TObject);
begin
  if lvInitialize then
    Exit;

  // Reset these, used in placement of dialogs
  gvLeft := Left;
  gvTop := Top;
  gvHeight := Height;
  gvWidth := Width;
end;

procedure TfrmMHA_SelectPatient.FormClose(Sender: TObject;
  var Action: TCloseAction);
begin
  Action := caFree;
end;

end.

