unit uMHA_Administration_IO;

interface

uses
  Classes,
  SysUtils,
  System.Contnrs,
  DateUtils,
  // Controls,
  fIA_dialogs, Dialogs,
  Trpcb,
  uMHA_Administration;

function ReadAdministrations(aDFN, OrdererDUZ: string; Broker: TRPCBroker): string;
procedure ReadOffLineAdministrations(aDFN: string);
function WriteAdministration(AnAdministration: TAdministration; Broker: TRPCBroker): string;
procedure SaveThisScore(anAdministrationID: string; Broker: TRPCBroker);
function WriteAdministrationProgressNote(anAdministrationID,
  aProgressNoteIEN, aCoSignerIEN: string; aNote: TStringList; Broker: TRPCBroker): string;
function WriteAdministrationConsultNote(anAdministrationID,
  aConsultNoteIEN, aCoSignerIEN: string; aNote: TStringList; Broker: TRPCBroker): string;  
//function SendHL7Message(anAdministrationID: string; Broker: TRPCBroker): Boolean;
function NotifyTestOrderer(AOrdererDUZ, ADFN, Instruments: string; Broker: TRPCBroker): string;
function DeleteAdministration(aAdminID: string; Broker: TRPCBroker): string;

implementation

uses
  uMHA_Administration_OPS,
  uMHA_LocalFiles,
  uIA_Patient,
  uIA_Utils,
  uIA_Const,
  uMHA_Const;

procedure ReadOffLineAdministrations(aDFN: string);
// List of incomplete Off-line administrations
// aDFN is the temporary, local ID for off-line patient
var
  sr                          : TSearchRec;
  TempList                    : TStringList;
  TempAdmin                   : TAdministration;

  procedure ReadAnAdministrationFile(aFileName: string);
  var
    RestartInt                : Integer;
    TempIsComplete            : Boolean;
    TempDate                  : TDateTime;
  begin
    //100037^AA Fine Bogus Test^3040414.163109^3040414.163114^11684^11684^N^5^5^N^100022^^N^2
    //100036^AA Fine Bogus Test^3040414.162807^3040414.162807^11684^11684^N^1^5^N^100022^^N^12
    //100019^AIMS^3040401.181246^3040401.181321^11684^11684^N^4^^Y^2^234^N^12
    //100017^BOGUS #8^3040316.180005^3040316.180005^5325^5325^N^1^5^N^100008^^N^12
    //100014^AUDC^3040310.163034^3040310.163035^5325^5325^N^1^^Y^5^208^N^12
    //^CAGE^3040310.1205^3040310^5325^^^1^Y^Y^^226^N^12
    //^AUIR^3040309.1439^3040309^11684^^^24^Y^Y^^7^Y^12
    //^AUDC^3040309.1244^3040309^11684^^^1^Y^Y^^208^N^12

    //1. Administration ID
    //2. Instrument Name
    //3. Date Given
    //4. Date Saved
    //5. Orderer
    //6. Admin.By
    //7. Signed
    //8. # Answers
    //9. Exempt/ R_Privl
    //10. Is Legacy
    //11. INSTRUMENT id 601.71
    //12. Test IENS 601
    //13. Is Copyrighted
    //14. Location IEN

    // Contents of adm file:
    //AdministrationID=X1
    //InstrumentName=AUDC
    //DateGiven=5/6/2004 9:32:22 AM
    //DateSaved=5/6/2004 9:32:26 AM
    //Signed=FALSE
    //IsComplete=TRUE
    //NumberOfQuestionsAnswered=3

    RestartInt := 2;

    if aDFN <> Piece(aFileName, FILE_NAME_DELIMITER, 1) then
      Exit;

    TempList.LoadFromFile(LocalFiles.AnswerFilesPath + '\' + aFileName);
    TempIsComplete := StrToBool(TempList.Values['IsComplete']);

    if TempIsComplete then
      exit
    else
      begin
        // To avoid bad data (e.g., Victor Rabbit in L1
        // DateGiven format is 09/18/2007 06:36:53 AM
        if TempList.Values['DateGiven'] = '' then
          Exit;

        // We only want the incomplete instruments for this user within the
        // past few days. The duration will be fine tuned below.

        TempDate := StrToDateTime(TempList.Values['DateGiven']);

        // Adjust the two-day Rule for the weekend;
        if DayOfWeek(TempDate) = 6 {Friday} then
          RestartInt := RestartInt + 2;
        if DayOfWeek(TempDate) = 7 {Saturday} then
          RestartInt := RestartInt + 1;

        if (DaysBetween(Now, TempDate) < RestartInt) then
          begin
            TempAdmin := TAdministration.Create;
            with TempAdmin do
              begin
                InstrumentName := TempList.Values['InstrumentName'];
                ID := TempList.Values['AdministrationID'];
                RecordID := '';
                AdministeredBy := NO_ID;
                PatientID := NO_ID;
                OrdererDUZ := NO_ID;
                LocationIEN := '';
                LocationStr := '';

                DateGiven := TempDate;
                DateSaved := Now;

                IsRestartable := true;
                IsComplete := False;
                Signed := False;

                NumberAnswered := StrToIntDef(TempList.Values['NumberOfQuestionsAnswered'], 0);
                NumberSkipped := 0;
                NumberStillRequired := 0;
              end; // with TempAdmin

            Administration_OPS.ListOfIncompletes.Add(TempAdmin);

            // Remove this file: The current administration will replace it.
            DeleteFile(LocalFiles.AnswerFilesPath + '\' + aFileName);

          end // within date range
        else
        // Remove incomplete off-line administration file
          begin
            LocalFiles.CleanupDatedOfflineFiles(LocalFiles.AnswerFilesPath + '\' +
              aFileName);
          end;
      end;
  end;

begin
  TempList := TStringList.Create;

  if (FindFirst(LocalFiles.AnswerFilesPath + '\' + aDFN
    + '*.' + ADMIN_FILE_EXT, faAnyFile, sr) = 0) then
    ReadAnAdministrationFile(sr.Name);

  while (FindNext(sr) = 0) do
    ReadAnAdministrationFile(sr.Name);

  SysUtils.FindClose(sr);
  TempList.Free;
end;


function ReadAdministrations(aDFN, OrdererDUZ: string; Broker: TRPCBroker): string;
var
  i,
    RestartInt                : Integer;
  TempStr                     : string;
  TempAdmin                   : TAdministration;
  TempDate                    : TDateTime;
begin
// Format returned:
//   1         2                  3           4             5    6   7 8 9 10 11   12 1314
// 100764^A Report-Script Test^3050317.13525^3050317.155349^6627^6627^N^11^^N^100065^^N^2~
// 100750^A Pretty Good Test^3050314.063229^3050314.083402^6627^6627^N^11^^N^100038^^N^12~

// 1. Administration ID
// 2. Instrument Name
// 3. Date Given
// 4. Date Saved
// 5. Orderer
// 6. Admin.By
// 7. Signed
// 8. # Answers
// 9. Exempt/ R_Privl
// 10. Is Legacy
// 11. INSTRUMENT id 601.71
// 12. Test IENS 601
// 13. Is Copyrighted
// 14. Location IEN
// 15. DaysToRestart

  Result := OK;
  //RestartInt := 0;

  with Broker do
    begin
      ClearResults := True;
      ClearParameters := True;
      CreateContext('YS BROKER1');
      //* LocalFiles.UpdateAPILog('Call to YTQ ADMINS BOTH: DFN=' + aDFN);
      RemoteProcedure := 'YTQ ADMINS BOTH';
      Param[0].Mult['"DFN"'] := aDFN;
      Param[0].Mult['"COMPLETE"'] := 'N';
      Param[0].PType := List;

      try
        Call;

      except
        on E: EBrokerError do
          begin
            //* LocalFiles.UpdateAPILog('Call to YTQ ADMINS BOTH: Error ');
            //* LocalFiles.UpdateAPILog('   ' + 'Broker Error: ' + E.message);
            exit;
          end;
        on E: EAccessViolation do
          begin
            //* LocalFiles.UpdateAPILog('Call to YTQ ADMINS BOTH: Error ');
            //* LocalFiles.UpdateAPILog('   ' + 'Access Violation: ' + E.message);
            exit;
          end;
      end; // try

      if not (Results[0] = DATA) then
        begin
          Result := BAD_DATA;
          //* LocalFiles.UpdateAPILog('Call to YTQ ADMINS BOTH: Error');
          //* for i := 0 to Results.Count - 1 do
            //* LocalFiles.UpdateAPILog('   ' + Results[i]);
          Exit;
        end;

      for i := 2 to Results.Count - 1 do
        begin

          TempStr := Results[i];

          //To avoid bad data (e.g., Victor Rabbit in L1
          if Piece(TempStr, U, 4) = '' then
            Continue;

          RestartInt := StrToIntDef(Piece(TempStr, U, 15), 2);

          if RestartInt = 0 then // DaysToRestart=0 means that incompletes are not allowed.
            Continue;

          // We only want the incomplete instruments for this user within the
          // past few days. The duration will be fine tuned below.
          TempDate := FMDateTimeToDateTime(StrToFloat(Piece(TempStr, U, 4)));

          // Adjust the two-day Rule for the weekend;
          if DayOfWeek(TempDate) = 6 {Friday} then
            RestartInt := RestartInt + 2;
          if DayOfWeek(TempDate) = 7 {Saturday} then
            RestartInt := RestartInt + 1;

          if ((Piece(TempStr, U, 5) = OrdererDUZ) and
            ((DaysBetween(Now, TempDate) < RestartInt)) or
            (RestartInt = -1)) then
            begin
              TempAdmin := TAdministration.Create;
              with TempAdmin do
                begin
                  InstrumentName := Piece(TempStr, U, 2);
                  ID := Piece(TempStr, U, 1);
                  RecordID := '';
                  AdministeredBy := Piece(TempStr, U, 6);
                  PatientID := Patient.DFN;
                  OrdererDUZ := Piece(TempStr, U, 5);
                  LocationIEN := Piece(TempStr, U, 14);
                  LocationStr := '';

                  DateGiven := FMDateTimeToDateTime(StrToFloat(Piece(TempStr, U, 3))); //TempDate;
                  DateSaved := FMDateTimeToDateTime(StrToFloat(Piece(TempStr, U, 4))); //Now;

                  IsRestartable := true;
                  IsComplete := False;
                  Signed := False;

                  NumberAnswered := StrToIntDef(Piece(TempStr, U, 8), 0);
                  NumberSkipped := 0;
                  NumberStillRequired := 0;
                  DaysToRestart := RestartInt;
                end;
              Administration_OPS.ListOfIncompletes.Add(TempAdmin);
            end;
        end;
      //* LocalFiles.UpdateAPILog('Call to YTQ ADMINS BOTH: Successful');
    end; //with DataModule1
end;


function ReadOffLineAnswerFileNames: string;
// File name is something like 3~X4.ANS
var
  sr                          : TSearchRec;
begin
  Result := '';
  if (FindFirst(LocalFiles.AnswerFilesPath + '\*.' + ANSWER_FILE_EXT,
    faAnyFile, sr) = 0) then
    begin
      Result := sr.Name + U; //RECORDEND;

      while (FindNext(sr) = 0) do
        Result := Result + sr.Name + U; //RECORDEND;

      Result := DropTrailer(Result);
    end;

  FindClose(sr);
end;


function NewOffLineID: string;
var
  i, Max, tmpID               : integer;
  tmpString, Piece2           : string;
begin
  // ReadOffLineAnswerFileNames returns filenames of all
  // the off-line administration files, file name format: PatientID~AdminID.ANS, e.g.,
  //   112212~X2.ANS^12321~X3.ANS^234324~X4.ANS

  tmpString := ReadOffLineAnswerFileNames;

  if (tmpString = '') then
    begin
      Result := 'X1';
      Exit;
    end;

  Max := 0;
  for i := 1 to NumberOfPieces(tmpString, U) do
    begin
    //Get the second piece of the left side of the answer file name
    //Example of file name:  1231233~X5.ANS
      Piece2 := Piece(Piece(Piece(tmpString, U, i), FILE_NAME_DELIMITER, 2),
        '.', 1);
    //Drop the X
      Piece2 := copy(Piece2, 2, Length(Piece2));
      tmpID := StrToIntDef(Piece2, -1);
      if (tmpID > Max) then
        Max := tmpID;
    end;

  Result := 'X' + IntToStr(Max + 1);
end;


function DeleteAdministration(aAdminID: string; Broker: TRPCBroker): string;
var
  i                           : integer;
begin
  result := '';

  with Broker do
    begin
      ClearParameters := True;
      ClearResults := True;
      CreateContext('YS BROKER1');
      //* LocalFiles.UpdateAPILog('Call to YTQ ADMIN DELETE: ' +
      //*   ' ID=' + aAdminID);
      RemoteProcedure := 'YTQ ADMIN DELETE';
      Param[0].Mult['"AD"'] := aAdminID;
      Param[0].PType := List;
      try
        Call;

      except
        on E: EBrokerError do
          begin
            //* LocalFiles.UpdateAPILog('Call to YTQ ADMIN DELETE: Error ');
            //* LocalFiles.UpdateAPILog('   ' + 'Broker Error: ' + E.message);
            exit;
          end;
        on E: EAccessViolation do
          begin
            //* LocalFiles.UpdateAPILog('Call to YTQ ADMIN DELETE: Error ');
            //* LocalFiles.UpdateAPILog('   ' + 'Access Violation: ' + E.message);
            exit;
          end;
      end; // try

      if Results[0] = DATA then
        begin
          //* LocalFiles.UpdateAPILog('Call to YTQ ADMIN DELETE: Successful');
          Result := OK;
        end
      else
        begin
          //* LocalFiles.UpdateAPILog('Call to YTQ ADMIN DELETE: Error');
          for i := 1 to Results.Count - 1 do
            begin
              Result := Result + ' ' + Results[i];
              //* LocalFiles.UpdateAPILog('   ' + Results[i]);
            end;
        end;

      CreateContext('OR CPRS GUI CHART');
    end;
end;

function WriteAdministration(AnAdministration: TAdministration; Broker: TRPCBroker): string;
var
  tmpList                     : TStringList;
  TempDateStr                 : string;
begin
  Result := '';
  //ShowMessage(ID +#13+
//            PatientID +#13+
//            InstrumentName +#13+
//            FloatToStr(DateTimeToFMDateTime(DateGiven))+#13+
//            OrdererDUZForNewAdministrations  +#13+
//            AdministeredBy +#13+
//            BoolToMStr(Signed) +#13+
//            BoolToMStr(IsComplete)+#13+
//            IntToStr(NumberAnswered) +#13+
//            LocationIEN );

  case DataBaseMode of
    dbVista:
      begin
        with Broker, AnAdministration do
          begin
            // Make sure date is okay for Vista. It must have format of
            // XXXXXX.Xn
            TempDateStr :=
              FloatToStr(DateTimeToFMDateTime(DateGiven));
            if Pos('.', TempDateStr) = 0 then
              TempDateStr := TempDateStr + '.1';

            ClearParameters := True;
            ClearResults := True;
            CreateContext('YS BROKER1');
            //* LocalFiles.UpdateAPILog('Call to YTQ SAVE: ' +
            //*   ' ID=' + ID +
            //*   ' 1=' + PatientID +
            //*   ' 2=' + InstrumentName +
            //*   ' 3=' + TempDateStr +
            //*   ' 5=' + OrdererDUZ +
            //*   ' 6=' + AdministeredBy +
            //*   ' 7=' + BoolToMStr(Signed) +
            //*   ' 8=' + BoolToMStr(IsComplete) +
            //*   ' 9=' + IntToStr(NumberAnswered) + ' ' +
            //*   ' 13=' + LocationIEN);
            RemoteProcedure := 'YTQ SAVE';
            Param[0].Mult['"FILEN"'] := FILE_ADMINISTRATIONS;

            if ID = NO_ID then
              Param[0].Mult['"1"'] := '.01^NEW^1'
            else
              Param[0].Mult['"IEN"'] := ID;

            Param[0].Mult['"2"'] := '1^`' + PatientID;
            Param[0].Mult['"3"'] := '2^' + InstrumentName;
            Param[0].Mult['"4"'] := '3^' + TempDateStr;
            Param[0].Mult['"5"'] := '4^NOW'; //Administration.DateSaved
            Param[0].Mult['"6"'] := '5^`' + OrdererDUZ;
            Param[0].Mult['"7"'] := '6^`' + AdministeredBy;
            Param[0].Mult['"8"'] := '7^' + BoolToMStr(Signed); //IsSigned
            Param[0].Mult['"9"'] := '8^' + BoolToMStr(IsComplete); //IsComplete
            Param[0].Mult['"10"'] := '9^' + IntToStr(NumberAnswered); //NumberOfQuestionsAnswered

            if (LocationIEN = '') or (LocationIEN = ' ') then
            // null causes crash
            else
              Param[0].Mult['"11"'] := '13^`' + LocationIEN; //Location
            Param[0].PType := List;
            try
              Call;

            except
              on E: EBrokerError do
                begin
                  //* LocalFiles.UpdateAPILog('Call to YTQ SAVE: Error ');
                  //* LocalFiles.UpdateAPILog('   ' + 'Broker Error: ' + E.message);
                  exit;
                end;
              on E: EAccessViolation do
                begin
                  //* LocalFiles.UpdateAPILog('Call to YTQ SAVE: Error ');
                  //* LocalFiles.UpdateAPILog('   ' + 'Access Violation: ' + E.message);
                  exit;
                end;
            end; // try

            if Results[0] = DATA then
              begin
                Result := Piece(Results[1], U, 2); // Administration.ID
                //* LocalFiles.UpdateAPILog('Call to YTQ SAVE: Successful');
              end
            else
              begin
                Result := 'ERROR ' + results[1];
                //* LocalFiles.UpdateAPILog('Call to YTQ SAVE: Error');
                //* for i := 0 to Results.Count - 1 do
                  //* LocalFiles.UpdateAPILog('   ' + Results[i]);
              end;

            CreateContext('OR CPRS GUI CHART');
          end; //DataModule1.RPCBroker1
      end; //case dbVista

    dbOffLine:
      begin
        with AnAdministration do
          begin
            tmpList := TStringList.Create;
            tmpList.Clear;

            Result := NewOffLineID;

            tmpList.Add('AdministrationID=' + Result);
            tmpList.Add('InstrumentName=' + InstrumentName);
            tmpList.Add('DateGiven=' + DateTimeToStr(DateGiven));
            tmpList.Add('DateSaved=' + DateTimeToStr(Now));
            tmpList.Add('Signed=' + BoolToStr(Signed));
            tmpList.Add('IsComplete=' + BoolToStr(IsComplete));
            tmpList.Add('NumberOfQuestionsAnswered=' + IntToStr(NumberAnswered));
            tmpList.Add('Location=');
            tmpList.Add('DaysToRestart=' + IntToStr(DaysToRestart));

            tmpList.SaveToFile(LocalFiles.BuildAdministrationFileName(PatientID,
              DateTimeToFileNameDateTime(DateGiven),
              InstrumentName));

            tmpList.Free;
          end;
      end; // Off line

  end; //case DataMode
end;


procedure SaveThisScore(anAdministrationID: string; Broker: TRPCBroker);
// All the work is done on the M-side.
begin
  with Broker do
    begin
      ClearParameters := True;
      ClearResults := True;
      CreateContext('YS BROKER1');
      //* LocalFiles.UpdateAPILog('Call to YTQ SCORE SAVE: AD=' + anAdministrationID);

      RemoteProcedure := 'YTQ SCORE SAVE';
      Param[0].Mult['"AD"'] := anAdministrationID;
      Param[0].PType := List;
      try
        Call;

      except
        on E: EBrokerError do
          begin
            //* LocalFiles.UpdateAPILog('Call to YTQ SCORE SAVE: Error ');
            //* LocalFiles.UpdateAPILog('   ' + 'Broker Error: ' + E.message);
            exit;
          end;
        on E: EAccessViolation do
          begin
            //* LocalFiles.UpdateAPILog('Call to YTQ SCORE SAVE: Error ');
            //* LocalFiles.UpdateAPILog('   ' + 'Access Violation: ' + E.message);
            exit;
          end;
      end; // try

      //* if Results[0] = DATA then
      //*   begin
          //* LocalFiles.UpdateAPILog('Call to YTQ SCORE SAVE: Successful');
      //*   end
      //* else
      //*   begin
          //* LocalFiles.UpdateAPILog('Call to YTQ SCORE SAVE: Error');
          //* for i := 0 to Results.Count - 1 do
            //* LocalFiles.UpdateAPILog('   ' + Results[i]);
       //*  end;

      CreateContext('OR CPRS GUI CHART');
    end;
end;

function WriteAdministrationConsultNote(anAdministrationID,
  aConsultNoteIEN, aCoSignerIEN: string; aNote: TStringList; Broker: TRPCBroker): string;
var
  i                           : Integer;
  YS                          : string;
begin
  Result := BAD_DATA;

  with Broker do
    begin
      ClearParameters := True;
      ClearResults := True;
      CreateContext('YS BROKER1');
      //* LocalFiles.UpdateAPILog('Call to YTQ CONSULT CREATE:' +
      //*   ' AD=' + anAdministrationID +
      //*   ' TIUIEN=' + aConsultNoteIEN);

      RemoteProcedure := 'YTQ CONSULT CREATE';
      Param[0].Mult['"AD"'] := anAdministrationID;

      if aConsultNoteIEN <> '' then
        Param[0].Mult['"CON"'] := aConsultNoteIEN;

      if aCoSignerIEN <> '' then
        Param[0].Mult['"COSIGNER"'] := aCoSignerIEN;

      for i := 0 to aNote.Count - 1 do
        begin
          YS := '"' + IntToStr(i + 1) + '"';
          Param[0].Mult[YS] := aNote.Strings[i];
        end;

      Param[0].PType := List;
      try
        Call;

      except
        on E: EBrokerError do
          begin
            //* LocalFiles.UpdateAPILog('Call to YTQ CONSULT CREATE: Error ');
            //* LocalFiles.UpdateAPILog('   ' + 'Broker Error: ' + E.message);
            exit;
          end;
        on E: EAccessViolation do
          begin
            //* LocalFiles.UpdateAPILog('Call to YTQ CONSULT CREATE: Error ');
            //* LocalFiles.UpdateAPILog('   ' + 'Access Violation: ' + E.message);
            exit;
          end;
      end; // try

      if Results[0] = DATA then
        begin
          Result := Results[1];
          //* LocalFiles.UpdateAPILog('Call to YTQ CONSULT CREATE: Successful');
        end;
      //* else
      //*   begin
          //* LocalFiles.UpdateAPILog('Call to YTQ CONSULT CREATE: Error');
          //* for i := 0 to Results.Count - 1 do
            //* LocalFiles.UpdateAPILog('   ' + Results[i]);
       //*  end;

      CreateContext('OR CPRS GUI CHART');
    end;
end;


function WriteAdministrationProgressNote(anAdministrationID,
  aProgressNoteIEN, aCoSignerIEN: string; aNote: TStringList; Broker: TRPCBroker): string;
var
  i                           : Integer;
  YS                          : string;
begin
  Result := BAD_DATA;

  with Broker do
    begin
      ClearParameters := True;
      ClearResults := True;
      CreateContext('YS BROKER1');
      //* LocalFiles.UpdateAPILog('Call to YTQ PN CREATE:' +
      //*   ' AD=' + anAdministrationID +
      //*   ' TIUIEN=' + aProgressNoteIEN);

      RemoteProcedure := 'YTQ PN CREATE';
      Param[0].Mult['"AD"'] := anAdministrationID;

      if aProgressNoteIEN <> '' then
        Param[0].Mult['"TIUIEN"'] := aProgressNoteIEN;

      if aCoSignerIEN <> '' then
        Param[0].Mult['"COSIGNER"'] := aCoSignerIEN;

      for i := 0 to aNote.Count - 1 do
        begin
          YS := '"' + IntToStr(i + 1) + '"';
          Param[0].Mult[YS] := aNote.Strings[i];
        end;

      Param[0].PType := List;
      try
        Call;

      except
        on E: EBrokerError do
          begin
            //* LocalFiles.UpdateAPILog('Call to YTQ PN CREATE: Error ');
            //* LocalFiles.UpdateAPILog('   ' + 'Broker Error: ' + E.message);
            exit;
          end;
        on E: EAccessViolation do
          begin
            //* LocalFiles.UpdateAPILog('Call to YTQ PN CREATE: Error ');
            //* LocalFiles.UpdateAPILog('   ' + 'Access Violation: ' + E.message);
            exit;
          end;
      end; // try

      if Results[0] = DATA then
        begin
          Result := Results[1];
          //* LocalFiles.UpdateAPILog('Call to YTQ PN CREATE: Successful');
        end;
      //* else
      //*   begin
          //* LocalFiles.UpdateAPILog('Call to YTQ PN CREATE: Error');
          //* for i := 0 to Results.Count - 1 do
            //* LocalFiles.UpdateAPILog('   ' + Results[i]);
       //*  end;

      CreateContext('OR CPRS GUI CHART');
    end;
end;

(* HL7 messages turn off Aug 2, 1011
function SendHL7Message(anAdministrationID: string; Broker: TRPCBroker): boolean;
var
  i                           : Integer;
//  Reply                       : TModalResult;
//  tempstr : string;

begin
  //NAME: YTQ HL7
  //    Input: AD as administration ID
  //    Output : DATA vs ERROR
  result := False;

  with Broker do
    begin
      ClearParameters := True;
      ClearResults := True;
      CreateContext('YS BROKER1');
      //* LocalFiles.UpdateAPILog('Call to YTQ HL7: AD=' + anAdministrationID);
      RemoteProcedure := 'YTQ HL7';
      Param[0].Mult['"AD"'] := anAdministrationID;
      Param[0].PType := List;
      try
        Call;

      except
        on E: EBrokerError do
          begin
            //* LocalFiles.UpdateAPILog('Call to YTQ HL7: Error ');
            //* LocalFiles.UpdateAPILog('   ' + 'Broker Error: ' + E.message);
            exit;
          end;
        on E: EAccessViolation do
          begin
            //* LocalFiles.UpdateAPILog('Call to YTQ HL7: Error ');
            //* LocalFiles.UpdateAPILog('   ' + 'Access Violation: ' + E.message);
            exit;
          end;
      end; // try

      if Results[0] = DATA then
        begin
        //* LocalFiles.UpdateAPILog('Call to YTQ HL7: Successful')
          result := true;
        end
      else
        begin
          //* LocalFiles.UpdateAPILog('Call to YTQ HL7: Error');
          //* for i := 0 to Results.Count - 1 do
            //* LocalFiles.UpdateAPILog('   ' + Results[i]);
          result := False;
        end;

      CreateContext('OR CPRS GUI CHART');
    end;
end;*)


function NotifyTestOrderer(aOrdererDUZ, aDFN, Instruments: string; Broker: TRPCBroker): string;
begin
  //Send Vista email to orderer.
  with Broker do
    begin

      ClearParameters := true;
      ClearResults := true;
      CreateContext('YS BROKER1');
      //* LocalFiles.UpdateAPILog('Call to YTQ TEST ORDER: ORD=' + AOrdererDUZ +
      //*   ' DFN=' + aDFN +
      //*   ' TEST1=' + Instruments);
      RemoteProcedure := 'YTQ TEST ORDER';
      Param[0].Mult['"ORD"'] := AOrdererDUZ;
      Param[0].Mult['"DFN"'] := ADFN;
      Param[0].Mult['"TEST1"'] := Instruments;
      Param[0].PType := List;

      try
        Call;

      except
        on E: EBrokerError do
          begin
            //* LocalFiles.UpdateAPILog('Call to YTQ TEST ORDER: Error ');
            //* LocalFiles.UpdateAPILog('   ' + 'Broker Error: ' + E.message);
            exit;
          end;
        on E: EAccessViolation do
          begin
            //* LocalFiles.UpdateAPILog('Call to YTQ TEST ORDER: Error ');
            //* LocalFiles.UpdateAPILog('   ' + 'Access Violation: ' + E.message);
            exit;
          end;
      end; // try

      if (Results[0] = DATA) then
        begin
          //* LocalFiles.UpdateAPILog('Call to YTQ TEST ORDER: Successful');
          Result := OK;
        end
      else
        begin
          Result := Results[1];
          //* LocalFiles.UpdateAPILog('Call to YTQ TEST ORDER: Error');
          //* for i := 0 to Results.Count - 1 do
            //* LocalFiles.UpdateAPILog('   ' + Results[i]);
        end;

      CreateContext('OR CPRS GUI CHART');
    end;
end;

end.

