unit uMHA_Instrument;

interface

uses
  Contnrs,
  classes;

type
  TInstrument = class(TComponent)
  private
    FIEN,
      FName,
      FPrintTitle,
      FVersion,
      FAuthor,
      FPublisher,
      FCopyright,
      FPublicationDate,
      FReference,
      FNormSample,
      FPurpose,
      FTargetPopulation,
      FEntryDate,
      FEnteredBy,
      FA_Privilege,
      FR_Privilege,
      FLastEditFMDate,
      FLastEditedBy,
      FLastEditDate: string;

    FIsNationalTest,
      FSubmitToNationalDB,
      FRequiresSignature,
      FRequiresLicense,
      FLicenseIsCurrent,
      FIsLegacy,
      FIsCopyrighted,
      FWriteFullText,
      FIsUnderDevelopment: Boolean;

    FIsOperational: CHAR;

    FDaysToRestart,
    FNumberOfQuestions: Integer;

    FListOfQuestions: TObjectList;
    FListOfIntroductions: TObjectList;
    FListOfSections: TObjectList;
    FListOfRules: TObjectList;
  protected
  public
    constructor Create(aOwner: TComponent); override;
    destructor Destroy; override;
  published
    {** Does an administration have to be signed by a staff person?}
    property RequiresSignature: BOOLEAN read FRequiresSignature write FRequiresSignature;
    {** Journal article, publisher manual, etc}
    property Reference: string read FReference write FReference;
    {** Is a license (or permission) required to administer instrument?
    NB: If yes, then cannot be released under FOIA}
    property RequiresLicense: BOOLEAN read FRequiresLicense write FRequiresLicense;
    {** Software is operational}
    property IsOperational: CHAR read FIsOperational write FIsOperational;
    {**Is the license current (relevant only if license is required)}
    property LicenseIsCurrent: BOOLEAN read FLicenseIsCurrent write FLicenseIsCurrent;

    property IsLegacy: BOOLEAN read FIsLegacy write FIsLegacy;
    property NumberOfQuestions: Integer read FNumberOfQuestions write
      FNumberOfQuestions;
     property DaysToRestart: Integer read FDaysToRestart write
      FDaysToRestart;
    property IEN: string read FIEN write FIEN;
    {** No duplicates! If instrument content (questions or choices) changes,
    then a new instrument with a new name must be created.}
    property Name: string read FName write FName;
    property PrintTitle: string read FPrintTitle write FPrintTitle;
    property Publisher: string read FPublisher write FPublisher;
    {** The version number in the test manual (e.g., ASI Version 5).}
    property Version: string read FVersion write FVersion;
    property Author: string read FAuthor write FAuthor;
    {** The copyright notice.}
    property Copyright: string read FCopyright write FCopyright;
    property PublicationDate: string read FPublicationDate write
      FPublicationDate;
    property NormSample: string read FNormSample write FNormSample;
    {** Results Privilege (eg. YSP)--What permission is required to see instrument results.}
    property R_Privilege: string read FR_Privilege write FR_Privilege;
    {** What is the test used for, e.g., an alcohol history, personality test, etc.}
    property Purpose: string read FPurpose write FPurpose;
    {** The clinical population for which the test is appropriate.}
    property TargetPopulation: string read FTargetPopulation write
      FTargetPopulation;
    property EntryDate: string read FEntryDate write FEntryDate;
    {** The name (and/or DFN?) of the person who entered the test into VistA.
    Needed to track locally developed tests.}
    property EnteredBy: string read FEnteredBy write FEnteredBy;
    {** Admin Privilege--What permission is required to administer the instrument.}
    property A_Privilege: string read FA_Privilege write FA_Privilege;
    {** The name (and/or DFN?) of the person who  last edited the test in VistA.}
    property LastEditedBy: string read FLastEditedBy write FLastEditedBy;
    property LastEditDate: string read FLastEditDate write FLastEditDate;
    {** Is this a local test or one that we released nationally.}
    property IsNationalTest: BOOLEAN read FIsNationalTest write FIsNationalTest;
    property LastEditFMDate: string read FLastEditFMDate write FLastEditFMDate;

    property SubmitToNationalDB: Boolean read FSubmitToNationalDB write FSubmitToNationalDB;
    //property AllDisplayTypes: TStringList read FAllDisplayTypes write FAllDisplayTypes;
    property IsCopyrighted: BOOLEAN read FIsCopyrighted write FIsCopyrighted;
    property WriteFullText: BOOLEAN read FWriteFullText write FWriteFullText;
    property IsUnderDevelopment: BOOLEAN read FIsUnderDevelopment write FIsUnderDevelopment;

    property ListOfQuestions: TObjectList read FListOfQuestions write FListOfQuestions;
    property ListOfIntroductions: TObjectList read FListOfIntroductions write FListOfIntroductions;
    property ListOfSections: TObjectList read FListOfSections write FListOfSections;
    property ListOfRules: TObjectList read FListOfRules write FListOfRules;

  end; { TInstrument }

  TAvailableInstrument = class
  private
    FName,
      FID,
      FR_Privilege,
      FLastEditDate: string;
    FRequiresLicense,
      FLicenseIsCurrent,
      FCopyrightStatusIsOK,
      FCanBeAdministered,
      FIsLegacy,
      FIsNationalInstrument,
      FHasBeenOperational: Boolean;
    FIsOperational: Char;
  protected
  public
    property Name: string read FName write FName;
    property ID: string read FID write FID;
    property R_Privilege: string read FR_Privilege write FR_Privilege;
    property LastEditDate: string read FLastEditDate write FLastEditDate;
    property RequiresLicense: BOOLEAN read FRequiresLicense write
      FRequiresLicense;
    property LicenseIsCurrent: BOOLEAN read FLicenseIsCurrent write
      FLicenseIsCurrent;
    property CopyrightStatusIsOK: BOOLEAN read FCopyrightStatusIsOK write
      FCopyrightStatusIsOK;
    property IsLegacy: BOOLEAN read FIsLegacy write FIsLegacy;
    property IsNationalInstrument: BOOLEAN read FIsNationalInstrument write
      FIsNationalInstrument;
    property CanBeAdministered: BOOLEAN read FCanBeAdministered write
      FCanBeAdministered;
    property HasBeenOperational: BOOLEAN read FHasBeenOperational write
      FHasBeenOperational;
    property IsOperational: Char read FIsOperational write FIsOperational;

  published
  end; { TAvailableInstrument }

var
  AvailableInstrument         : TAvailableInstrument;


implementation


constructor TInstrument.Create(aOwner: TComponent);
begin
  inherited Create(nil);

  FListOfQuestions := TObjectList.Create;
  FListOfIntroductions := TObjectList.Create;
  FListOfSections := TObjectList.Create;
  FListOfRules := TObjectList.Create;
end;

destructor TInstrument.Destroy;
begin
  ListOfIntroductions.Free; // This also frees the Introduction objects
  ListOfSections.Free;
  ListOfQuestions.Free;
  ListOfRules.Free;

  inherited Destroy;
end;


end.
