unit uMHA_Report_OPS;


interface


uses
  SysUtils,
  classes,
  ComCtrls,
  Contnrs,
  fIA_dialogs, Dialogs,
  Trpcb,
  forms;

type
  TMHA_Report_OPS = class
  private
    FListOfQuestionsForReport,
      FListOfAnswersForReport,
      FListOfScalesForReport,
      FListOfKeysForReport,
      FListOfScriptsForReport: TObjectList;

    // patch 129, added aInstrumentName parameter to print totals for specific reports llh
    procedure ReplaceDataText(ForceWrap: Boolean; aInstrumentName: String;
      var aReportStrList: TStringList);

    function Replacement(DataText: string): string;
    function EvaluateForScript(AScriptName: string): string;
    function EvaluateThisScript(
      WhichQuestion,
      WhichAnswer,
      ScriptAnswer,
      ScriptAnswerIndex,
      ScriptOperation: string): string;

    function GetScaleScore(AScaleName: string): string;
    function BuildSectionHeader(ACaption: string): string;
    function AddCommentsToReport(AdminIENS: string): string;
    function GetAnswerText(ChoicesIn: TStringList; ChoiceID: string): string;
    function IsQuestionMultiChoice(QID: string): Integer;
    procedure BuildDataTextMarkers;

    function LoadDefaultReport(
      aInstrumentName,
      aAdministrationIENS,
      aDateGiven,
      aDateSaved: string;
      aWriteFullText: Boolean): string;

    procedure LoadComments(AdminIENS: string);
  protected
  public
    constructor Create;
    destructor Destroy; override;

    property ListOfQuestionsForReport: TObjectList read FListOfQuestionsForReport write FListOfQuestionsForReport;
    property ListOfAnswersForReport: TObjectList read FListOfAnswersForReport write FListOfAnswersForReport;
    property ListOfScalesForReport: TObjectList read FListOfScalesForReport write FListOfScalesForReport;
    property ListOfKeysForReport: TObjectList read FListOfKeysForReport write FListOfKeysForReport;

    function BuildAllReportLists(aInstrumentName, AdminID: string): Boolean;

    function ReadRoughReport(aInstrumentName, aAdmin, aOrderer, aLocation,
      aDateGiven, aDateSaved: string;
      var aReportStrList: TStringList;
      var ReportIEN: string): string;

    // patch 108, added aInstrumentName parameter to print totals for specific reports llh
    procedure FinalReport(ForceWrap: Boolean; aInstrumentName: String;
      var aReportStrList: TStringList);

    function LoadReport(aInstrumentName,
      aAdministrationIENS,
      aAdminBy,
      aOrderedBy,
      aLocation,
      aDateGiven,
      aDateSaved: string;
      aTestIENS: Integer;
      ForceWrap, // Used for progress notes
      aIsLegacy,
      aWriteFullText: Boolean;
      aDLLString: string;
      var aReportStrList: TStringList): string;

    function Load508LegacyReport(aInstrumentName,
      aAdministrationIENS,
      aPatientDFN,
      aAdminBy,
      aOrderedBy,
      aLocation, // uses Location IEN
      aDateGiven,
      aDateSaved: string;
      aTestIENS: Integer;
      aDataInLegacyFile: boolean;
      var aReportStrList: TStringList): string;

    function LoadLegacyTestReport(aInstrumentName,
      aAdministrationIENS,
      aAdminBy,
      aOrderedBy,
      aLocation,
      aDateGiven,
      aDateSaved: string;
      aTestIENS: Integer;
      var aReportStrList: TStringList): string;

    function LoadReportTemplate(aInstrumentName,
      aAdministrationIENS,
      aAdminBy,
      aOrderedBy,
      aLocation,
      aDateGiven,
      aDateSaved: string;
      ForceWrap, // Used for progress notes
      aWriteFullText: Boolean;
      var aReportStrList: TStringList): string;

    function SaveComments(aAdminID, aComment: string): string;
    function GetAnswer(QID: string): string;
  end; { TMHA_Report }

var
  MHA_Report_OPS              : TMHA_Report_OPS;

implementation

uses
  uIA_Const,
  uIA_Answer,
  uIA_Scale,
  uIA_Key,
  uIA_Patient,
  uIA_Question,
  uIA_MultiChoice_IO,
  uMHA_Report_IO,
  uMHA_Location_IO,
  uMHA_InstrumentResults_IO,
  uIA_GUIControl,
  uIA_Scripts,
  uIA_Utils,
  uIA_ThisUser,
  uMHA_DM;

const

  lcMySeparator               =
    '---------------------------------------------------------------------------';
  lcSpace                     = '   ';

  lcDataTextMarkers           : string =

  'Patient_Name_First_Last^' + //1 *** Keep these in order ***
    'Patient_Name_Last_First^' + //2
    'Patient_Name_Mr_Last^' + // 3
    'Patient_Name_Ms_Last^' + // 4
    'Patient_Name_Mrs_Last^' + // 5
    'Patient_Name_Dr_Last^' + // 6
    'Patient_Name_First^' + // 7
    'Patient_Gender^' + // 8
    'Patient_Date_Of_Birth^' + // 9
    'Patient_Date_Of_Death^' + // 10
    'Patient_Age^' + // 11
    'Patient_SSN^' + // 12
    'Staff_Ordered_By^' + // 13
    'Staff_Interviewed_By^' + // 14
    'Location^' + //15
    'Date_Given^' + // 16
    'Date_Saved^' + // 17
    'Today''s_Date^' + // 18
    'Today''s_Date_&_Time^' + // 19
    'DLL_String^'; //20

var
  lvComments,
    lvInstrumentName,
    lvAdministerBy,
    lvOrderedBy,
    lvLocation,
    lvDateSaved,
    lvDateGiven,
    lvDLL_Scores,
    lvDLL_String              : string;
  lvWhichScript               : Integer;
  lvDataTextInUse             : TStringList;

constructor TMHA_Report_OPS.Create;
begin
  inherited Create;

  FListOfKeysForReport := TObjectList.Create;
  FListOfScalesForReport := TObjectList.Create;
  FListOfAnswersForReport := TObjectList.Create;
  FListOfQuestionsForReport := TObjectList.Create;
  FListOfScriptsForReport := TObjectList.Create;
  lvDataTextInUse := TStringList.Create;
  BuildDataTextMarkers;

end;


destructor TMHA_Report_OPS.Destroy;
begin
  if lvDataTextInUse <> nil then
    lvDataTextInUse.Free;
  FListOfKeysForReport.Free;
  FListOfScalesForReport.Free;
  FListOfAnswersForReport.Free;
  FListOfQuestionsForReport.Free;
  FListOfScriptsForReport.Free;
  inherited Destroy;
end;

function TMHA_Report_OPS.BuildAllReportLists(aInstrumentName, AdminID: string): Boolean;
begin
  Result := False;

  FListOfKeysForReport.Clear;
  FListOfScalesForReport.Clear;
  FListOfAnswersForReport.Clear;
  FListOfQuestionsForReport.Clear;

  if LoadQuestionsForReport(aInstrumentName, DataModule1.RPCBroker1) <> OK then
    Exit;
  if LoadQuestionChoicesForReport(aInstrumentName, DataModule1.RPCBroker1) <> OK then
    Exit;
  if LoadAnswersForReport(AdminID, Patient.DFN, DataModule1.RPCBroker1) <> OK then
    Exit;
  if LoadScalesForReport(aInstrumentName, DataModule1.RPCBroker1) <> OK then
    Exit;
  if FListOfScalesForReport.Count > 0 then
    ReadScoringKeys(aInstrumentName, DataModule1.RPCBroker1); // uMHA_Report_IO
  Result := True;
end;

// patch 108, added aInstrumentName parameter to print totals for specific reports llh
procedure TMHA_Report_OPS.FinalReport(ForceWrap: Boolean; aInstrumentName: string ;
  var aReportStrList: TStringList);
begin
  lvComments := '';
  // patch 108, added aInstrumentName parameter to print totals for specific reports llh
  ReplaceDataText(ForceWrap, aInstrumentName, aReportStrList);
end;


function TMHA_Report_OPS.ReadRoughReport(aInstrumentName, aAdmin, aOrderer, aLocation,
  aDateGiven, aDateSaved: string;
  var aReportStrList: TStringList;
  var ReportIEN: string): string;

var
  i,
    ReportEnd                 : integer;
  TempString,
    TempStr,
    CompleteText              : string;
  AScript                     : TScript;
begin
  lvInstrumentName := aInstrumentName;
  lvAdministerBy := aAdmin;
  lvOrderedBy := aOrderer;
  lvLocation := aLocation;
  lvDateSaved := aDateSaved;
  lvDateGiven := aDateGiven;
//ShowMessage('read rough report  '+lvLocation);
  result := 'ERROR';

  FListOfScriptsForReport.Clear;
  CompleteText := '';

  // The string saved contains the report text and the stringList of
  // scripts. The marker END_OF_REPORT separates them.

  CompleteText := ReadReportTemplate(aInstrumentName, ReportIEN, DataModule1.RPCBroker1);
  // in uMHA_Report_IO

  if CompleteText = 'ERROR' then
    exit;

  Result := '';

  // Find the position of the ENDOFREPORT
  ReportEnd := POS(END_OF_REPORT, CompleteText);

  // Brand new will be blank.
  if ReportEnd = 0 then
    begin
      CompleteText := CompleteText + END_OF_REPORT;
      ReportEnd := Length(CompleteText);
    end;

  CompleteText := Copy(CompleteText, 1, ReportEnd - 1);
  TempStr := '';

  for i := 0 to length(CompleteText) do
    begin
      if CompleteText[i] = CARRIAGE_RETURN_MARKER then
        begin
          if TempStr = ' .' then // First line
            TempStr := ' ';

          aReportStrList.Add(TempStr);
          TempStr := ' ';
        end
      else
        TempStr := TempStr + CompleteText[i];
    end;

  // Add the last line, just in case it does not end with a CR marker.
  aReportStrList.Add(TempStr);

  
  //This does not work--only gets about half the CRs
  //aReportStrList.Text := StringReplace(aReportStrList.Text,
  //  CARRIAGE_RETURN_MARKER, CRLF, [rfReplaceAll]);

  Result := 'OK';

  // Construct the ScriptList
  CompleteText := Copy(CompleteText, ReportEnd + 1, Length(CompleteText));

  for i := 0 to NumberOfPieces(CompleteText, RECORD_END) do
    begin
      TempString := Piece(CompleteText, RECORD_END, i);

      if Length(Trim(TempString)) > 6 then
        begin
          AScript := TScript.Create;

          AScript.ScriptName := Piece(TempString, '=', 1);
          TempStr := Piece(TempString, '=', 2);

          if TempStr = '' then
            Exit;

          AScript.FirstCaption := Piece(TempStr, SCRIPT_DELIMITER, 1);
          AScript.FirstQuestionID := Piece(TempStr, SCRIPT_DELIMITER, 2);
          AScript.FirstAnswer := Piece(TempStr, SCRIPT_DELIMITER, 3);
          AScript.FirstAnswerIndex := Piece(TempStr, SCRIPT_DELIMITER, 4);
          AScript.FirstOperation := Piece(TempStr, SCRIPT_DELIMITER, 5);
          AScript.SecondCaption := Piece(TempStr, SCRIPT_DELIMITER, 6);
          AScript.SecondQuestionID := Piece(TempStr, SCRIPT_DELIMITER, 7);
          AScript.SecondAnswer := Piece(TempStr, SCRIPT_DELIMITER, 8);
          AScript.SecondAnswerIndex := Piece(TempStr, SCRIPT_DELIMITER, 9);
          AScript.SecondOperation := Piece(TempStr, SCRIPT_DELIMITER, 10);
          AScript.ScriptText := Piece(TempStr, SCRIPT_DELIMITER, 11);

          FListOfScriptsForReport.Add(AScript);
        end;
    end;

      //if Length(Trim(TempStr)) > 6 then
      //  Script.ScriptList.Add(TempStr);

end;



procedure TMHA_Report_OPS.BuildDataTextMarkers;
var
  i                           : integer;
begin
  for i := 1 to NumberOfPieces(lcDataTextMarkers, U) do
    lvDataTextInUse.Add(Piece(lcDataTextMarkers, U, i));

end;


function TMHA_Report_OPS.Replacement(DataText: string): string;
// DataText format: <. _____ .>
var
  i                           : integer;
  TempStr                     : string;
  TempUser                    : TThisUser;

  function ReplacementTable(Got: string): string;
  var
    TempLastName,
      TempFirstName           : string;
  begin
    TempFirstName := MixedCase(Trim(Piece(Patient.Name, ',', 2)));
    TempLastName := MixedCase(Trim(Piece(Patient.Name, ',', 1)));

    if Got = Piece(lcDataTextMarkers, U, 1) then //'Patient_Name_First_Last
      result := MixedCase(TempFirstName) + MixedCase(TempLastName);
    if Got = Piece(lcDataTextMarkers, U, 2) then //Patient_Name_Last_First
      result := TempLastName + ', ' + TempFirstName;
    if Got = Piece(lcDataTextMarkers, U, 3) then //Patient_Name_Mr_Last
      Result := 'Mr. ' + TemplastName;
    if Got = Piece(lcDataTextMarkers, U, 4) then //Patient_Name_Ms_Last
      Result := 'Ms. ' + TemplastName;
    if Got = Piece(lcDataTextMarkers, U, 5) then //Patient_Name_Mrs_Last
      Result := 'Mrs. ' + TemplastName;
    if Got = Piece(lcDataTextMarkers, U, 6) then //Patient_Name_Dr_Last
      Result := 'Dr. ' + TemplastName;
    if Got = Piece(lcDataTextMarkers, U, 7) then //Patient_Name_First
      Result := TempFirstName;
    if Got = Piece(lcDataTextMarkers, U, 8) then //Patient_Gender
      begin
        result := 'Male';
        if Patient.Sex = 'F' then
          result := 'Female';
      end;
    if Got = Piece(lcDataTextMarkers, U, 9) then //Patient_Date_Of_Birth
      result := MixedCase(Patient.DOB);
    if Got = Piece(lcDataTextMarkers, U, 10) then //Patient_Date_Of_Death
      result := MixedCase(Patient.DOD);
    if Got = Piece(lcDataTextMarkers, U, 11) then //Patient_Age
      result := IntToStr(Patient.Age);
    if Got = Piece(lcDataTextMarkers, U, 12) then //Patient_SSN
      result := Patient.FormattedSSN;
    if Got = Piece(lcDataTextMarkers, U, 13) then //Staff_Ordered_By
      begin
        TempUser.Load(lvOrderedBy, DataModule1.RPCBroker1);
        result := MixedCase(TempUser.Name); //MixedCase(DUZToName(lvOrderedBy));
      end;
    if Got = Piece(lcDataTextMarkers, U, 14) then //Staff_Interviewed_By
      begin
        TempUser.Load(lvAdministerBy, DataModule1.RPCBroker1);
        result := MixedCase(TempUser.Name); //MixedCase(DUZToName(lvOrderedBy));
      end;
    if Got = Piece(lcDataTextMarkers, U, 15) then //Location
      result := MixedCase(LocationIndexToName(lvLocation, DataModule1.RPCBroker1));
    if Got = Piece(lcDataTextMarkers, U, 16) then //'Date_Given'
      result := lvDateGiven;
    if Got = Piece(lcDataTextMarkers, U, 17) then //'Date_Saved'
      result := lvDateSaved;
    if Got = Piece(lcDataTextMarkers, U, 18) then //'Today''s_Date'
      result := DateToStr(Now);
    if Got = Piece(lcDataTextMarkers, U, 19) then //'Today''s_Date_Time'
      result := DateTimeToStr(Now);
    if Got = Piece(lcDataTextMarkers, U, 20) then //'DLL_String'
      result := lvDLL_String;

  end;

begin
  result := '';
  TempUser := TThisUser.Create;

  TempStr := DataText;
  Delete(TempStr, 1, 2);
  Delete(TempStr, Length(TempStr) - 1, 2);

  for i := 0 to lvDataTextInUse.Count - 1 do
    if UpperCase(lvDataTextInUse[i]) = UpperCase(TempStr) then
      Result := ReplacementTable(lvDataTextInUse[i]);
  TempUser.Free;
end;



function GetQuestionID(QID: string): string;
// Input is <*Answer_100234*>: Output is 100234
var
  TempID                      : string;
begin
  result := '';

  TempID := QID;
  Delete(TempID, 1, 9);
  Delete(TempID, Length(TempID) - 1, 2);
  // TempID is now simply the Question.QuestionID
  result := TempID;
end;

// patch 108, added aInstrumentName parameter to print totals for specific reports llh
procedure TMHA_Report_OPS.ReplaceDataText(ForceWrap: Boolean; aInstrumentName: string;
  var aReportStrList: TStringList);
var
  TempStr                     : string;
  i                           : integer;
  TempStrList                 : TStringList;

  //patch 108 adding total to report when scale scores exist llh
  total                       : integer ;

  function ReplaceText(StringIn: string): string;
  var
    ii, jj                    : Integer;
    TempStr2,
      TempStrr                : string;
  begin
    result := StringIn;
    TempStrr := StringIn;
    TempStr2 := '';

    while Pos(START_TEXT, Result) > 0 do
      begin
        for ii := 1 to Length(TempStrr) - 1 do
          begin
            if ((TempStrr[ii] + TempStrr[ii + 1]) = START_TEXT) then
              begin

                for jj := 1 to Length(TempStrr) - ii do
                  begin
                    if ((TempStrr[ii + jj - 1] + TempStrr[ii + jj]) = END_TEXT) then
                      begin
                        Delete(TempStrr, ii, jj + 1);
                        Insert(Replacement('<' + TempStr2 + '>'), TempStrr, ii);
                        TempStr2 := '';
                        break;
                      end
                    else
                      TempStr2 := TempStr2 + TempStrr[ii + jj];
                  end; // for jj
                break;
              end;
          end;
        result := TempStrr;
      end; // while
  end;

  function ReplaceAnswers(StringIn: string): string;
  var
    ii, jj                    : Integer;
    TempStr2,
      TempStrr                : string;
  begin
    result := StringIn;
    TempStrr := StringIn;
    TempStr2 := '';

    while Pos(START_ANSWER, Result) > 0 do
      begin

        for ii := 1 to Length(TempStrr) - 1 do
          begin
            if ((TempStrr[ii] + TempStrr[ii + 1]) = START_ANSWER) then
              begin

                for jj := 1 to Length(TempStrr) - ii do
                  begin
                    if ((TempStrr[ii + jj - 1] + TempStrr[ii + jj]) = END_ANSWER) then
                      begin
                        Delete(TempStrr, ii, jj + 1);
                        Insert(GetAnswer(GetQuestionID('<' + TempStr2 + '>')), TempStrr, ii);
                        TempStr2 := '';
                        break;
                      end
                    else
                      TempStr2 := TempStr2 + TempStrr[ii + jj];
                  end; // for jj
                break;
              end;
          end;
        result := TempStrr;
      end; // while
  end;

  function ReplaceScores(StringIn: string): string;
  var
    ii, jj                    : Integer;
    TempStr2,
      TempStrr                : string;

    // patch 108 add total score to report when there are scale scores without using the AUX dll llh
    tempstr3                  : string;

  begin
    result := StringIn;
    TempStrr := StringIn;
    TempStr2 := '';

    while Pos(START_SCALE, Result) > 0 do
      begin
        for ii := 1 to Length(TempStrr) - 1 do
          begin
            if ((TempStrr[ii] + TempStrr[ii + 1]) = START_SCALE) then
              begin

                for jj := 1 to Length(TempStrr) - ii do
                  begin
                    if ((TempStrr[ii + jj - 1] + TempStrr[ii + jj]) = END_SCALE) then
                      begin
                        Delete(TempStrr, ii, jj + 1);
                        Delete(TempStr2, 1, 1);
                        Delete(TempStr2, Length(TempStr2), 1);
                        //patch 108 add total score to report
                        tempstr3 := GetScaleScore(TempStr2);
                        if (aInstrumentName = 'YBOCSII') or (aInstrumentName = 'SIP-2L' ) or
                           (aInstrumentName = 'CEMI') then
                           total := total + strtoint(tempstr3) ;
                        if TempStr2 = 'Total' then
                          insert(inttostr(total), TempStrr, (ii-1))
                        else
                          insert(tempstr3, TempStrr, ii);
                        //Insert(GetScaleScore(TempStr2), TempStrr, ii);    // original logic
                        //end patch 108 changes for total score llh
                        TempStr2 := '';
                        break;
                      end
                    else
                      TempStr2 := TempStr2 + TempStrr[ii + jj];
                  end; // for jj
                break;
              end;
          end;
        result := TempStrr;
      end; // while
  end;
(*
    function ReplaceDLLScores(StringIn: string): string;
  var
    ii, jj                    : Integer;
    TempStr2,
      TempStrr                : string;
  begin
    result := StringIn;
    TempStrr := StringIn;
    TempStr2 := '';

    while Pos(START_SCALE, Result) > 0 do
      begin
        for ii := 1 to Length(TempStrr) - 1 do
          begin
            if ((TempStrr[ii] + TempStrr[ii + 1]) = START_SCALE) then
              begin

                for jj := 1 to Length(TempStrr) - ii do
                  begin
                    if ((TempStrr[ii + jj - 1] + TempStrr[ii + jj]) = END_SCALE) then
                      begin
                        Delete(TempStrr, ii, jj + 1);
                        Delete(TempStr2, 1, 1);
                        Delete(TempStr2, Length(TempStr2), 1);
                        Insert(GetScaleScore(TempStr2), TempStrr, ii);
                        TempStr2 := '';
                        break;
                      end
                    else
                      TempStr2 := TempStr2 + TempStrr[ii + jj];
                  end; // for jj
                break;
              end;
          end;
        result := TempStrr;
      end; // while
  end;  *)

  function ReplaceScripts(StringIn: string): string;
  var
    ii, jj                    : Integer;
    TempStr2,
      TempStrr                : string;
  begin
    result := StringIn;
    if (FListOfScriptsForReport.Count = 0) then
      Exit;
    TempStrr := StringIn;
    TempStr2 := '';

    while Pos(START_SCRIPT, Result) > 0 do
      begin
        for ii := 1 to Length(TempStrr) - 1 do
          begin
            if ((TempStrr[ii] + TempStrr[ii + 1]) = START_SCRIPT) then
              begin

                for jj := 1 to Length(TempStrr) - ii do
                  begin
                    if ((TempStrr[ii + jj - 1] + TempStrr[ii + jj]) = END_SCRIPT) then
                      begin
                        Delete(TempStrr, ii, jj + 1);
                        Delete(TempStr2, 1, 1);
                        Delete(TempStr2, Length(TempStr2), 1);
                        Insert(EvaluateForScript(TempStr2), TempStrr, ii);
                        TempStr2 := '';
                        break;
                      end
                    else
                      TempStr2 := TempStr2 + TempStrr[ii + jj];
                  end; // for jj
                break;
              end;
          end;
        result := TempStrr;
      end; // while
  end;

  procedure WrapThisString(aWidth: Integer; InStr: string);
  var
    x                         : Integer;
    Str, Str1                 : string;
    Done                      : Boolean;
  begin
    x := aWidth;
    Done := False;
    Str1 := Copy(Instr, 1, x);
    Str := Copy(InStr, x + 1, (Length(InStr) - x + 1));

    while not Done do
      begin
        // Don't break inside a word
        if Str1[x] = ' ' then
          begin
            Str1 := ReplaceText(Str1);
            Str1 := ReplaceAnswers(Str1);
            Str1 := ReplaceScores(Str1);
            Str1 := ReplaceScripts(Str1);
            aReportStrList.Add(Str1);

            if Length(Str) > aWidth then
              begin
                Str1 := Copy(Str, 1, aWidth);
                Delete(Str, 1, aWidth);
                x := aWidth;
              end
            else
              begin
                Str := ReplaceText(Str);
                Str := ReplaceAnswers(Str);
                Str := ReplaceScores(Str);
                Str := ReplaceScripts(Str);
                aReportStrList.Add(Str);

                Done := True;
              end;
          end
        else
          begin
            Str := Str1[x] + Str;
            Delete(Str1, x, 1);
            Dec(x);
          end;
      end; // While
  end;

begin
  if aReportStrList.text = '' then
    exit;

  //patch 108 adding total to report when scale scores exist llh
  total := 0 ;
  TempStrList := TStringList.Create;
  TempStrList.Assign(aReportStrList);

  aReportStrList.Clear;

  for i := 1 to TempStrList.Count - 1 do
    begin

      TempStr := TempStrList[i];

      if TempStr = '' then
        Continue;

      //TempStr := StringReplace(TempStr, '   ', ' ', [rfReplaceAll, rfIgnoreCase]);
      //TempStr := StringReplace(TempStr, '  ', ' ', [rfReplaceAll, rfIgnoreCase]);

      if ForceWrap and (Length(TempStr) > 80) then
        begin
          WrapThisString(80, TempStr);
        end
      else
        begin
          Tempstr := ReplaceText(TempStr);
          Tempstr := ReplaceAnswers(TempStr);
          Tempstr := ReplaceScores(TempStr);
          Tempstr := ReplaceScripts(TempStr);

          aReportStrList.Add(TempStr);
        end;
    end;
  TempStrList.Free;
end;

//llh troubleshooting the IMRA

function TMHA_Report_OPS.GetScaleScore(AScaleName: string): string;
// AScaleName has to be just the name
var
  Score,
    i, j, k, m                : Integer;
  TempStr,
    // patch 129 llh
    TmpStr2                  : string;

begin
  Result := 'X';
  Score := 0;

  if FListOfScalesForReport.Count = 0 then
    Exit;

  for i := 0 to FListOfScalesForReport.Count - 1 do
    begin
      if AScaleName = TScale(FListOfScalesForReport[i]).ScaleName then
        begin
          for j := 0 to FListOfKeysForReport.Count - 1 do
            begin
              if TScale(FListOfScalesForReport[i]).IEN =
                TKey(FListOfKeysForReport[j]).ScaleID then
                begin
                  for k := 0 to FListOfAnswersForReport.Count - 1 do
                    begin
                      if TAnswer(FListOfAnswersForReport[k]).QuestionID =
                        TKey(FListOfKeysForReport[j]).QuestionID then
                        begin

                          TempStr := TAnswer(FListOfAnswersForReport[k]).AnswerText;

                          // Out-of-the-box, AnswerText is the ChoiceId of the
                          // multiple-choice item.
                          // Convert to its text so the Key.Value can be compared.
                          for m := 0 to FListOfQuestionsForReport.Count - 1 do
                            begin
                              if (TQuestion(FListOfQuestionsForReport[m]).QuestionID =
                                TAnswer(FListOfAnswersForReport[k]).QuestionID) and
                                (TQuestion(FListOfQuestionsForReport[m]).ResponseType =
                                rtMultiChoice) then
                                TempStr :=
                                  GetAnswerText(TQuestion(FListOfQuestionsForReport[m]).MultiChoice.Choices,
                                  TAnswer(FListOfAnswersForReport[k]).AnswerValue);
                            end;

                            // llh -- internal comment remove, this compares the Choice IEN text
                            //        with the Key text and then add the value
                          if TempStr =
                            TKey(FListOfKeysForReport[j]).AnswerText then
                            Score := Score + StrToInt(TKey(FListOfKeysForReport[j]).Value);

                       end;
                    end; // for k
                end; // if
            end; // for j
        end; // if
    end; // for i
  Result := IntToStr(Score);
end;

function TMHA_Report_OPS.IsQuestionMultiChoice(QID: string): integer;
var
  i                           : Integer;
begin
  result := -1;
  for i := 0 to FListOfQuestionsForReport.Count - 1 do
    begin
      if TQuestion(FListOfQuestionsForReport[i]).QuestionID = QID then
        begin
          if TQuestion(FListOfQuestionsForReport[i]).ResponseType = rtMultiChoice then
            result := i;
          break;
        end;
    end;
end;

function TMHA_Report_OPS.GetAnswer(QID: string): string;
// For multiple choice questions, the result will be the text of the
// item, less designator.
var
  i, x                        : integer;
begin
  Result := '';

  for i := 0 to FListOfAnswersForReport.Count - 1 do
    begin
      with TAnswer(FListOfAnswersForReport[i]) do
        begin
          if QuestionID = QID then
            begin
              // Default answer, so far
              Result := AnswerText;

              // Vista can not handle #13, so #124 is substituted.
              if Pos(CARRIAGE_RETURN_MARKER, Result) > 0 then
                Result := StringReplace(AnswerText, CARRIAGE_RETURN_MARKER,
                  CRLF + lcSpace, [rfReplaceAll]);

              if AnswerValue = CHOICE_ID_OF_SKIPPED_BY_RULE_QUESTION then
                begin
                  Result := SKIPPED_BY_RULE;
                  if lvInstrumentName = 'AUDC' then
                    Result := 'Not asked (patient reports no drinking in past year)';
                  Exit;
                end;
              if AnswerValue = CHOICE_ID_OF_SKIPPED_QUESTION then
                begin
                  Result := SKIPPED;
                  Exit;
                end;
              if AnswerValue = CHOICE_ID_OF_SKIPPED_REQUIRED_QUESTION then
                begin
                  Result := SKIPPED_BUT_REQUIRED;
                  Exit;
                end;
              if AnswerValue = BLANK_DATE then
                begin
                  Result := '';
                  Exit;
                end;

              if AnswerValue = BAD_DATA then
                begin
                  Result := 'Not specified';
                  Exit;
                end;

              x := IsQuestionMultiChoice(QID);

              if x > -1 then // Yes, is multichoice
                begin
                  Result := GetAnswerText(
                    TQuestion(FListOfQuestionsForReport[x]).MultiChoice.Choices,
                    AnswerValue);
                end;
            end;
        end;
    end;
end;


function TextAnswer(TextIn, ScriptIn, OperationIn: string): Boolean;
begin
  Result := false;

  if (OperationIn = Piece(OPERATOR_STRINGS, U, 1)) and
    (UpperCase(TextIn) = UpperCase(ScriptIn)) then
    Result := true;

  if (OperationIn <> Piece(OPERATOR_STRINGS, U, 1)) and
    (UpperCase(TextIn) <> UpperCase(ScriptIn)) then
    Result := true;
end;


function MultiChoiceAnswer(ChoiceIENS, ScriptIENS, OperationIn: string):
  Boolean;
begin
  Result := false;

  if (OperationIn = Piece(OPERATOR_STRINGS, U, 1)) and
    (ChoiceIENS = ScriptIENS) then
    Result := true;

  if (OperationIn <> Piece(OPERATOR_STRINGS, U, 1)) and
    (ChoiceIENS <> ScriptIENS) then
    Result := true;
end;


function CurrencyAnswer(CurrencyIn, ScriptIn, OperationIn: string): Boolean;
begin
  Result := false;

  Delete(ScriptIn, 1, 1); // remove $

  if (OperationIn = Piece(OPERATOR_STRINGS, U, 1)) and
    (CurrencyIn = ScriptIn) then
    Result := true;
  if (OperationIn = Piece(OPERATOR_STRINGS, U, 2)) and
    (CurrencyIn <> ScriptIn) then
    Result := true;
  if (OperationIn = Piece(OPERATOR_STRINGS, U, 3)) and
    (StrToIntDef(CurrencyIn, 0) < StrToIntDef(ScriptIn, 0)) then
    Result := true;
  if (OperationIn = Piece(OPERATOR_STRINGS, U, 4)) and
    (StrToIntDef(CurrencyIn, 0) > StrToIntDef(ScriptIn, 0)) then
    Result := true;
  if (OperationIn = Piece(OPERATOR_STRINGS, U, 5)) and
    (StrToIntDef(CurrencyIn, 0) <= StrToIntDef(ScriptIn, 0)) then
    Result := true;
  if (OperationIn = Piece(OPERATOR_STRINGS, U, 6)) and
    (StrToIntDef(CurrencyIn, 0) >= StrToIntDef(ScriptIn, 0)) then
    Result := true;

end;


function IntegerAnswer(IntegerIn, ScriptIn, OperationIn: string): Boolean;
begin
  Result := false;

  if (OperationIn = Piece(OPERATOR_STRINGS, U, 1)) and
    (IntegerIn = ScriptIn) then
    Result := true;
  if (OperationIn = Piece(OPERATOR_STRINGS, U, 2)) and
    (IntegerIn <> ScriptIn) then
    Result := true;
  if (OperationIn = Piece(OPERATOR_STRINGS, U, 3)) and
    (StrToIntDef(IntegerIn, 0) < StrToIntDef(ScriptIn, 0)) then
    Result := true;
  if (OperationIn = Piece(OPERATOR_STRINGS, U, 4)) and
    (StrToIntDef(IntegerIn, 0) > StrToIntDef(ScriptIn, 0)) then
    Result := true;
  if (OperationIn = Piece(OPERATOR_STRINGS, U, 5)) and
    (StrToIntDef(IntegerIn, 0) <= StrToIntDef(ScriptIn, 0)) then
    Result := true;
  if (OperationIn = Piece(OPERATOR_STRINGS, U, 6)) and
    (StrToIntDef(IntegerIn, 0) >= StrToIntDef(ScriptIn, 0)) then
    Result := true;

end;


function DateAnswer(DateIn, ScriptIn, OperationIn: string): Boolean;
var
  TempDate                    : TDateTime;
begin
  Result := false;

  if DateIn = '' then
    exit;

  if ScriptIn = '' then
    ScriptIn := DateToStr(Now);

  try
    TempDate := StrToDate(ScriptIn);
  except
    on E: EConvertError do
      begin
        ShowMessage(E.ClassName + CRLF + E.Message);
        exit;
      end;
  end;

  try
    TempDate := StrToDate(DateIn);
  except
    on E: EConvertError do
      begin
        ShowMessage(E.ClassName + CRLF + E.Message);
        exit;
      end;
  end;

  if (OperationIn = Piece(DATE_OPERATOR_STRINGS, U, 1)) and // is same date as
    (DateIn = ScriptIn) then
    Result := true;

  if (OperationIn = Piece(DATE_OPERATOR_STRINGS, U, 2)) and // is earlier than
    (TempDate < StrToDate(ScriptIn)) then
    Result := true;

  if (OperationIn = Piece(DATE_OPERATOR_STRINGS, U, 3)) and // is later than
    (TempDate > StrToDate(ScriptIn)) then
    Result := true;

  if (OperationIn = Piece(DATE_OPERATOR_STRINGS, U, 4)) and // is before today
    (TempDate < Now) then
    Result := true;

  if (OperationIn = Piece(DATE_OPERATOR_STRINGS, U, 5)) and // is after today
    (TempDate > Now) then
    Result := true;

end;


function TMHA_Report_OPS.EvaluateThisScript(
  WhichQuestion,
  WhichAnswer,
  ScriptAnswer,
  ScriptAnswerIndex,
  ScriptOperation: string): string;
var
  i                           : Integer;
begin
  result := '';

  for i := 0 to FListOfQuestionsForReport.Count - 1 do
    begin
      if WhichQuestion = TQuestion(FListOfQuestionsForReport[i]).QuestionID then
        begin
          with TQuestion(FListOfQuestionsForReport[i]) do
            begin
              //This handles string items.
              if (ResponseType = rtString) or
                (ResponseType = rtVista) or
                (ResponseType = rtMemo) or
                (ResponseType = rtMaskEdit) then
                begin
                  // Get text for answer
                  if TextAnswer(WhichAnswer,
                    ScriptAnswer, ScriptOperation) then
                    Result := TScript(FListOfScriptsForReport[lvWhichScript]).ScriptText
                  else
                    exit;
                end;

              if ResponseType = rtDate then
                begin
                  if DateAnswer(WhichAnswer,
                    ScriptAnswer, ScriptOperation) then
                    Result := TScript(FListOfScriptsForReport[lvWhichScript]).ScriptText
                  else
                    exit;
                end;

              if ResponseType = rtCurrency then
                begin
                  if CurrencyAnswer(WhichAnswer,
                    ScriptAnswer, ScriptOperation) then
                    Result := TScript(FListOfScriptsForReport[lvWhichScript]).ScriptText
                  else
                    exit;
                end;

              if (ResponseType = rtInteger) or
                (ResponseType = rtTrackBar) then
                begin
                  if IntegerAnswer(WhichAnswer,
                    ScriptAnswer, ScriptOperation) then
                    Result := TScript(FListOfScriptsForReport[lvWhichScript]).ScriptText
                  else
                    exit;
                end;

              if (ResponseType = rtMultiChoice) or
                (ResponseType = rtCheckList) then
                begin
                  if MultiChoiceAnswer(WhichAnswer,
                    GetChoiceOnly(ScriptAnswer),
                    ScriptOperation) then
                    Result := TScript(FListOfScriptsForReport[lvWhichScript]).ScriptText;
                end;
            end; // with TQuestion(Instrument.ListOfQuestions[i])
        end; // if
    end; // for
end;


function TMHA_Report_OPS.EvaluateForScript(AScriptName: string): string;
var
  i                           : Integer;
begin
  Result := '';

  for i := 0 to FListOfScriptsForReport.Count - 1 do
    begin
      if Trim(AScriptName) =
        Trim(TScript(FListOfScriptsForReport[i]).ScriptName) then
        begin
          with TScript(FListOfScriptsForReport[i]) do
            begin
              if FirstQuestionID = '' then
                exit;

              lvWhichScript := i;

              Result := EvaluateThisScript(
                FirstQuestionID,
                GetAnswer(FirstQuestionID), // Answer Text
                FirstAnswer,
                FirstAnswerIndex,
                FirstOperation);

              // Try second condition of script
              if SecondQuestionID <> '' then
                begin
                  Result := EvaluateThisScript(
                    SecondQuestionID,
                    GetAnswer(SecondQuestionID),
                    SecondAnswer,
                    SecondAnswerIndex,
                    SecondOperation);
                end;
            end; // if
        end; // with TScript(FListOfScriptsForReport[i])
    end; // for i
end;

function TMHA_Report_OPS.LoadLegacyTestReport(aInstrumentName,
  aAdministrationIENS,
  aAdminBy,
  aOrderedBy,
  aLocation,
  aDateGiven,
  aDateSaved: string;
  aTestIENS: Integer;
  var aReportStrList: TStringList): string;

var
TempDateStr,
  YSText                      : string;

const
  ErrorMsg                    = 'Unable to retrieve the report for this administration.' + CRLF;
begin
  Result := 'ERROR';

  if (aTestIENS < 0) then
    begin
      Result := ErrorMsg + 'No TestIENS for ' +
        aInstrumentName + '.';
      Exit;
    end;

  if (aAdministrationIENS = '') then
    begin
      //Set YSText = FMDate of the test + ',' + test iens
      
      // Make sure date is okay for Vista, a time is needed
      TempDateStr := DateStrToFMDateStr(aDateSaved);
      // patch 105 - removed this code, it's causing an error when legacy administrations
      //             are loaded for reviewing the report.  llh 09/27/12
      //if POS('.', TempDateStr) = 0 then TempDateStr := TempDateStr + '.1';
      YSText := TempDateStr + ',' + IntToStr(aTestIENS);

      aReportStrList.Text := LegacyResultsReadTestReport1(Patient.DFN, YSText,
        DataModule1.RPCBroker1);
    end
  else
    begin

      aReportStrList.Text := LegacyResultsReadTestReport2(Patient.DFN,
        aAdministrationIENS, DataModule1.RPCBroker1);
    end;

  Result := 'OK';

end;


function TMHA_Report_OPS.BuildSectionHeader(ACaption: string): string;
var
  tmpLine                     : string;
begin
  tmpLine := lcMySeparator;
  Insert(ACaption, tmpLine, Length(lcSpace) + 1);
  Result := CRLF
    + copy(tmpLine, 1, Length(tmpLine) - Length(ACaption)) + CRLF + CRLF;
end;



function TMHA_Report_OPS.LoadReportTemplate(aInstrumentName,
  aAdministrationIENS,
  aAdminBy,
  aOrderedBy,
  aLocation,
  aDateGiven,
  aDateSaved: string;
  ForceWrap, // Used for progress notes
  aWriteFullText: Boolean;
  var aReportStrList: TStringList): string;

var
  ReportID                    : string;

begin
  Result := 'ERROR';

  ReportID := '0';

  if ReadRoughReport(aInstrumentName,
    aAdminBy,
    aOrderedBy,
    aLocation,
    aDateGiven,
    aDateSaved,
    aReportStrList,
    ReportID) = 'ERROR' then
    exit;

  if ReportID = '0' then // No Report-Template
    begin
      aReportStrList.Text := LoadDefaultReport(
        aInstrumentName,
        aAdministrationIENS,
        aDateGiven,
        aDateSaved,
        aWriteFullText);
      Result := 'OK';
      exit;
    end;

  // patch 108, added aInstrumentName parameter to print totals for specific reports llh
  FinalReport(ForceWrap, aInstrumentName, aReportStrList);
   // Scripts processed.

  Result := 'OK';
end;

function TMHA_Report_OPS.GetAnswerText(ChoicesIn: TStringList; ChoiceID: string): string;
var
  i                           : Integer;
begin
  Result := '';

  for i := 0 to ChoicesIn.Count - 1 do
    begin
      if ChoiceID = Piece(ChoicesIn[i], U, 2) then
        begin
          Result := Piece(ChoicesIn[i], U, 3);
          break;
        end;
    end;
end;


function TMHA_Report_OPS.LoadDefaultReport(
  aInstrumentName,
  aAdministrationIENS,
  aDateGiven,
  aDateSaved: string;
  aWriteFullText: Boolean): string;
var
  i, j, k, score              : Integer;
  aScaleName,
    tmpQuest,
    tmpAns,
    tmpCaption                : string;

begin
  Score := 0;
  //Instrument Section
  //if (Instrument.PrintTitle = '') then
  tmpCaption := Result + aInstrumentName;
  //else
  //  tmpCaption := Result + Instrument.PrintTitle;

  Result := BuildSectionHeader('Instrument')
    + lcspace + 'Name:       ' + tmpCaption + CRLF
    + lcspace + 'Date Given: ' + aDateGiven + CRLF;

  //Patient Info Section
  Result := Result + BuildSectionHeader('Patient Information');

  Result := Result + lcspace + 'Name:    ' + Patient.Name + CRLF
    + lcspace + 'SSN:     ' + Patient.FormattedSSN + CRLF
    + lcspace + 'Gender:  ' + Patient.Sex + CRLF
    + lcspace + 'DOB:     ' + Patient.DOB + CRLF
    + lcspace + 'Age:     ' + IntToStr(Patient.Age) + CRLF + CRLF;

  // Scales and its Score
  if FListOfScalesForReport.Count > 1 then
    begin
      Result := Result + BuildSectionHeader('Scale(s)');
      // Out-of-the-box, AnswerValue is the ChoiceId of the multiple-choice item
      // Convert to its text so the Key.Value can be compared.
      for i := 0 to FListOfAnswersForReport.Count - 1 do
        begin
          if TQuestion(FListOfQuestionsForReport[i]).ResponseType = rtMultiChoice then
            TAnswer(FListOfAnswersForReport[i]).AnswerText :=
              GetAnswerText(TQuestion(FListOfQuestionsForReport[i]).MultiChoice.Choices,
              TAnswer(FListOfAnswersForReport[i]).AnswerValue);
        end;

      for i := 0 to FListOfScalesForReport.Count - 1 do
        begin
          AScaleName := TScale(FListOfScalesForReport[i]).ScaleName;

          for j := 0 to FListOfKeysForReport.Count - 1 do
            begin
              if TScale(FListOfScalesForReport[i]).IEN =
                TKey(FListOfKeysForReport[j]).ScaleID then
                begin
                  for k := 0 to FListOfAnswersForReport.Count - 1 do
                    begin
                      if ((TAnswer(FListOfAnswersForReport[k]).QuestionID) =
                        (TKey(FListOfKeysForReport[j]).QuestionID)) and
                        ((TAnswer(FListOfAnswersForReport[k]).AnswerText) =
                        (TKey(FListOfKeysForReport[j]).AnswerText)) then
                        Score := Score + StrToInt(TKey(FListOfKeysForReport[j]).Value);
                    end; // for k

                end; // if
            end; // for j
        end; // for i
      Result := Result + lcSpace + lcSpace + AScaleName + lcSpace + IntToStr(Score) + CRLF;
    end;

  Result := Result + BuildSectionHeader('Answers');

  for i := 0 to FListOfQuestionsForReport.Count - 1 do
    begin
      tmpAns := GetAnswer(TQuestion(FListOfQuestionsForReport[i]).QuestionID);

      if aWriteFullText then
        tmpQuest := TQuestion(FListOfQuestionsForReport[i]).qCaption + CRLF + lcspace + lcspace + tmpAns
      else
        tmpQuest := TQuestion(FListOfQuestionsForReport[i]).Designator + lcspace + tmpAns;

      Result := Result + lcspace + tmpQuest + CRLF;
    end;

  Result := DropTrailer(Result);
end;

// Load a 508 version of the legacy reports.

function TMHA_Report_OPS.Load508LegacyReport(aInstrumentName,
  aAdministrationIENS,
  aPatientDFN,
  aAdminBy,
  aOrderedBy, // DUZ
  aLocation, // uses Location IEN
  aDateGiven,
  aDateSaved: string;
  aTestIENS: Integer;
  aDataInLegacyFile: Boolean;
  var aReportStrList: TStringList): string;

var
  i                           : integer;
  TempStr1,
    TempStr2,
    TempStr                   : string;
    TempUser                    : TThisUser;
begin
  TempUser := TThisUser.Create;
  Result := 'ERROR';

  TempStr := '';
  // uMHA_InstrumentResults_IO
  // Output Format is: ScaleName1=ScaleScore1~ScaleName2=ScaleScore2~
  // If there are no scores (i.e., no scales), ReadScores returns a blank.
  TempStr := ReadScores(aDataInLegacyFile, aAdministrationIENS, aInstrumentName,
    aPatientDFN, DateStrToFMDateStr(aDateSaved), DataModule1.RPCBroker1);

  if TempStr = '' then
    exit;

  aReportStrList.Add('Date of Report '+ DateToStr(Date));
  aReportStrList.Add(Patient.FormattedSSN + ' ' + Patient.Name + ' Gender ' + Patient.Sex +
    ' Date of birth ' + Patient.DOB);
  aReportStrList.Add('Results of ' + aInstrumentName);
  aReportStrList.Add('Date taken ' + aDateSaved);
  aReportStrList.Add('Scales and their scores follow');
  aReportStrList.Add('');
  for i := 1 to NumberofPieces(TempStr, RECORD_END) do
    begin
      TempStr1 := Piece(TempStr, RECORD_END, i);
      TempStr2 := '';
      if POS(EQ, TempStr1) > 0 then
        begin
          TempStr2 := 'Scale ' + Piece(TempStr1, EQ, 1) + ' has raw score of ' +
            Piece(Piece(TempStr1, EQ, 2), U, 1);
          if Piece(Piece(TempStr1, EQ, 2), U, 2) <> '' then
            TempStr2 := TempStr2 + ' and a T-score of ' +
              Piece(Piece(TempStr1, EQ, 2), U, 2);
        end
      else
        begin
          TempStr2 := 'Scale ' + Piece(TempStr1, U, 1);
          if Piece(TempStr1, U, 2) <> '' then
            TempStr2 := Tempstr2 + ' and a T score of ' +
              Piece(TempStr1, U, 2);
        end;

      aReportStrList.Add(TempStr2);
    end;
    
  aReportStrList.Add('');
  aReportStrList.Add('This instrument is copyrighted, all rights reserved.');
  aReportStrList.Add('Not valid unless signed: Reviewed by ____________________');
  aReportStrList.Add('Printed by: ' + ThisUser.Name);
  TempUser.Load(aOrderedBy, DataModule1.RPCBroker1);
  aReportStrList.Add('Ordered by: ' + MixedCase(TempUser.Name));
  aReportStrList.Add('');
  aReportStrList.Add('Questions and Answers follow');

  for i := 0 to ListOfQuestionsForReport.Count - 1 do
    begin
      TempStr := TQuestion(MHA_Report_OPS.ListOfQuestionsForReport[i]).Designator +
        ' ' +
        GetAnswer(TQuestion(MHA_Report_OPS.ListOfQuestionsForReport[i]).QuestionID);

      aReportStrList.Add(TempStr);
    end;

  TempUser.Free;
  Result := OK;
end;

// Load normal report, no special 508 stuff

function TMHA_Report_OPS.LoadReport(aInstrumentName,
  aAdministrationIENS,
  aAdminBy,
  aOrderedBy,
  aLocation, // uses Location IEN
  aDateGiven,
  aDateSaved: string;
  aTestIENS: Integer;
  ForceWrap, // Used for progress notes
  aIsLegacy,
  aWriteFullText: Boolean;
  aDLLString: string;
  var aReportStrList: TStringList): string;

begin

  lvDLL_String := aDLLString; // From YS_MHA_AUX.dll; usually blank

  // Instrument administered through roll-and-scroll or MHA2
  if aAdministrationIENS = '' then
    Result := LoadLegacyTestReport(
      aInstrumentName,
      aAdministrationIENS,
      aAdminBy,
      aOrderedBy,
      aLocation,
      aDateGiven,
      aDateSaved,
      aTestIENS,
      aReportStrList)
  else
  // Instrument administered through MHA3
    begin
      if (not aIsLegacy) then
        Result := LoadReportTemplate(
          aInstrumentName,
          aAdministrationIENS,
          aAdminBy,
          aOrderedBy,
          aLocation,
          aDateGiven,
          aDateSaved,
          ForceWrap,
          aWriteFullText,
          aReportStrList);

      // NOTE: llh -- although don't know why but if its a legacy instrument the WriteFullText field
      //              is not used, all questions text will be written in report.
      if (aIsLegacy) then
        Result := LoadLegacyTestReport(
          aInstrumentName,
          aAdministrationIENS,
          aAdminBy,
          aOrderedBy,
          aLocation,
          aDateGiven,
          aDateSaved,
          aTestIENS,
          aReportStrList);
    end;

  if Result = 'ERROR' then
    exit;

  aReportStrList.text := aReportStrList.Text +
    AddCommentsToReport(aAdministrationIENS);

end;


function TMHA_Report_OPS.SaveComments(aAdminID, aComment: string): string;
begin
  Result := WriteComments(aAdminID, aComment, DataModule1.RPCBroker1);
end;

procedure TMHA_Report_OPS.LoadComments(AdminIENS: string);
begin

  lvComments := ReadComments(AdminIENS, DataModule1.RPCBroker1);

  if (lvComments = BAD_DATA) then
    lvComments := '';
end;


function TMHA_Report_OPS.AddCommentsToReport(AdminIENS: string): string;
begin

  if (AdminIENS = '') then
    begin
      Result := '';
      Exit;
    end;

  LoadComments(AdminIENS);

  if lvComments <> '' then
    begin
      Result := Result + CRLF + BuildSectionHeader('Comments') + lvComments;
      Result := StringReplace(Result, RECORD_END, CRLF, [rfReplaceAll]);
    end;

end;


initialization
  MHA_Report_OPS := TMHA_Report_OPS.Create;

finalization
  MHA_Report_OPS.Free;


end.

