unit uMHA_Rule_IO;

interface

uses
  Trpcb,
  Classes,
  uIA_Instrument;

function ReadRules(var aInstrument: TInstrument): Boolean;


implementation

uses
  SysUtils,
  dialogs,
  uIA_Const,
  uMHA_DM,
  uIA_Utils,
  uIA_Question,
  uIA_Rule;

const
  NO_RULES                    : string = 'No Rules';


function ReadRulesInVista(aInstrumentName: string): string;
var
  tmpRules, tmpDelimiter      : string;
  i                           : integer;
begin
  tmpRules := '';


  with DataModule1.RPCBroker1 do
    begin
      ClearParameters := True;
      ClearResults := True;
      CreateContext('YS BROKER1');
      RemoteProcedure := 'YTQ RULES';
      Param[0].Mult['"CODE"'] := aInstrumentName;
      Param[0].PType := List;
      Call;

      if not (Results[0] = DATA) then
        begin
          Result := BAD_DATA;
          Exit;
        end; //bad data

      for i := 1 to Results.Count - 1 do
        begin
          if Results[1] = NO_RULES then
            begin
              Result := NO_RULES;
              Exit;
            end;
          if ((i mod 3) = 0) then
            tmpDelimiter := RECORD_END
          else
            tmpDelimiter := U;
          tmpRules := tmpRules + Results[i] + tmpDelimiter;
        end;
    end; //with DataModuleEdit.RPCBroker1

  Result := DropTrailer(tmpRules);

end;

function ConvertVistaDataToRuleObject(var aInstrument: TInstrument): Boolean;
var
  j                           : Integer;
  TempStr                     : string;
  TempList                    : TStringList;
  ARule                       : TRule;

  function MyStrToBoolOp(ABoolOperatorString: string): TBoolOperatorType;
  begin
    Result := boAND; //the default value
    if (ABoolOperatorString = 'OR') then
      Result := boOR;
  end; //MyStrToBoolOp

  function MyStrToOperator(AOperatorString: string): TOperatorType;
  var
    tmpFirst                  : integer;
    tmpString                 : string;
  begin
    //toEQ, toLT, toLE, toGT, toGE, toNE
   //  OPERATOR_STRINGS = 'Equals^Does not equal^Is less than^Is greater than^'
   //    + 'Equals or is less than^Equals or is greater than';

    tmpString := AOperatorString;
    tmpFirst := PieceIndex(OPERATOR_STRINGS, tmpString, U);

    case tmpFirst of
      2: Result := toNE;
      3: Result := toLT;
      4: Result := toGT;
      5: Result := toLE;
      6: Result := toGE;
    else
      Result := toEQ;
    end; //case

  end; //MyStrToSecondOp

  function MyStrToDataType(ADataTypeString: string): TValueDataType;
  begin
    //dtString, dtDate, dtNumber
    Result := dtString;
    if (ADataTypeString = DATE_STR) then
      Result := dtDate;
    if (ADataTypeString = NUMBER_STR) then
      Result := dtNumber;
    if (ADataTypeString = INTERVAL_STR) then
      Result := dtTimeInterval;
  end; //MyStrToDataType

begin
  Result := True;
  TempList := TStringList.Create;

  PiecesToList(ReadRulesInVista(aInstrument.Name), RECORD_END, TempList);

// Raw data from Vista looks like:
//  100153=101620^STRING^Equals^0^AND^^STRING^Equals^SELF^^
//  100154=101620^STRING^Equals^1^AND^^STRING^Equals^SELF^^
//  100155=101620^STRING^Equals^2^AND^^STRING^Equals^SELF^^
//  100156=101620^STRING^Equals^3^AND^^STRING^Equals^SELF^^
//  100157=101620^STRING^Equals^4^AND^^STRING^Equals^SELF^^
//  100158=101620^STRING^Equals^5^AND^^STRING^Equals^SELF^^
//  100160=101642^STRING^Equals^1^AND^^STRING^Equals^SELF^^
//  100159=101703^STRING^Equals^0^AND^^STRING^Equals^SELF^^

//   100145=101340^STRING^Equals^0^AND^101342^DATE^Equals^07/20/2005^This is my message.^YES
//
//   Decode:     100145 is Rule number
//           {1} 101340 is QuestionID (A combo box in this case)
//           {2} STRING is Answer type (Item index for combo box)
//           {3} Equals is relation
//           {4} 0 is answer, first item in this combo box
//           {5} AND is relational operation between first & second condition
//           {6} 101342 is QuestionID for second question (if blank, no 2nd question)
//           {7} DATE is answer type (String, number, date or time interval are possible)
//           {8} Equals is relation
//           {9} 07/20/2005 is Answer, in this case a date
//           {10} This is my message. is message string
//           {11} YES toggle for IsConsistencyCheck : Not used

  // This catches the "No rules" condition
  if (TempList[0] = NO_RULES) then
    begin
      Result := true;
      TempList.Free;
      Exit;
    end;

  if (TempList[0] = BAD_DATA) then
    begin
      Result := False;
      TempList.Free;
      Exit;
    end;

  for j := 0 to TempList.Count - 1 do
    begin
      TempStr := Piece(TempList[j], EQ, 2);

      ARule := TRule.Create;
      ARule.ID := Piece(TempList[j], EQ, 1);
      ARule.FirstQuestionID := Piece(TempStr, U, 1);
      ARule.FirstValueDataType := MyStrToDataType(Piece(TempStr, U, 2));
      ARule.FirstOperator := MyStrToOperator(Piece(TempStr, U, 3));
      ARule.FirstValue := Piece(TempStr, U, 4);
      ARule.BoolOperator := MyStrToBoolOp(Piece(TempStr, U, 5));
      ARule.SecondQuestionID := Piece(TempStr, U, 6);
      ARule.SecondValueDataType := MyStrToDataType(Piece(TempStr, U, 7));
      ARule.SecondOperator := MyStrToOperator(Piece(TempStr, U, 8));
      ARule.SecondValue := Piece(TempStr, U, 9);
      ARule.MessageText := Piece(TempStr, U, 10);

      aInstrument.ListOfRules.Add(ARule);
    end; // for j

  TempList.Free;
end;



function ReadQuestionsToSkip(WhichInstrument: string):
  string;
var
  Skip                        : string;
  icount                      : integer;
begin
  Skip := '';

  with DataModule1.RPCBroker1 do
    begin
      ClearParameters := True;
      ClearResults := True;
      CreateContext('YS BROKER1');
      RemoteProcedure := 'YTQ SKIP';
      Param[0].Mult['"CODE"'] := WhichInstrument;
      Param[0].PType := List;
      Call;

      if not (Results[0] = DATA) then
        begin
          Result := BAD_DATA;
          Exit;
        end; //bad data

      for icount := 1 to Results.Count - 1 do
        Skip := Skip + Results[icount] + RECORD_END;

    end; //with DataModuleEdit.RPCBroker1

  Result := DropTrailer(Skip);

  // Skip looks like this:
  //  15^84~
  //  15^85~
  //  Where 15 is the rule ID and 84 and 85 are the questionID to skip
end;


function ReadRules(var aInstrument: TInstrument): boolean;
var
  i, j                        : Integer;
  TempStr,
    TempSkip                  : string;
begin
  Result := ConvertVistaDataToRuleObject(aInstrument);

  // Place the questions that will be skipped in the rule object
  if Result then
    begin

      TempSkip := ReadQuestionsToSkip(aInstrument.Name);
      if TempSkip = '' then
        Exit;
      for i := 0 to aInstrument.ListOfRules.Count - 1 do
        begin
          for j := 1 to NumberOfPieces(TempSkip, RECORD_END) do
            begin
              TempStr := Piece(TempSkip, RECORD_END, j);
              if TRule(aInstrument.ListOfRules[i]).ID =
                Piece(TempStr, U, 1) then
                TRule(aInstrument.ListOfRules[i]).SkipQuestionIDs :=
                  TRule(aInstrument.ListOfRules[i]).SkipQuestionIDs +
                  Piece(TempStr, U, 2) + U;
            end; // for j
        end; // for i
    end; // result
end;

end.

