unit uMHA_UploadResultsController;

interface

uses
  SysUtils,
  dialogs,
 uIA_Patient,
  Classes;

type
  TMatchConfidenceLevel = (mlLow, mlGood, mlHigh);

type
  TUploadResultsController = class
    MatchConfidenceLevel: TMatchConfidenceLevel;
  private
    FAdministrations: TStringList;
    FNumberOfAdministrationFiles: Integer;
    FTestName: string;
    FTestDate: string;
    FNumberOfPatientsFound: Integer;
    FPatientSearchResults: TStringList;
    FCPRSPatientName: string;
    FCPRSPatientDFN: string;
    function PatientSearchSSNByName(AName: string): string;
    function PatientSearchDOBByName(AName: string): string;
    function PatientSearchSexByName(AName: string): string;
    function PatientSearchDFNByName(AName: string): string;
  protected
  public
    property NumberOfAdministrationFiles: Integer read
      FNumberOfAdministrationFiles write FNumberOfAdministrationFiles;
    property TestName: string read FTestName write FTestName;
    property TestDate: string read FTestDate write FTestDate;
    property NumberOfPatientsFound: Integer read FNumberOfPatientsFound write
      FNumberOfPatientsFound;
    property CPRSPatientName: string read FCPRSPatientName write
      FCPRSPatientName;
    property CPRSPatientDFN: string read FCPRSPatientDFN write FCPRSPatientDFN;

    destructor Destroy; override;
    constructor Create;
    // Used by UploadResults
    procedure LoadAdministrationFileNames;
    function PatientSearchNameByIndex(AIndex: integer): string;
    procedure LoadByIndex(APatient: TPatient; AIndex: integer);
    procedure SearchForPatients(ASearchString: string);
    procedure LoadMatchConfidenceLevel(APatient: TPatient; BPatient: TPatient);
    procedure LoadSearchPatient(var APatient: TPatient; AName: string);
    procedure DeleteLocalFiles(aDFN: string);
    function LoadOffLineAdministration(aDFN, APatientID: string): BOOLEAN;
    function LoadOffLineAnswers(aDFN: string): BOOLEAN;
    function SaveOffLineInVista(aDFN: string): BOOLEAN;
  published
  end; { TUploadResultsController }

var
  UploadResultsController     : TUploadResultsController;

implementation

uses
  uIA_Utils,
  uIA_Const,
  uMHA_OffLinePatient_List,
  uMHA_LocalFiles,
  uMHA_Administration,
  uIA_Instrument,
  uIA_Question,
  uIA_Answers,
  //uMHA_Answers_IO,
  uMHA_Administration_IO,
  uMHA_UploadResults_IO,
  Windows;

var
  lvAdministration            : TAdministration;
  lvInstrument                : TInstrument;

destructor TUploadResultsController.Destroy;
begin
  if OffLinePatient_List <> nil then
    OffLinePatient_List.Free;

  if lvAdministration <> nil then
    lvAdministration.Free;

  if lvInstrument <> nil then
    lvInstrument.Free;
  if FPatientSearchResults <> nil then
    FPatientSearchResults.Free;
  if FAdministrations <> nil then
    FAdministrations.Free;
  inherited Destroy;
end;

constructor TUploadResultsController.Create;
begin
  inherited Create;
  FAdministrations := TStringList.Create;
  FPatientSearchResults := TStringList.Create;
  lvInstrument := TInstrument.Create(nil);
  lvAdministration := TAdministration.Create;
  OffLinePatient_List := TOffLinePatient_List.Create;
end;

procedure TUploadResultsController.LoadAdministrationFileNames;
var
  i                           : integer;
begin
  PiecesToList(ReadOffLineAdministrations, '$', FAdministrations);
  //ReadOffLineAdministrations returns filenames of all the
  //off-line administration files
  //1~3031023-151753~CAGE.adm where Piece 1 = DFN, 2 = FMDate (kind of),
  //and 3 = test

  if FAdministrations.Count = 0 then
    FNumberOfAdministrationFiles := 0
  else
    begin
      for i := 0 to FAdministrations.Count - 1 do
        FAdministrations[i] := StringReplace(FAdministrations[i], '.' +
          ADMIN_FILE_EXT, '', [rfReplaceAll]);
      FNumberOfAdministrationFiles := FAdministrations.Count;
    end;

end;

procedure TUploadResultsController.LoadByIndex(APatient: TPatient; AIndex:
  integer);
var
  tmpString                   : string;
begin
  if (AIndex < 0) or (AIndex > FAdministrations.Count) then
    exit;

  tmpString := FAdministrations[AIndex];

  FTestDate := FileNameDateTimeToDateTimeString(Piece(tmpString,
    FILE_NAME_DELIMITER, 2));
  FTestName := Piece(tmpString, FILE_NAME_DELIMITER, 3);

  tmpString := Piece(tmpString, FILE_NAME_DELIMITER, 1);
  OffLinePatient_List.LoadPatientByDFN(APatient, tmpString);
end;

procedure TUploadResultsController.SearchForPatients(ASearchString: string);
var
  tmpResult, tmpCase, tmpName : string;
  i                           : integer;
begin
  //10052970^RABBIT,VICTOR^6/26/1947^123-45-6789^Sex~
  //becomes
  //RABBIT, VICTOR=6/26/1947^123-45-6789^10052970^Sex
  FPatientSearchResults.Clear;

  tmpResult := PatientSearch(ASearchString, 100);

  if tmpResult = 'TO MANY RESULTS' then
    begin
      MessageBox(0,
        'Please refine your search criteria by adding one or more characters to the text ' +
        'for which you are searching. Then try again.',
        'Search returned too many results', MB_ICONSTOP or MB_OK);
      FNumberOfPatientsFound := 0;
      exit;
    end;

  if not (tmpResult = BAD_DATA) then
    for i := 1 to NumberOfPieces(tmpResult, RECORD_END) do
      begin
        tmpCase := Piece(tmpResult, RECORD_END, i);
        tmpName := Piece(tmpCase, U, 2);
      //Insert a space between last and first names to match OffLinePatient.Name
        tmpName := Trim(Piece(tmpName, ',', 1)) + ', ' + Trim(Piece(tmpName,
          ',', 2));
        FPatientSearchResults.Add(tmpName + '=' + Pieces(tmpCase, U, 3, 5) + U
          + Piece(tmpCase, U, 1));
      end;

  FNumberOfPatientsFound := FPatientSearchResults.Count;

end;

function TUploadResultsController.PatientSearchNameByIndex(AIndex: integer):
  string;
begin
  Result := FPatientSearchResults[AIndex];
end;

function TUploadResultsController.PatientSearchSSNByName(AName: string): string;
begin
  //RABBIT, VICTOR=4/20/1947^123-45-6789^10052970^sex
  Result := Piece(FPatientSearchResults.Values[AName], U, 2);
end;

function TUploadResultsController.PatientSearchDOBByName(AName: string): string;
begin
  Result := Piece(FPatientSearchResults.Values[AName], U, 1);
end;

function TUploadResultsController.PatientSearchSexByName(AName: string): string;
begin
  Result := Piece(FPatientSearchResults.Values[AName], U, 3);
end;

function TUploadResultsController.PatientSearchDFNByName(AName: string): string;
begin
  Result := Piece(FPatientSearchResults.Values[AName], U, 4);
end;

procedure TUploadResultsController.LoadMatchConfidenceLevel(APatient: TPatient;
  BPatient: TPatient);
var
  TempStrA, TempStrB          : string;
begin

  TempStrA := Trim(UpperCase(APatient.Sex));
  TempStrB := Trim(UpperCase(BPatient.Sex));

  MatchConfidenceLevel := mlLow;

  if (APatient.Name = BPatient.Name)
    and (APatient.SSN = BPatient.SSN)
    and (APatient.DOB = BPatient.DOB)
    and (APatient.Sex = BPatient.Sex) then
    begin
      MatchConfidenceLevel := mlHigh;
      Exit;
    end;

  if (APatient.Name = BPatient.Name)
    or (APatient.SSN = BPatient.SSN) then
    MatchConfidenceLevel := mlGood;

  if (Length(TempStrA) > 0) and (Length(TempStrB) > 0) then
    if (TempStrA[1] <> TempStrB[1]) then
      MatchConfidenceLevel := mlLow;

end;

procedure TUploadResultsController.LoadSearchPatient(var APatient: TPatient;
  AName: string);
begin
  APatient.Name := AName;
  APatient.FormattedSSN := PatientSearchSSNByName(AName);
  APatient.SSN := StringReplace(APatient.FormattedSSN, '-', '', [rfReplaceAll]);
  APatient.DFN := PatientSearchDFNByName(AName);
  APatient.DOB := PatientSearchDOBByName(AName);
  APatient.Sex := PatientSearchSexByName(AName);
end;

procedure TUploadResultsController.DeleteLocalFiles(aDFN: string);
begin
  DeleteLocalAdministrationAndAnswerFiles(aDFN, lvAdministration.ID);
end;

function TUploadResultsController.LoadOffLineAdministration(aDFN, APatientID:
  string): BOOLEAN;
// aDFN is the temporary, arbitrary number used for off-line identification. It
// is not the PatientID used in Vista.
var
  tmpFMDate                   : string;
  tmpDateTime                 : TDateTime;
  tmpList                     : TStringList;
begin
  Result := False;
  tmpDateTime := StrToDateTime(FTestDate);
  tmpFMDate := DateTimeToFileNameDateTime(tmpDateTime);
  LocalFiles.LoadAdministrationFileName(aDFN, tmpFMDate, FTestName);

  if not FileExists(LocalFiles.AdministrationFileName) then
    Exit;

  tmpList := TStringList.Create;
  tmpList.LoadFromFile(LocalFiles.AdministrationFileName);

  with lvAdministration do
    begin
      ID := tmpList.Values['AdministrationID'];
      InstrumentName := tmpList.Values['InstrumentName'];
      DateGiven := StrToDateTime(tmpList.Values['DateGiven']);
      DateSaved := StrToDateTime(tmpList.Values['DateSaved']);
      Signed := StrToBool(tmpList.Values['Signed']);
      IsComplete := StrToBool(tmpList.Values['IsComplete']);
      NumberAnswered :=
        StrToIntDef(tmpList.Values['NumberOfQuestionsAnswered'], 0);
    // Orderer.DUZ set in fMHA3_UploadAdmin
    // AdministeredBy set in fMHA3_UploadAdmin
    // Location set in fMHA3_UploadAdmin
      PatientID := APatientID;
    end;

  tmpList.Free;

  Result := True;
end;

function TUploadResultsController.LoadOffLineAnswers(aDFN: string): BOOLEAN;
var
  i                           : Integer;
  aAnswer                     : TAnswer;
  TempList                    : TStringList;
  TempStr                     : string;
begin
  Result := False;

  if not FileExists(LocalFiles.AnswerFileName(aDFN, lvAdministration.ID)) then
    Exit;

  TempList := TStringList.Create;
  TempList.LoadFromFile(LocalFiles.AnswerFilename(aDFN,
    lvAdministration.ID));
  lvAdministration.ListOfAnswers.Clear;

  for i := 0 to TempList.Count - 1 do
    begin
      aAnswer.Create;
      TempStr := TempList[i];

      with aAnswer do
        begin
          //AnswerID  :=
          QuestionID := Piece(TempStr, U, 1);
          //AnswerVUID   :=
          AnswerValue := Piece(TempStr, U, 2);
          AnswerText := Piece(TempStr, U, 3);
          //Designator  :=
          LegacyValue := Piece(TempStr, U, 4);
          IsMultiChoice := StrToBool(Piece(TempStr, U, 5));
        end;

      lvAdministration.ListOfAnswers.Add(aAnswer);
    end;

  TempList.Free;
  Result := lvAdministration.ListOfAnswers.Count > 0;
end;

function TUploadResultsController.SaveOffLineInVista(aDFN: string): BOOLEAN;
var
  FOffLineID, tmpQuestionID, ParamStr: string;
  i                           : integer;
begin
  Result := False;


//**
(*
  if not Instrument.Load(Administration.InstrumentName) then
    Exit;

  //This dance is necessary so a valid Administration.ID will be created
  //in VistA
  FOffLineID := Administration.ID;
  Administration.ID := NO_ID;
  //Administration.OrdererDUZForNewAdministrations := Administration.Orderer.DUZ;

  if Administration.SaveNew then // This sets the new Administration.ID
  begin

    ParamStr := Administration.ID + U //1
    + Administration.AdministeredBy + U //2
    + BoolToStr(Administration.Signed) + U //3
    + BoolToStr(Administration.IsComplete) + U //4
    + IntToStr(Administration.NumberOfQuestionsAnswered) + U //5
    + Administration.InstrumentName + U //6
    + DateTimeToStr(Administration.DateGiven) + U //7
    + Administration.PatientID; //8

    AdministrationWriteUpdatedProperties(ParamStr);

    for i := 0 to Administration.NumberOfQuestionsAnswered - 1 do
    begin
      tmpQuestionID := Answers.AnswerList.Names[i];
      Question.Load(tmpQuestionID);
      ParamStr := tmpQuestionID + U
        + Administration.ID + U
        + Administration.PatientID + U
        + 'FALSE^' //IsASI
      + BoolToStr(Question.ResponseType = rtMultiChoice);

      //Do not use Answers.WriteAnwer here because
      // Answer is already converted
      AnswerWrite(ParamStr, Answers.AnswerList.Values[tmpQuestionID]);
    end; //for i...

    Administration.Close;
    Administration.ID := FOffLineID;
    DeleteLocalFiles(aDFN);
    Result := True;
  end; //SaveNew
*)
end;

end.

