unit uMHA_UploadResults_OPS;

interface

uses
  SysUtils,
  Windows,
  fIA_dialogs, Dialogs,
  Contnrs,
  Classes,
  uMHA_Administration,
  uIA_Patient;

type
  TMatchConfidenceLevel = (mlLow, mlGood, mlHigh);

type
  TUploadResults_OPS = class
    //MatchConfidenceLevel: TMatchConfidenceLevel;
  private
    FListOfOffLineFiles: TObjectList;
    FListOfOffLinePatients: TObjectList;
    FMatchConfidenceLevel: TMatchConfidenceLevel;

    FNumberOfPatientsFound: Integer;
    FPatientSearchResults: TStringList;
    FCPRSPatientName: string;
    FCPRSPatientDFN: string;

    function PatientSearchSSNByName(AName: string): string;
    function PatientSearchDOBByName(AName: string): string;
    function PatientSearchSexByName(AName: string): string;
    function PatientSearchDFNByName(AName: string): string;
  protected
  public
    property ListOfOffLineFiles: TObjectList read FListOfOffLineFiles write FListOfOffLineFiles;
    property ListOfOffLinePatients: TObjectList read FListOfOffLinePatients write FListOfOffLinePatients;
    property MatchConfidenceLevel: TMatchConfidenceLevel read FMatchConfidenceLevel write FMatchConfidenceLevel;
    property NumberOfPatientsFound: Integer read FNumberOfPatientsFound write
      FNumberOfPatientsFound;
    property CPRSPatientName: string read FCPRSPatientName write
      FCPRSPatientName;
    property CPRSPatientDFN: string read FCPRSPatientDFN write FCPRSPatientDFN;

    destructor Destroy; override;
    constructor Create;

    // Used by UploadResults
    procedure LoadOffLineFiles;
    function PatientSearchNameByIndex(AIndex: integer): string;
    procedure SearchForPatients(ASearchString: string);
    procedure LoadMatchConfidenceLevel(APatient: TPatient; BPatient: TPatient);
    procedure LoadSearchPatient(var APatient: TPatient; AName: string);

    function SaveOffLineInVista(aDFN, aDate: string; aAdmin: TAdministration): BOOLEAN;
    procedure LoadOffLinePatients;
    procedure LoadOffLineAnswers(aDFN: string;
      var aAdmin: TAdministration);
    procedure LoadOffLineAdministration(aDFN,
      aDate, aInstrument, aPatientID: string; var aAdmin: TAdministration);
  end; { TUploadResults_OPS }

var
  UploadResults_OPS           : TUploadResults_OPS;

implementation

uses
  uIA_Utils,
  uIA_Const,
  uMHA_LocalFiles,
  uIA_Instrument,
  uIA_Question,
  uIA_Answer,
  uMHA_DM,
  uMHA_OffLineFile,
  uMHA_Administration_OPS,
  uMHA_UploadResults_IO,
  uMHA_OffLinePatient_IO;


constructor TUploadResults_OPS.Create;
begin
  inherited Create;

  FListOfOffLineFiles := TObjectList.Create;
  FListOfOffLinePatients := TObjectList.Create;
  FPatientSearchResults := TStringList.Create;
end;


destructor TUploadResults_OPS.Destroy;
begin
  if FPatientSearchResults <> nil then
    FPatientSearchResults.Free;
  ListOfOffLineFiles.Free;
  ListOfOffLinePatients.Free;
  inherited Destroy;
end;


procedure TUploadResults_OPS.LoadOffLineFiles;
begin
  ReadOffLineAdministrations;
end;

procedure TUploadResults_OPS.LoadOffLinePatients;
var
  TempSSN,
    TempStr,
    TempStr1                  : string;
  i                           : Integer;
  aPatient                    : TPatient;
begin
  //FListOfOffLinePatients.Clear;
  TempStr := OffLinePatientsRead;
  // TempStr Format: Name = SSN ^ DOB ^ Sex ^ Off-line ID ~

  if (TempStr = BAD_DATA) or (TempStr = '') then
    Exit;

  for i := 1 to NumberOfPieces(TempStr, RECORD_END) - 1 do
    begin
      TempStr1 := Piece(TempStr, RECORD_END, i);

      aPatient := TPatient.Create;
      with aPatient do
        begin
          Name := Piece(TempStr1, EQ, 1);
          TempStr1 := Piece(TempStr1, EQ, 2);
          SSN := Piece(TempStr1, U, 1);
          TempSSN := SSN;
          if Length(TempSSN) > 6 then
            begin
              Insert('-', TempSSN, 4);
              Insert('-', TempSSN, 7);
            end;
          FormattedSSN := TempSSN;
          DOB := Piece(TempStr1, U, 2);
          Sex := Piece(TempStr1, U, 3);
          DFN := Piece(TempStr1, U, 4);

          FListOfOffLinePatients.Add(aPatient);
        end;

    end;
end;

(*
procedure TUploadResults_OPS.LoadByIndex(APatient: TPatient; AIndex:
  integer);
var
  tmpString                   : string;
begin
  if (AIndex < 0) or (AIndex > FAdministrations.Count) then
    exit;

  tmpString := FAdministrations[AIndex];

  FTestDate := FileNameDateTimeToDateTimeString(Piece(tmpString,
    FILE_NAME_DELIMITER, 2));
  FTestName := Piece(tmpString, FILE_NAME_DELIMITER, 3);

  tmpString := Piece(tmpString, FILE_NAME_DELIMITER, 1);
  OffLinePatient_List.LoadPatientByDFN(APatient, tmpString);
end; *)

procedure TUploadResults_OPS.SearchForPatients(ASearchString: string);
var
  tmpResult, tmpCase, tmpName : string;
  i                           : integer;
begin
  // 10052970^RABBIT,VICTOR^6/26/1947^123-45-6789^Sex~
  // becomes
  // RABBIT, VICTOR=6/26/1947^123-45-6789^10052970^Sex
  FPatientSearchResults.Clear;

  tmpResult := PatientSearch(ASearchString, 100, DataModule1.RPCBroker1);

  if tmpResult = 'TO MANY RESULTS' then
    begin
      MessageBox(0,
        'Please refine your search criteria by adding one or more characters to the text ' +
        'for which you are searching. Then try again.',
        'Search returned too many results', MB_ICONSTOP or MB_OK);
      FNumberOfPatientsFound := 0;
      exit;
    end;

  if not (tmpResult = BAD_DATA) then
    for i := 1 to NumberOfPieces(tmpResult, RECORD_END) do
      begin
        tmpCase := Piece(tmpResult, RECORD_END, i);
        tmpName := Piece(tmpCase, U, 2);
      //Insert a space between last and first names to match OffLinePatient.Name
        tmpName := Trim(Piece(tmpName, ',', 1)) + ', ' + Trim(Piece(tmpName,
          ',', 2));
        FPatientSearchResults.Add(tmpName + '=' + Pieces(tmpCase, U, 3, 5) + U
          + Piece(tmpCase, U, 1));
      end;

  FNumberOfPatientsFound := FPatientSearchResults.Count;

end;

function TUploadResults_OPS.PatientSearchNameByIndex(AIndex: integer):
  string;
begin
  Result := FPatientSearchResults[AIndex];
end;

function TUploadResults_OPS.PatientSearchSSNByName(AName: string): string;
begin
  // RABBIT, VICTOR=4/20/1947^123-45-6789^10052970^sex
  Result := Piece(FPatientSearchResults.Values[AName], U, 2);
end;

function TUploadResults_OPS.PatientSearchDOBByName(AName: string): string;
begin
  Result := Piece(FPatientSearchResults.Values[AName], U, 1);
end;

function TUploadResults_OPS.PatientSearchSexByName(AName: string): string;
begin
  Result := Piece(FPatientSearchResults.Values[AName], U, 3);
end;

function TUploadResults_OPS.PatientSearchDFNByName(AName: string): string;
begin
  Result := Piece(FPatientSearchResults.Values[AName], U, 4);
end;

procedure TUploadResults_OPS.LoadMatchConfidenceLevel(APatient: TPatient;
  BPatient: TPatient);
var
  TempStrA, TempStrB          : string;
begin

  TempStrA := Trim(UpperCase(APatient.Sex));
  TempStrB := Trim(UpperCase(BPatient.Sex));

  MatchConfidenceLevel := mlLow;

  if (APatient.Name = BPatient.Name)
    and (APatient.SSN = BPatient.SSN)
    and (APatient.DOB = BPatient.DOB)
    and (APatient.Sex = BPatient.Sex) then
    begin
      MatchConfidenceLevel := mlHigh;
      Exit;
    end;

  if (APatient.Name = BPatient.Name)
    or (APatient.SSN = BPatient.SSN) then
    MatchConfidenceLevel := mlGood;

  if (Length(TempStrA) > 0) and (Length(TempStrB) > 0) then
    if (TempStrA[1] <> TempStrB[1]) then
      MatchConfidenceLevel := mlLow;

end;

procedure TUploadResults_OPS.LoadSearchPatient(var APatient: TPatient;
  AName: string);
begin
  APatient.Name := AName;
  APatient.FormattedSSN := PatientSearchSSNByName(AName);
  APatient.SSN := StringReplace(APatient.FormattedSSN, '-', '', [rfReplaceAll]);
  APatient.DFN := PatientSearchDFNByName(AName);
  APatient.DOB := PatientSearchDOBByName(AName);
  APatient.Sex := PatientSearchSexByName(AName);
end;


procedure TUploadResults_OPS.LoadOffLineAdministration(aDFN,
  aDate, aInstrument, aPatientID: string; var aAdmin: TAdministration);
// aDFN is the temporary, arbitrary number used for off-line identification. It
// is not the PatientID used in Vista.
var
  TempFileName,
    tmpFMDate                 : string;
  tmpDateTime                 : TDateTime;
  tmpList                     : TStringList;
begin

  tmpDateTime := StrToDateTime(aDate); // DateGiven
  tmpFMDate := DateTimeToFileNameDateTime(tmpDateTime);
  TempFileName := LocalFiles.BuildAdministrationFileName(aDFN, tmpFMDate, aInstrument);

  if not FileExists(TempFileName) then
    Exit;

  tmpList := TStringList.Create;
  tmpList.LoadFromFile(TempFileName);

  with aAdmin do
    begin
      ID := tmpList.Values['AdministrationID'];
      InstrumentName := tmpList.Values['InstrumentName'];
      DateGiven := StrToDateTime(tmpList.Values['DateGiven']);
      DateSaved := StrToDateTime(tmpList.Values['DateSaved']);
      Signed := StrToBool(tmpList.Values['Signed']);
      IsComplete := StrToBool(tmpList.Values['IsComplete']);
      NumberAnswered :=
        StrToIntDef(tmpList.Values['NumberOfQuestionsAnswered'], 0);
      // Orderer.DUZ set in fMHA_UploadAdmin
      // AdministeredBy set in fMHA_UploadAdmin
      // Location set in fMHA_UploadAdmin
      // Consult IEN is set in fMHA_UploadAdmin
      PatientID := aPatientID;
    end;

  tmpList.Free;
end;

procedure TUploadResults_OPS.LoadOffLineAnswers(aDFN: string;
  var aAdmin: TAdministration);
var
  i                           : Integer;
  aAnswer                     : TAnswer;
  TempList                    : TStringList;
  TempStr                     : string;
begin
  if not FileExists(LocalFiles.AnswerFileName(aDFN, aAdmin.ID)) then
    Exit;

  TempList := TStringList.Create;
  TempList.LoadFromFile(LocalFiles.AnswerFilename(aDFN,
    aAdmin.ID));

  for i := 0 to TempList.Count - 1 do
    begin
      aAnswer := TAnswer.Create;
      TempStr := TempList[i];

      with aAnswer do
        begin
          QuestionID := Piece(TempStr, U, 1);
          AnswerValue := Piece(TempStr, U, 2);
          AnswerText := Piece(TempStr, U, 3);
          AnswerLegacyValue := Piece(TempStr, U, 4);
          IsMultiChoice := StrToBool(Piece(TempStr, U, 5));
          AnswerItemIndex := StrToIntDef(Piece(TempStr, U, 6), -1);
          AnswerChoiceID := Piece(TempStr, U, 7);
        end;

      aAdmin.ListOfAdminAnswers.Add(aAnswer);
    end;

  TempList.Free;
end;


function TUploadResults_OPS.SaveOffLineInVista(aDFN, aDate: string;
  aAdmin: TAdministration): BOOLEAN;
var
  Reply,
    tmpFMDate                 : string;
  TempAdmin                   : TAdministration;
  tmpDateTime                 : TDateTime;
  
begin

  TempAdmin := TAdministration.Create;
  TempAdmin := aAdmin;

  TempAdmin.ID := NO_ID;

  Reply := Administration_OPS.SaveAdmin(TempAdmin);

  if Pos('ERROR', Reply) = 0 then // No error
    TempAdmin.ID := Reply
  else
    begin
      result := False;
      Exit;
    end;

  if TempAdmin.ID = NO_ID then
    begin
      result := False;
      Exit;
    end;

  Result := Administration_OPS.SaveAnswers(TempAdmin);

  if Result then
    begin
    // Data now saved to Vista, remove off-line files
      with TempAdmin do
        begin
          tmpDateTime := StrToDateTime(aDate); // DateGiven
          tmpFMDate := DateTimeToFileNameDateTime(tmpDateTime);
          SysUtils.DeleteFile(LocalFiles.BuildAdministrationFileName(aDFN,
            tmpFMDate, InstrumentName));

          SysUtils.DeleteFile(LocalFiles.AnswerFileName(aDFN, aAdmin.ID));
        end; // with
    end;
end;

initialization
  UploadResults_OPS := TUploadResults_OPS.Create;

finalization
  UploadResults_OPS.Free;

end.

