unit uMHA_AUX;
// Handles unusual, complicated instruments.

// llh there are many changes that were slated for patch 108 which was never released. References to patch 108 are
//     left unless intended changes are being released with patch 129.

interface

uses
  windows,
  Classes,
  //patch 108 removed fIA_dialogs,
  //fIA_dialogs,
  SysUtils,
  ComCtrls,
  Trpcb,
  //llh - patch 108
  StrUtils,
  dialogs;

procedure DefaultAnswers(
  InstrumentName,
  UserName,
  StationCode,
  StationSuffix,
  PatientName,
  PatientSSN,
  PatientDOB,
  PatientSex,
  AnswerString: string;
  var
  ResultString: string); stdcall;

// Ensure the answers are correct.
// Output is error message: Blank output means okay.
procedure VerifyAnswers(
  InstrumentName,
  PatientName,
  PatientDOB,
  PatientSex,
  AnswerString: string;
  var
  ResultString: string); stdcall;


// Modify acceptable anwsers to a preferred or legacy format
// Output is new answer list.
procedure GroomAnswers(
  InstrumentName,
  PatientName,
  PatientDOB,
  PatientSex,
  AnswerString: string;
  var
  ResultString: string); stdcall;


// Score the answers. ~ScaleName^ScaleIEN^ScaleScore~
procedure ScoreAnswers(
  InstrumentName,
  PatientName,
  PatientDOB,
  PatientSex,
  AnswerString: string;
  var
  ResultString: string); stdcall;


// Hard coded report
procedure ProgressNote(
  InstrumentName,
  PatientName,
  PatientDOB,
  PatientSex,
  AnswerString: string;
  var
  ResultString: string); stdcall;

// Special operations, custom coding
// Output is new answer list.
procedure Custom(
  InstrumentName,
  PatientName,
  PatientDOB,
  PatientSex,
  AnswerString: string;
  var
  ResultString: string); stdcall;

// Output is string, placed on form when built
procedure StaticLookup(
  InstrumentName,
  PatientIEN,
  OrderedByDUZ: string;
  var
  ResultString: string); stdcall;

// Output is string, placed on form when built
procedure VistaLookup(
  Broker: TRPCBroker;
  InstrumentName,
  PatientIEN,
  OrderedByDUZ: string;
  var
  ResultString: string); stdcall;

procedure SpecialResults(
  var
  ResultString: string); stdcall;


implementation

uses
  uMHA_AUX_BAI,
  uMHA_AUX_BDI2,
  uMHA_AUX_BHS,
  uMHA_AUX_BSI,
  uMHA_AUX_CAM,
  uMHA_AUX_SPSI,
  uMHA_AUX_WHODAS,
  uMHA_AUX_WHOQOL_BREF,
  uMHA_AUX_STMS,
  uMHA_AUX_VR12,
  uMHA_AUX_POQ,
  uMHA_AUX_BASIS,
  uMHA_AUX_ISMI,
  uMHA_AUX_MINICOG,
  uMHA_AUX_AUDC,
  uMHA_AUX_WHYMPI,
  uMHA_AUX_NEOPI3,
  uMHA_AUX_BAM,
  uMHA_AUX_CDR,
  uMHA_AUX_SBR,
  uMHA_AUX_SBR_II,
  uMHA_AUX_FAST,
  uMHA_AUX_MMPI2RF,
  uMHA_AUX_QOLI,
  uMHA_AUX_MDQ,
  uMHA_AUX_DBAS,
  uMHA_AUX_SpecialResults,
  //patch 105, added positive negative statement to PC PTSD and PHQ2 instrument llh
  uMHA_AUX_PC_PTSD,
  uMHA_AUX_PHQ2,
  //patch 108, added SST-VOI & SST-VOF, SOCRATES 8A and 8D, BSL-23, DERS, FAD, QOLIE-31  llh
  uMHA_AUX_SSTVOI,
  uMHA_AUX_SSTVOF,
  uMHA_AUX_SOCRATES_8A,
  uMHA_AUX_SOCRATES_8D,
  uMHA_AUX_BSL23,
  uMHA_AUX_DERS,
  uMHA_AUX_FAD,
  uMHA_AUX_QOLIE31;


procedure DefaultAnswers(
  InstrumentName,
  UserName,
  StationCode,
  StationSuffix,
  PatientName,
  PatientSSN,
  PatientDOB,
  PatientSex,
  AnswerString: string;
  var
  ResultString: string); stdcall;
begin
  ResultString := '';
(*
  if Pos('PTSD STATUS FORM', UpperCase(InstrumentName)) > 0 then
    begin
      ResultString := DefaultAnswers_PTSD_Status_Form(InstrumentName,
        UserName,
        StationCode,
        StationSuffix,
        PatientName,
        PatientSSN,
        PatientDOB,
        PatientSex,
        AnswerString);
      Exit;
    end;
    *)
end;


procedure VerifyAnswers(
  InstrumentName,
  PatientName,
  PatientDOB,
  PatientSex,
  AnswerString: string;
  var
  ResultString: string); stdcall;
begin
  // Error message

  ResultString := '';
  (*
  if Pos('PTSD STATUS FORM', UpperCase(InstrumentName)) > 0 then
    begin
      ResultString := Verify_PTSD_Status_Form(InstrumentName,
        PatientName,
        PatientDOB,
        PatientSex,
        AnswerString);
      Exit;
    end; *)
  (*
  if Pos('VAMSTA-A', UpperCase(InstrumentName)) > 0 then
    begin
      ResultString := Verify_VAMSTA_A_Form(InstrumentName,
        PatientName,
        PatientDOB,
        PatientSex,
        AnswerString);
      Exit;
    end; *)

  if Pos('BAM', UpperCase(InstrumentName)) > 0 then
    begin
      if (UpperCase(InstrumentName) = 'BAM') or
        (UpperCase(InstrumentName) = 'BAM-R') or
        (UpperCase(InstrumentName) = 'BAM-IOP') then
        ResultString := Verify_BAM(InstrumentName,
          PatientName,
          PatientDOB,
          PatientSex,
          AnswerString);
      if UpperCase(InstrumentName) = 'BAM-C' then
        ResultString := Verify_BAMC(InstrumentName,
          PatientName,
          PatientDOB,
          PatientSex,
          AnswerString);
      Exit;
    end;
end;


procedure GroomAnswers(InstrumentName,
  PatientName,
  PatientDOB,
  PatientSex,
  AnswerString: string;
  var
  ResultString: string); stdcall;
begin
  ResultString := '';
 (*
  if Pos('PTSD STATUS FORM', UpperCase(InstrumentName)) > 0 then
    begin
      ResultString := GroomAnswers_PTSD_Status_Form(InstrumentName,
        PatientName,
        PatientDOB,
        PatientSex,
        AnswerString);
      Exit;
    end; *)
end;

procedure ScoreAnswers(
  InstrumentName,
  PatientName,
  PatientDOB,
  PatientSex,
  AnswerString: string;
  var
  ResultString: string); stdcall;
// Output: ~ScaleName^ScaleIEN^ScaleScore~

// added llh
var
  TempStr                     : AnsiString;

begin
  ResultString := '';

  // patch 108 added QOLIE llh
  if Pos('QOLIE-31', UpperCase(InstrumentName)) > 0 then
    begin
      ResultString := ScoreAnswers_QOLIE31(
        PatientName,
        PatientDOB,
        PatientSex,
        AnswerString);
      Exit;
    end;

  // patch 108 added FAD llh
  if Pos('FAD', UpperCase(InstrumentName)) > 0 then
    begin
      ResultString := ScoreAnswers_FAD(
        PatientName,
        PatientDOB,
        PatientSex,
        AnswerString);
      Exit;
    end;

  // patch 108 added DERS llh
  if Pos('DERS', UpperCase(InstrumentName)) > 0 then
    begin
      ResultString := ScoreAnswers_DERS(
        PatientName,
        PatientDOB,
        PatientSex,
        AnswerString);
      Exit;
    end;

  // patch 108 added BSL-23 llh
  if Pos('BSL-23', UpperCase(InstrumentName)) > 0 then
    begin
      ResultString := ScoreAnswers_BSL23(
        PatientName,
        PatientDOB,
        PatientSex,
        AnswerString);
      Exit;
    end;

  // patch 108 added SOCRATES_8D llh
  if Pos('SOCRATES 8D', UpperCase(InstrumentName)) > 0 then
    begin
      ResultString := ScoreAnswers_SOCRATES_8D(
        PatientName,
        PatientDOB,
        PatientSex,
        AnswerString);
      Exit;
    end;

  // patch 108 added SOCRATES_8A llh
  if Pos('SOCRATES 8A', UpperCase(InstrumentName)) > 0 then
    begin
      ResultString := ScoreAnswers_SOCRATES_8A(
        PatientName,
        PatientDOB,
        PatientSex,
        AnswerString);
      Exit;
    end;

  // patch 108 added SST-VOI llh
  if Pos('SST-VOI', UpperCase(InstrumentName)) > 0 then
    begin
      ResultString := ScoreAnswers_SSTVOI(
        PatientName,
        PatientDOB,
        PatientSex,
        AnswerString);
      Exit;
    end;

  // patch 108 added SST-VOF llh
  if Pos('SST-VOF', UpperCase(InstrumentName)) > 0 then
    begin
      ResultString := ScoreAnswers_SSTVOF(
        PatientName,
        PatientDOB,
        PatientSex,
        AnswerString);
      Exit;
    end;

  if Pos('BHS', UpperCase(InstrumentName)) > 0 then
    begin
      ResultString := ScoreAnswers_BHS(
        AnswerString);
      Exit;
    end;

  if Pos('BSI', UpperCase(InstrumentName)) > 0 then
    begin
      if InstrumentName = 'BSI18' then
        exit;
      ResultString := ScoreAnswers_BSI(
        AnswerString);
      Exit;
    end;

  if Pos('SPSI', UpperCase(InstrumentName)) > 0 then
    begin
      ResultString := ScoreAnswers_SPSI(
        PatientName,
        PatientDOB,
        PatientSex,
        AnswerString);
      Exit;
    end;

  if Pos('CAM', UpperCase(InstrumentName)) > 0 then
    begin
      ResultString := ScoreAnswers_CAM(
        PatientName,
        PatientDOB,
        PatientSex,
        AnswerString);
      Exit;
    end;

  if Pos('DBAS', UpperCase(InstrumentName)) > 0 then
    begin
      ResultString := ScoreAnswers_DBAS(
        PatientName,
        PatientDOB,
        PatientSex,
        AnswerString);
      Exit;
    end;

  if Pos('MDQ', UpperCase(InstrumentName)) > 0 then
    begin
      ResultString := ScoreAnswers_MDQ(
        PatientName,
        PatientDOB,
        PatientSex,
        AnswerString);
      Exit;
    end;

  // llh - patch 108, there are now 3 QOLIE instruments, only 2 use the AUX
  //       BUT the Pos function returned true for the QOLIE-10, changed to use CompareStr
  //if Pos('QOLI', UpperCase(InstrumentName)) > 0 then
  if CompareStr('QOLI', UpperCase(InstrumentName)) = 0  then
    begin
      ResultString := ScoreAnswers_QOLI(
        PatientName,
        PatientDOB,
        PatientSex,
        AnswerString);
      Exit;
    end;

  if Pos('MMPI-2-RF', UpperCase(InstrumentName)) > 0 then
    begin
      ResultString := ScoreAnswers_MMPI2RF(
        PatientName,
        PatientDOB,
        PatientSex,
        AnswerString);
      Exit;
    end;

  if Pos('WHODAS 2', UpperCase(InstrumentName)) > 0 then
    begin
      ResultString := ScoreAnswers_WHODAS(
        PatientName,
        PatientDOB,
        PatientSex,
        AnswerString);
      Exit;
    end;

  if Pos('WHOQOL-BREF', UpperCase(InstrumentName)) > 0 then
    begin
      ResultString := ScoreAnswers_WHOQOL_BREF(
        PatientName,
        PatientDOB,
        PatientSex,
        AnswerString);
      Exit;
    end;

  if Pos('NEO-PI-3', UpperCase(InstrumentName)) > 0 then
    begin
    // Output: ~ScaleName^RawScore^T-Score~
      ResultString := ScoreAnswers_NEOPI3(
        PatientName,
        PatientDOB,
        PatientSex,
        AnswerString);
      Exit;
    end;


  if Pos('VR-12', UpperCase(InstrumentName)) > 0 then
    begin
      ResultString := ScoreAnswers_VR12(
        PatientName,
        PatientDOB,
        PatientSex,
        AnswerString);
      Exit;
    end;

  if Pos('BAM-C', UpperCase(InstrumentName)) > 0 then
    begin
      ResultString := ScoreAnswers_BAMC(
        PatientName,
        PatientDOB,
        PatientSex,
        AnswerString);
      Exit;
    end;

  if Pos('BAM-R', UpperCase(InstrumentName)) > 0 then
    begin
      ResultString := ScoreAnswers_BAMR(
        PatientName,
        PatientDOB,
        PatientSex,
        AnswerString);
      Exit;
    end;

  if Pos('WHYMPI', UpperCase(InstrumentName)) > 0 then
    begin
      ResultString := ScoreAnswers_WHYMPI(
        PatientName,
        PatientDOB,
        PatientSex,
        AnswerString);
      Exit;
    end;

  if Pos('MINICOG', UpperCase(InstrumentName)) > 0 then
    begin
      ResultString := ScoreAnswers_MINICOG(
        PatientName,
        PatientDOB,
        PatientSex,
        AnswerString);
      Exit;
    end;

  if Pos('STMS', UpperCase(InstrumentName)) > 0 then
    begin
      ResultString := ScoreAnswers_STMS(
        PatientName,
        PatientDOB,
        PatientSex,
        AnswerString);
      Exit;
    end;

  if Pos('BASIS-24', UpperCase(InstrumentName)) > 0 then
    begin
      ResultString := ScoreAnswers_BASIS(
        PatientName,
        PatientDOB,
        PatientSex,
        AnswerString);
      Exit;
    end;

  if Pos('ISMI', UpperCase(InstrumentName)) > 0 then
    begin
      ResultString := ScoreAnswers_ISMI(
        PatientName,
        PatientDOB,
        PatientSex,
        AnswerString);
      Exit;
    end;

  if Pos('POQ', UpperCase(InstrumentName)) > 0 then
    begin
      ResultString := ScoreAnswers_POQ(
        PatientName,
        PatientDOB,
        PatientSex,
        AnswerString);
      Exit;
    end;

  if Pos('FAST', UpperCase(InstrumentName)) > 0 then
    begin
      ResultString := ScoreAnswers_FAST(
        PatientName,
        PatientDOB,
        PatientSex,
        AnswerString);
      Exit;
    end;

  if Pos('CDR', UpperCase(InstrumentName)) > 0 then
    begin
      ResultString := ScoreAnswers_CDR(
        PatientName,
        PatientDOB,
        PatientSex,
        AnswerString);
      Exit;
    end;

  if Pos('SBR II', UpperCase(InstrumentName)) > 0 then
    begin
      ResultString := ScoreAnswers_SBR_II(
        PatientName,
        PatientDOB,
        PatientSex,
        AnswerString);
      Exit;
    end;

//  if Pos('AUDC', UpperCase(InstrumentName)) > 0 then
//    begin
//      ResultString := ScoreAnswers_AUDC(
//        AnswerString);
//      Exit;
//    end;

{
  if Pos('BAI', UpperCase(InstrumentName)) > 0 then
    begin
      ResultString := ScoreAnswers_BAI(
        AnswerString);
      Exit;
    end;
}

{
  if Pos('BDI2', UpperCase(InstrumentName)) > 0 then
    begin
      ResultString := ScoreAnswers_BDI2(
        AnswerString);
      Exit;
    end;
}
end;


procedure ProgressNote(
  InstrumentName,
  PatientName,
  PatientDOB,
  PatientSex,
  AnswerString: string;
  var
  ResultString: string); stdcall;
// NOTE:  <.DLL_String.> provides location for this string in the
// report.

var
  TempStr                     : string;
begin
  ResultString := '';

  //patch 108, added QOLIE 31 llh
  if Pos('QOLIE-31', UpperCase(InstrumentName)) > 0 then
    begin
      TempStr := ScoreAnswers_QOLIE31(
        PatientName,
        PatientDOB,
        PatientSex,
        AnswerString);

      if TempStr = '' then
        Exit;

      ResultString := ProgressNote_QOLIE31(
        PatientName,
        PatientDOB,
        PatientSex,
        TempStr,
        AnswerString);
      Exit;
    end;

  //patch 108, added FAD llh
  if Pos('FAD', UpperCase(InstrumentName)) > 0 then
    begin
      TempStr := ScoreAnswers_FAD(
        PatientName,
        PatientDOB,
        PatientSex,
        AnswerString);

      if TempStr = '' then
        Exit;

      ResultString := ProgressNote_FAD(
        PatientName,
        PatientDOB,
        PatientSex,
        TempStr,
        AnswerString);
      Exit;
    end;

  //patch 108, added DERS llh
  if Pos('DERS', UpperCase(InstrumentName)) > 0 then
    begin
      TempStr := ScoreAnswers_DERS(
        PatientName,
        PatientDOB,
        PatientSex,
        AnswerString);

      if TempStr = '' then
        Exit;

      ResultString := ProgressNote_DERS(
        PatientName,
        PatientDOB,
        PatientSex,
        TempStr,
        AnswerString);
      Exit;
    end;

  //patch 108, added BSL-23 llh
  if Pos('BSL-23', UpperCase(InstrumentName)) > 0 then
    begin
      TempStr := ScoreAnswers_BSL23(
        PatientName,
        PatientDOB,
        PatientSex,
        AnswerString);

      if TempStr = '' then
        Exit;

      ResultString := ProgressNote_BSL23(
        PatientName,
        PatientDOB,
        PatientSex,
        TempStr,
        AnswerString);
      Exit;
    end;

  //patch 108, added SOCRATES 8D llh
  if Pos('SOCRATES 8D', UpperCase(InstrumentName)) > 0 then
    begin
      TempStr := ScoreAnswers_SOCRATES_8D(
        PatientName,
        PatientDOB,
        PatientSex,
        AnswerString);

      if TempStr = '' then
        Exit;

      ResultString := ProgressNote_SOCRATES_8D(
        PatientName,
        PatientDOB,
        PatientSex,
        TempStr,
        AnswerString);
      Exit;
    end;

  //patch 108, added SOCRATES 8A llh
  if Pos('SOCRATES 8A', UpperCase(InstrumentName)) > 0 then
    begin
      TempStr := ScoreAnswers_SOCRATES_8A(
        PatientName,
        PatientDOB,
        PatientSex,
        AnswerString);

      if TempStr = '' then
        Exit;

      ResultString := ProgressNote_SOCRATES_8A(
        PatientName,
        PatientDOB,
        PatientSex,
        TempStr,
        AnswerString);
      Exit;
    end;

  //patch 108, added SST-VOI llh
  if Pos('SST-VOI', UpperCase(InstrumentName)) > 0 then
    begin
      TempStr := ScoreAnswers_SSTVOI(
        PatientName,
        PatientDOB,
        PatientSex,
        AnswerString);

      if TempStr = '' then
        Exit;

      ResultString := ProgressNote_SSTVOI(
        PatientName,
        PatientDOB,
        PatientSex,
        TempStr,
        AnswerString);
      Exit;
    end;

  //patch 108, added SST-VOF llh
  if Pos('SST-VOF', UpperCase(InstrumentName)) > 0 then
    begin
      TempStr := ScoreAnswers_SSTVOF(
        PatientName,
        PatientDOB,
        PatientSex,
        AnswerString);

      if TempStr = '' then
        Exit;

      ResultString := ProgressNote_SSTVOF(
        PatientName,
        PatientDOB,
        PatientSex,
        TempStr,
        AnswerString);
      Exit;
    end;

  // patch 105, added PHQ-2 so that positive and negative statements can be added to report llh
    if Pos('PHQ-2', UpperCase(InstrumentName)) > 0 then
    begin
      TempStr := ScoreAnswers_PHQ2(
        AnswerString);

      if TempStr = '' then
        Exit;

      ResultString := ProgressNote_PHQ2(
        TempStr);

      Exit;
    end;

  // patch 105, added PC PTSD so that positive and negative statements can be added to report llh
    if Pos('PC PTSD', UpperCase(InstrumentName)) > 0 then
    begin
      TempStr := ScoreAnswers_PCPTSD(
        AnswerString);

      if TempStr = '' then
        Exit;

      ResultString := ProgressNote_PCPTSD(
        TempStr);

      Exit;
    end;

  if Pos('BAI', UpperCase(InstrumentName)) > 0 then
    begin
      TempStr := ScoreAnswers_BAI(
        AnswerString);

      if TempStr = '' then
        Exit;

      ResultString := ProgressNote_BAI(
        TempStr);

      Exit;
    end;

  if Pos('BDI2', UpperCase(InstrumentName)) > 0 then
    begin
      TempStr := ScoreAnswers_BDI2(
        AnswerString);

      if TempStr = '' then
        Exit;

      ResultString := ProgressNote_BDI2(
        TempStr);

      Exit;
    end;

  if Pos('BHS', UpperCase(InstrumentName)) > 0 then
    begin
      TempStr := ScoreAnswers_BHS(
        AnswerString);

      if TempStr = '' then
        Exit;

      ResultString := ProgressNote_BHS(
        TempStr);

      Exit;
    end;

  if Pos('BSI', UpperCase(InstrumentName)) > 0 then
    begin
      if InstrumentName = 'BSI18' then
        exit;

      TempStr := ScoreAnswers_BSI(
        AnswerString);

      if TempStr = '' then
        Exit;

      ResultString := ProgressNote_BSI(
        TempStr);

      Exit;
    end;

  if Pos('SPSI', UpperCase(InstrumentName)) > 0 then
    begin
      TempStr := ScoreAnswers_SPSI(
        PatientName,
        PatientDOB,
        PatientSex,
        AnswerString);

      if TempStr = '' then
        Exit;

      ResultString := ProgressNote_SPSI(
        PatientName,
        PatientDOB,
        PatientSex,
        TempStr,
        AnswerString);
      Exit;
    end;

  if Pos('CAM', UpperCase(InstrumentName)) > 0 then
    begin
      TempStr := ScoreAnswers_CAM(
        PatientName,
        PatientDOB,
        PatientSex,
        AnswerString);

      if TempStr = '' then
        Exit;

      ResultString := ProgressNote_CAM(
        PatientName,
        PatientDOB,
        PatientSex,
        TempStr,
        AnswerString);
      Exit;
    end;

  if Pos('DBAS', UpperCase(InstrumentName)) > 0 then
    begin
      TempStr := ScoreAnswers_DBAS(
        PatientName,
        PatientDOB,
        PatientSex,
        AnswerString);

      if TempStr = '' then
        Exit;

      ResultString := ProgressNote_DBAS(
        PatientName,
        PatientDOB,
        PatientSex,
        TempStr,
        AnswerString);
      Exit;
    end;

  if Pos('MDQ', UpperCase(InstrumentName)) > 0 then
    begin
      TempStr := ScoreAnswers_MDQ(
        PatientName,
        PatientDOB,
        PatientSex,
        AnswerString);

      if TempStr = '' then
        Exit;

      ResultString := ProgressNote_MDQ(
        PatientName,
        PatientDOB,
        PatientSex,
        TempStr,
        AnswerString);
      Exit;
    end;

  if Pos('QOLI', UpperCase(InstrumentName)) > 0 then
    begin
      TempStr := ScoreAnswers_QOLI(
        PatientName,
        PatientDOB,
        PatientSex,
        AnswerString);

      if TempStr = '' then
        Exit;

      ResultString := ProgressNote_QOLI(
        PatientName,
        PatientDOB,
        PatientSex,
        TempStr,
        AnswerString);
      Exit;
    end;


  if Pos('MMPI-2-RF', UpperCase(InstrumentName)) > 0 then
    begin

      TempStr := ScoreAnswers_MMPI2RF(
        PatientName,
        PatientDOB,
        PatientSex,
        AnswerString);

      if TempStr = '' then
        Exit;

      ResultString := ProgressNote_MMPI2RF(
        PatientName,
        PatientDOB,
        PatientSex,
        TempStr);

      Exit;
    end;


  if Pos('WHODAS 2', UpperCase(InstrumentName)) > 0 then
    begin

      TempStr := ScoreAnswers_WHODAS(
        PatientName,
        PatientDOB,
        PatientSex,
        AnswerString);

      if TempStr = '' then
        Exit;

      ResultString := ProgressNote_WHODAS(
        PatientName,
        PatientDOB,
        PatientSex,
        TempStr,
        AnswerString);

      Exit;
    end;

  if Pos('WHOQOL-BREF', UpperCase(InstrumentName)) > 0 then
    begin

      TempStr := ScoreAnswers_WHOQOL_BREF(
        PatientName,
        PatientDOB,
        PatientSex,
        AnswerString);

      if TempStr = '' then
        Exit;

      ResultString := ProgressNote_WHOQOL_BREF(
        PatientName,
        PatientDOB,
        PatientSex,
        TempStr,
        AnswerString);

      Exit;
    end;

  if Pos('NEO-PI-3', UpperCase(InstrumentName)) > 0 then
    begin
      // This has raw and t-scores
      TempStr := ScoreAnswers_NEOPI3(
        PatientName,
        PatientDOB,
        PatientSex,
        AnswerString);

      if TempStr = '' then
        Exit;

      ResultString := ProgressNote_NEOPI3(
        PatientName,
        PatientDOB,
        PatientSex,
        TempStr,
        AnswerString);

      Exit;
    end;

  if Pos('BAM-C', UpperCase(InstrumentName)) > 0 then
    begin

      TempStr := ScoreAnswers_BAMC(
        PatientName,
        PatientDOB,
        PatientSex,
        AnswerString);

      if TempStr = '' then
        Exit;

      ResultString := ProgressNote_BAMC(
        PatientName,
        PatientDOB,
        PatientSex,
        TempStr,
        AnswerString);

      Exit;
    end;

  if Pos('BAM-R', UpperCase(InstrumentName)) > 0 then
    begin
      TempStr := ScoreAnswers_BAMR(
        PatientName,
        PatientDOB,
        PatientSex,
        AnswerString);

      if TempStr = '' then
        Exit;

      ResultString := ProgressNote_BAMR(
        PatientName,
        PatientDOB,
        PatientSex,
       TempStr,
        AnswerString);

      Exit;
    end;

  if Pos('VR-12', UpperCase(InstrumentName)) > 0 then
    begin

      TempStr := ScoreAnswers_VR12(
        PatientName,
        PatientDOB,
        PatientSex,
        AnswerString);

      if TempStr = '' then
        Exit;

      ResultString := ProgressNote_VR12(
        PatientName,
        PatientDOB,
        PatientSex,
        TempStr,
        AnswerString);

      Exit;
    end;

  if Pos('MINICOG', UpperCase(InstrumentName)) > 0 then
    begin

      TempStr := ScoreAnswers_MINICOG(
        PatientName,
        PatientDOB,
        PatientSex,
        AnswerString);

      if TempStr = '' then
        Exit;

      ResultString := ProgressNote_MINICOG(
        PatientName,
        PatientDOB,
        PatientSex,
        TempStr,
        AnswerString);

      Exit;
    end;

  if Pos('STMS', UpperCase(InstrumentName)) > 0 then
    begin

      TempStr := ScoreAnswers_STMS(
        PatientName,
        PatientDOB,
        PatientSex,
        AnswerString);

      if TempStr = '' then
        Exit;

      ResultString := ProgressNote_STMS(
        PatientName,
        PatientDOB,
        PatientSex,
        TempStr,
        AnswerString);

      Exit;
    end;

  if Pos('CDR', UpperCase(InstrumentName)) > 0 then
    begin

      TempStr := ScoreAnswers_CDR(
        PatientName,
        PatientDOB,
        PatientSex,
        AnswerString);

      if TempStr = '' then
        Exit;

      ResultString := ProgressNote_CDR(
        PatientName,
        PatientDOB,
        PatientSex,
        TempStr,
        AnswerString);

      Exit;
    end;

  if Pos('FAST', UpperCase(InstrumentName)) > 0 then
    begin

      TempStr := ScoreAnswers_FAST(
        PatientName,
        PatientDOB,
        PatientSex,
        AnswerString);

      if TempStr = '' then
        Exit;

      ResultString := ProgressNote_FAST(
        PatientName,
        PatientDOB,
        PatientSex,
        TempStr,
        AnswerString);

      Exit;
    end;

  if Pos('BASIS-24', UpperCase(InstrumentName)) > 0 then
    begin
      TempStr := ScoreAnswers_BASIS(
        PatientName,
        PatientDOB,
        PatientSex,
        AnswerString);

      if TempStr = '' then
        Exit;

      ResultString := ProgressNote_BASIS(
        PatientName,
        PatientDOB,
        PatientSex,
        TempStr,
        AnswerString);

      Exit;
    end;

  if Pos('ISMI', UpperCase(InstrumentName)) > 0 then
    begin

      TempStr := ScoreAnswers_ISMI(
        PatientName,
        PatientDOB,
        PatientSex,
        AnswerString);

      if TempStr = '' then
        Exit;

      ResultString := ProgressNote_ISMI(
        PatientName,
        PatientDOB,
        PatientSex,
        TempStr,
        AnswerString);

      Exit;
    end;

   // Screen reader version: place before the non-screen reader
  if Pos('POQ-SR', UpperCase(InstrumentName)) > 0 then
    begin
      TempStr := ScoreAnswers_POQ(
        PatientName,
        PatientDOB,
        PatientSex,
        AnswerString);

      if TempStr = '' then
        Exit;

      ResultString := ProgressNote_POQ_SR(
        PatientName,
        PatientDOB,
        PatientSex,
        TempStr,
        AnswerString);

      Exit;
    end;

  if Pos('POQ', UpperCase(InstrumentName)) > 0 then
    begin

      TempStr := ScoreAnswers_POQ(
        PatientName,
        PatientDOB,
        PatientSex,
        AnswerString);

      if TempStr = '' then
        Exit;

      ResultString := ProgressNote_POQ(
        PatientName,
        PatientDOB,
        PatientSex,
        TempStr,
        AnswerString);

      Exit;
    end;

  if InstrumentName = 'AUDC' then
    begin
      TempStr := ScoreAnswers_AUDC(
        AnswerString);

      if TempStr = '' then
        Exit;

      ResultString := ProgressNote_AUDC(
        PatientSex,
        TempStr);
      Exit;
    end;

  if Pos('WHYMPI', UpperCase(InstrumentName)) > 0 then
    begin

      TempStr := ScoreAnswers_WHYMPI(
        PatientName,
        PatientDOB,
        PatientSex,
        AnswerString);

      if TempStr = '' then
        Exit;

      ResultString := ProgressNote_WHYMPI(
        PatientName,
        PatientDOB,
        PatientSex,
        TempStr,
        AnswerString);

      Exit;
    end;

  if Pos('SBR II', UpperCase(InstrumentName)) > 0 then
    begin

      TempStr := ScoreAnswers_SBR_II(
        PatientName,
        PatientDOB,
        PatientSex,
        AnswerString);

      if TempStr = '' then
        Exit;

      ResultString := ProgressNote_SBR_II(
        PatientName,
        PatientDOB,
        PatientSex,
        TempStr,
        AnswerString);

      Exit;
    end;
end;

procedure Custom(
  InstrumentName,
  PatientName,
  PatientDOB,
  PatientSex,
  AnswerString: string;
  var
  ResultString: string); stdcall;

begin
  ResultString := '';
  (*
  if Trim(InstrumentName) = 'ROGERS BOGUS' then
    begin

      ResultString := Custom_BOGUS(
        PatientName,
        PatientDOB,
        PatientSex,
        AnswerString);
    end;
    *)
end;

procedure StaticLookup(
  InstrumentName,
  PatientIEN,
  OrderedByDUZ: string;
  var
  ResultString: string); stdcall;
// ResultString looks like:
//     Number=text^Number=Text
//     where number is place in form's sequence
begin
  ResultString := '';

  if Trim(InstrumentName) = 'GDS DEMENTIA' then
    begin
      ResultString :=

      '0=                        Global Deterioration Scale' + #13 + #13 +
        '1. No cognitive decline' + #13 +
        'No subjective complaints of memory deficit. No memory deficit evident on ' +
        'clinical interview. ' + #13 + #13 +

      '2. Very mild cognitive decline (Age Associated Memory Impairment) ' + #13 +
        'Subjective complaints of memory deficit, most frequently in following areas: (a) ' +
        'forgetting where one has placed familiar objects; (b) forgetting names one ' +
        'formerly knew well. No objective evidence of memory deficit on clinical ' +
        'interview. No objective deficits in employment or social situations. Appropriate ' +
        'concern with respect to symptomatology. ' + #13 + #13 +

      '3. Mild cognitive decline (Mild Cognitive Impairment) ' + #13 +
        'Earliest clear-cut deficits. Manifestations in more than one of the following areas: ' +
        '(a) patient may have gotten lost when traveling to an unfamiliar location; (b) coworkers ' +
        'become aware of patient''s relatively poor performance; (c) word and ' +
        'name finding deficit becomes evident to intimates; (d) patient may read a passage ' +
        'or a book and retain relatively little material; (e) patient may demonstrate ' +
        'decreased facility in remembering names upon introduction to new people; (f) ' +
        'patient may have lost or misplaced an object of value; (g) concentration deficit ' +
        'may be evident on clinical testing. Objective evidence of memory deficit obtained ' +
        'only with an intensive interview. Decreased performance in demanding ' +
        'employment and social settings. Denial begins to become manifest in patient. ' +
        'Mild to moderate anxiety accompanies symptoms. ' + #13 + #13 +

      '4. Moderate cognitive decline (Mild Dementia) ' + #13 +
        'Clear-cut deficit on careful clinical interview. Deficit manifest in following areas: ' +
        '(a) decreased knowledge of current and recent events; (b) may exhibit some ' +
        'deficit in memory of ones personal history; (c) concentration deficit elicited on ' +
        'serial subtractions; (d) decreased ability to travel, handle finances, etc. Frequently ' +
        'no deficit in following areas: (a) orientation to time and place; (b) recognition of ' +
        'familiar persons and faces; (c) ability to travel to familiar locations. Inability to ' +
        'perform complex tasks. Denial is dominant defense mechanism. Flattening of ' +
        'affect and withdrawal from challenging situations frequently occur. ' + #13 + #13 +

      '5. Moderately severe cognitive decline (Moderate Dementia) ' + #13 +
        'Patient can no longer survive without some assistance. Patient is unable during ' +
        'interview to recall a major relevant aspect of their current lives, e.g., an address or ' +
        'telephone number of many years, the names of close family members (such as ' +
        'grandchildren), the name of the high school or college from which they ' +
        'graduated. Frequently some disorientation to time (date, day of week, season, ' +
        'or to place. An educated person may have difficulty counting back from 40 ' +
        'by 4s or from 20 by 2s. Persons at this stage retain knowledge of many major ' +
        'facts regarding themselves and others. They invariably know their own names ' +
        'and generally know their spouses'' and children''s names. They require no ' +
        'assistance with toileting and eating, but may have some difficulty choosing the ' +
        'proper clothing to wear. ' + #13 + #13 +

      '6. Severe cognitive decline (Moderately Severe Dementia) ' + #13 +
        'May occasionally forget the name of the spouse upon whom they are entirely ' +
        'dependent for survival. Will be largely unaware of all recent events and ' +
        'experiences in their lives. Retain some knowledge of their past lives but this is ' +
        'very sketchy. Generally unaware of their surroundings, the year, the season, etc. ' +
        'May have difficulty counting from 10, both backward and, sometimes, forward. ' +
        'Will require some assistance with activities of daily living, e.g., may become  ' +
        'incontinent, will require travel assistance but occasionally will be able to travel to ' +
        'familiar locations. Diurnal rhythm frequently disturbed. Almost always recall ' +
        'their own name. Frequently continue to be able to distinguish familiar from ' +
        'unfamiliar persons in their environment. Personality and emotional changes ' +
        'occur. These are quite variable and include: (a) delusional behavior, e.g., patients ' +
        'may accuse their spouse of being an impostor, may talk to imaginary figures in ' +
        'the environment, or to their own reflection in the mirror; (b) obsessive symptoms, ' +
        'e.g., person may continually repeat simple cleaning activities; (c) anxiety ' +
        'symptoms, agitation, and even previously nonexistent violent behavior may ' +
        'occur; (d) cognitive abulla, i.e., loss of willpower because an individual cannot ' +
        'carry a thought long enough to determine a purposeful course of action. ' + #13 + #13 +

      '7. Very severe cognitive decline (Severe Dementia) ' + #13 +
        'All verbal abilities are lost over the course of this stage. Frequently there is no ' +
        'speech at all -only unintelligible utterances and rare emergence of seemingly ' +
        'forgotten words and phrases. Incontinent of urine, requires assistance toileting ' +
        'and feeding. Basic psychomotor skills, e.g., ability to walk, are lost with the ' +
        'progression of this stage. The brain appears to no longer be able to tell the body ' +
        'what to do. Generalized rigidity and developmental neurologic reflexes are ' +
        'frequently present.^ ';
    end;

  if Trim(InstrumentName) = 'SLUMS' then
    begin
      ResultString :=
        '17= Jill was a very successful stockbroker. She made a lot of money on the stock market. ' +
        'She then met Jack, a devastatingly handsome man. She married him and had three children. ' +
        'They lived in Chicago. She then stopped work and stayed at home to bring up her children. ' +
        'When they were teenagers, she went back to work. She and Jack lived happily ever after.^';
    end;
end;

procedure VistaLookup(
  Broker: TRPCBroker;
  InstrumentName,
  PatientIEN,
  OrderedByDUZ: string;
  var
  ResultString: string); stdcall;
// ResultString looks like:
//     Number=text^Number=Text^
//     where number is place in form's sequence
begin
  ResultString := '';

  if Trim(InstrumentName) = 'SBR' then
    begin
      ResultString := GetVistaText_SBR(
        Broker,
        InstrumentName,
        PatientIEN,
        OrderedByDUZ);
    end;

  if Trim(InstrumentName) = 'SBR II' then
    begin
      ResultString := GetVistaText_SBR_II(
        Broker,
        InstrumentName,
        PatientIEN,
        OrderedByDUZ);
    end;

end;



procedure SpecialResults(
  var
  ResultString: string); stdcall;
begin
  ResultString := GetSpecialResults;
end;




end.

 