unit uMHA_AUX_CDR;

interface

uses
  SysUtils,Dialogs;

function ScoreAnswers_CDR(
  PatientName,
  PatientDOB,
  PatientSex,
  AnswerString: string): string;

function ProgressNote_CDR(
  PatientName,
  PatientDOB,
  PatientSex,
  ScoreString,
  AnswerString: string): string;   


implementation

uses
  uMHA_AUX_Const,
  //patch 129 removed fIA_dialogs,
  //fIA_dialogs,
  uMHA_AUX_Utils;


{*
  The format for output is used by Clinical reminders as their preferred method.
  The second field in the output is the Scale IEN, which is not used in IAT.

  The CDR has an overall score based primary on the Memory score (Question 1).
*}

function ScoreAnswers_CDR(
  PatientName,
  PatientDOB,
  PatientSex,
  AnswerString: string): string;

var
  Count, i, Mem,
    Above, Below,
    TempAnswer
                              : Integer;

  CDR_Score,
    HighMajority,
    LowMajority,
    TempResult
                              : string;

  function MostFrequent(StringIn: string): string;
  // Returns most frequent score; ties return a blank;
  var
    ii, aCount,
      GotZero,
      GotOne, GotTwo,
      GotThree, GotFour       : integer;

  begin
    result := '';

    GotZero := 0;
    GotOne := 0;
    GotTwo := 0;
    GotThree := 0;
    GotFour := 0;

    aCount := NumberOfPieces(StringIn, '^');
    if aCount = 0 then
      exit;

    for ii := 1 to aCount - 1 do
      begin
        if Piece(StringIn, '^', ii) = '0' then
          GotZero := GotZero + 1;
        if Piece(StringIn, '^', ii) = '1' then
          GotOne := GotOne + 1;
        if Piece(StringIn, '^', ii) = '2' then
          GotTwo := GotTwo + 1;
        if Piece(StringIn, '^', ii) = '3' then
          GotThree := GotThree + 1;
        if Piece(StringIn, '^', ii) = '4' then
          GotFour := GotFour + 1;
      end;

    if (GotZero > GotOne) and (GotZero > GotTwo) and (GotZero > GotThree) and
      (GotZero > GotFour) then
      Result := '0';
    if (GotOne > GotZero) and (GotOne > GotTwo) and (GotOne > GotThree) and
      (GotOne > GotFour) then
      Result := '0.5';
    if (GotTwo > GotZero) and (GotTwo > GotOne) and (GotTwo > GotThree) and
      (GotTwo > GotFour) then
      Result := '1';
    if (GotThree > GotZero) and (GotThree > GotOne) and (GotThree > GotTwo) and
      (GotThree > GotFour) then
      Result := '2';
    if (GotFour > GotZero) and (GotFour > GotOne) and (GotFour > GotTwo) and
      (GotFour > GotThree) then
      Result := '3';

    // No clear winner, get the ties
    if result = '' then
      begin
        if (GotZero = 2) and ((GotZero = GotOne) or (GotZero = GotTwo) or (GotZero = GotThree) or
          (GotZero = GotFour)) then
          Result := '0^';
        if (GotOne = 2) and ((GotOne = GotZero) or (GotOne = GotTwo) or (GotOne = GotThree) or
          (GotOne = GotFour)) then
          Result := Result + '0.5^';
        if (GotTwo = 2) and ((GotTwo = GotZero) or (GotTwo = GotOne) or (GotTwo = GotThree) or
          (GotTwo = GotFour)) then
          Result := Result + '1^';
        if (GotThree = 2) and ((GotThree = GotZero) or (GotThree = GotOne) or (GotThree = GotTwo) or
          (GotThree = GotFour)) then
          Result := Result + '2^';
        if (GotFour = 2) and ((GotFour = GotZero) or (GotFour = GotOne) or (GotFour = GotTwo) or
          (GotFour = GotThree)) then
          Result := Result + '3^';
      end;
  end;

begin
  // AnswerString Format:
  //      83=1^1330~
  //      84=0^214~...
  //
  // '83' is QuestionID,
  // '1' is ItemIndex for radio group, but can be text from edit field
  // '1130' is ChoiceID (not used here)

  // This is used by InstrumentResults to do a quick check on the
  // instrument as to its use of the dll
  if AnswerString = 'CDR' then
  begin
     Result := 'YES';
     exit;
  end;

  Result := '';
  Count := NumberOfPieces(AnswerString, RECORD_END);
  CDR_Score := '';
  HighMajority := '';
  LowMajority := '';
  Mem := -1;
  Above := 0;
  Below := 0;

  if Count < 1 then
    Exit;

  for i := 1 to Count - 1  do
    begin
      TempResult := Piece(AnswerString, RECORD_END, i);
      TempAnswer := StrToIntDef(Piece(Piece(TempResult, U, 1), EQ, 2), -1);

      if (i = 1) then // Memory
        begin
          Mem := TempAnswer;
          // Set default value
          if TempAnswer = 0 then
            CDR_Score := '0';
          if TempAnswer = 1 then
            CDR_Score := '0.5';
          if TempAnswer = 2 then
            CDR_Score := '1';
          if TempAnswer = 3 then
            CDR_Score := '2';
          if TempAnswer = 4 then
            CDR_Score := '3';
        end
      else
        begin
          if (i = 6) and (TempAnswer <> 0) then // Personal Care
            TempAnswer := TempAnswer + 1;  // No ".5" score
          if TempAnswer > Mem then
            begin
              HighMajority := HighMajority + inttostr(TempAnswer) + '^';
              Above := Above + 1;
            end;
          if TempAnswer < Mem then
            begin
              Below := Below + 1;
              LowMajority := LowMajority + inttostr(TempAnswer) + '^';
            end;
        end;
    end; // for i

  if Mem = -1 then
    exit;

  if (Above > 2) and (Below > 1) then
    Result := CDR_Score;

  if (Above > 2) and (Below < 2) then
    begin
      // CDR is the score of the majority
      CDR_Score := MostFrequent(HighMajority);

      if POS('^', CDR_Score) > 0 then // tied scores: pick lowest
        CDR_Score := Piece(CDR_Score, '^', 1);
    end;

  if (Below > 2) and (Above > 1) then
    Result := CDR_Score;

  if (Below > 2) and (Above < 2) then
    begin
      // CDR is the score of the majority
      CDR_Score := MostFrequent(LowMajority);

      if POS('^', CDR_Score) > 0 then // tied scores: pick highest
        CDR_Score := Piece(CDR_Score, '^', 2);

    end;

  // Review fixed rules for scoring...
  if (Mem = 1) and (Above > 2) then
    CDR_Score := '1';

  if (Mem = 0) and (Above > 1) then
    CDR_Score := '0.5';

  if (CDR_Score = '0') and (Mem > 0) then
    CDR_Score := '0.5';

  // Note: "CDR" is the name of the scale and it must be used here.
  // ~ScaleName^ScaleIEN^ScaleScore~  
  Result := 'CDR^^' + CDR_Score + RECORD_END;
  //Showmessage('CDR Result: ' + Result);
end;


function ProgressNote_CDR(
  PatientName,
  PatientDOB,
  PatientSex,
  ScoreString,
  AnswerString: string): string;
// The header, footer and questions and answers are handled elsewhere; avoiding
// broker calls here.

const
  Indent                      : string = '   ';
var
  i                           : Integer;
  TempStr                     : string;
begin
  Result := '';

  if (ScoreString = '') or (AnswerString = '') then
    Exit;

  Result :=  Piece(Piece(ScoreString, RECORD_END, 1), U, 3);

end;

end.

