unit uMHA_AUX_MINICOG;

interface

uses
  SysUtils;

function ScoreAnswers_MINICOG(
  PatientName,
  PatientDOB,
  PatientSex,
  AnswerString: string): string;

function ProgressNote_MINICOG(
  PatientName,
  PatientDOB,
  PatientSex,
  ScoreString,
  AnswerString: string): string;

implementation

uses
  uMHA_AUX_Const,
  //patch 129 removed fIA_dialogs,
  //fIA_dialogs,
  uMHA_AUX_Utils;


{*
  The format for output is used by Clinical reminders as their preferred method.
  The second field in the output is the Scale IEN, which is not used in IAT.

  The Mini-Cog has the subject do two tasks, remember three words and draw a clock.
*}

function ScoreAnswers_MINICOG(
  PatientName,
  PatientDOB,
  PatientSex,
  AnswerString: string): string;

const
  Spacer                      : string = '      ';

var
  clock                       : boolean;
  Count, i,SkipQues,
    Total                     : Integer;
  Statement,
    TempResult,
    TempAnswer                : string;

begin
  // AnswerString Format:
  //      83=1^1330~
  //      84=0^214~...
  //
  // '83' is QuestionID,
  // '1' is ItemIndex for radio group, but can be text from edit field
  // '1130' is ChoiceID (not used here)

  // This is used by InstrumentResults to do a quick check on the
  // instrument as to its use of the dll
  if AnswerString = 'MINICOG' then
    begin
      Result := 'YES';
      exit;
    end;

  Result := '';
  clock := true;
  Statement := '';
  Total := 0;
  Count := NumberOfPieces(AnswerString, RECORD_END);
  //patch 105; count skipped questions, if any, invalid results  llh
  SkipQues := 0 ;


  if Count < 1 then
    Exit;

  for i := 1 to Count - 1 do
    begin
      TempResult := Piece(AnswerString, RECORD_END, i);
      TempAnswer := Piece(Piece(TempResult, U, 1), EQ, 2);

      // Patch 105, Count the skipped (Blank) answers llh
      if (i = 1) and (TempAnswer = '1') then
        clock := false;

      if (TempAnswer = '') or (TempAnswer = ' ') or
        (TempAnswer = '-1') or
        (TempAnswer = NOT_ASKED) or
        (TempAnswer = SKIPPED) or
        (TempAnswer = '1155') or
        (TempAnswer = '1157') then
            inc(SkipQues)
      else
        if TempAnswer <> '1' then
          inc(Total);
{
      if i = 1 then
        begin
          if TempAnswer = '1' then
            clock := false;
        end
      else
        if TempAnswer <> '1' then
          inc(Total);
}
    end; // for i

  // patch 105 change =3 to >2 llh
  if total > 2 then
    Statement := 'Negative screen for dementia.';
  if total = 0 then
    Statement := 'Positive for cognitive impairment.';
  if (total = 1) or (Total = 2) then
    begin
      if clock then
        Statement := 'Negative screen for dementia.'
      else
        Statement := 'Positive for cognitive impairment.';
    end;

  // patch 105 if skipped questions, indicate invalid report
  if SkipQues > 0 then
    begin
     Total := 0 ;
     Statement := 'Too many items were skipped to score this administration.' ;
    end;


  // Note: "Mini-Cog Score" is the name of the scale and it must be used here.
  // ~ScaleName^ScaleIEN^ScaleScore~
  If Statement = 'Too many items were skipped to score this administration.' then
    Result := 'Mini-Cog Score^^^' + Statement + RECORD_END
  else
    Result := 'Mini-Cog Score^^' + IntToStr(Total) + U + '  ' + Statement + RECORD_END;

end;


function ProgressNote_MINICOG(
  PatientName,
  PatientDOB,
  PatientSex,
  ScoreString,
  AnswerString: string): string;
// The header, footer and questions and answers are handled elsewhere; avoiding
// broker calls here. <.DLL_String.> provides location for this string in the
// report.

const
  Indent                      : string = '   ';
var
  TempStr                   : string;

begin
  Result := '';
  TempStr := '';

  if (ScoreString = '') then
    Exit;

  TempStr := Piece(ScoreString, U, 4); // 6~

  Delete(TempStr, Length(TempStr), 1);

  Result := #13 + #13 +
    'Mini-Cog Results: ' + Piece(ScoreString, U, 3) + TempStr + #13 + #13;
end;




end.

