unit uMHA_AUX_SSTVOI;

interface

uses
  SysUtils;

function ScoreAnswers_SSTVOI(
  PatientName,
  PatientDOB,
  PatientSex,
  AnswerString: string): string;

function ProgressNote_SSTVOI(
  PatientName,
  PatientDOB,
  PatientSex,
  ScoreString,
  AnswerString: string): string;

implementation

uses
  uMHA_AUX_Const,
  //patch 108 removed fIA_dialogs,
  //fIA_dialogs,
  uMHA_AUX_Utils;


{*
  The format for output is used by Clinical reminders as their preferred method.
  The second field in the output is the Scale IEN, which is not used in IAT.

  Questions 1, 3, 5 ,8 are used for the Satisfaction Scale.  Add the results and
  divide by 4 to get the score. Sum should be 4-28. The higher the scale the greater
  the satisfaction.

  Questions 2, 4, 6 ,9 are used for the Skill Scale.  Add the results and
  divide by 4 to get the score. Sum should be 4-28. The higher the scale the greater
  the skill.

*}

function ScoreAnswers_SSTVOI(
  PatientName,
  PatientDOB,
  PatientSex,
  AnswerString: string): string;

var
  cnt1, cnt2                  : double ;
  i,Count,TempAnswer          : Integer;

    TempResult, ansSkipped    : string;
begin
  // AnswerString Format:
  //      83=1^1330~
  //      84=0^214~...
  //
  // '83' is QuestionID,
  // '1' is ItemIndex for radio group, but can be text from edit field
  // '1130' is ChoiceID (not used here)

  // This is used by InstrumentResults to do a quick check on the
  // instrument as to its use of the dll

  if AnswerString = 'SST-VOI' then
    begin
      Result := 'YES';
      exit;
    end;

  cnt1 := 0 ;
  cnt2 := 0;
  Result := '';
  Count := NumberOfPieces(AnswerString, RECORD_END);

  if Count < 1 then
    Exit;

  for i := 1 to Count - 1 do
    begin
      TempResult := Piece(AnswerString, RECORD_END, i);
      TempAnswer := StrToIntDef(Piece(Piece(TempResult, U, 1), EQ, 2), -1);
      ansSkipped := Piece(Piece(TempResult, U, 1), EQ, 2);

      // if question has been skipped, can't add it to the score, so get next question response
      if (ansSkipped = '') or (ansSkipped = ' ') or
         (ansSkipped = '-1') or
         (ansSkipped = CHOICE_ID_OF_SKIPPED_MULTIPLE_CHOICE_QUESTION) or
         (ansSkipped = CHOICE_ID_OF_SKIPPED_BY_RULE_MC_QUESTION) or
         (ansSkipped = CHOICE_ID_OF_SKIPPED_REQUIRED_ANSWER) then     // do nothing
      else
      begin
        // itemindex 0 = 1 so need to add 1 to TempAnswer
        TempAnswer := TempAnswer + 1 ;
        // patch 108 llh - added after modifying response order at request of BA
        // convert responses to correct value: a response of 1 = a score of 7,
        // 2 =  6, 3 = 5, 4 = 4, 5 = 3, 6 = 2, 7 = 1
          case TempAnswer of
            1: TempAnswer := 7 ;
            2: TempAnswer := 6 ;
            3: TempAnswer := 5 ;
            4: TempAnswer := 4 ;
            5: TempAnswer := 3 ;
            6: TempAnswer := 2 ;
            7: TempAnswer := 1 ;
           else
             TempAnswer := 0 ;
           end;

        if ( (i = 1) or (i = 3) or (i = 5) or (i = 8) ) then
          cnt1 := cnt1 + TempAnswer ;

        if ( (i = 2) or (i = 4) or (i = 6) or (i = 9) ) then
          cnt2 := cnt2 + TempAnswer ;
      end;
    end ;

  if cnt1 > 0 then cnt1 := cnt1/4 ;

  if cnt2> 0 then cnt2 := cnt2/4 ;
    
  // Note: "Satisfaction" is the name of the scale and it must be used here.
  // ~ScaleName^ScaleIEN^ScaleScore~
  Result := 'Satisfaction^^' + floattostr(cnt1) + RECORD_END +
            'Skill^^' + floattoStr(cnt2) + RECORD_END;
          
end;


function ProgressNote_SSTVOI(
  PatientName,
  PatientDOB,
  PatientSex,
  ScoreString,
  AnswerString: string): string;

// The header, footer and questions and answers are handled elsewhere; avoiding
// broker calls here.

const
  Indent                      : string = '   ';
var
  i                           : Integer;
  TempStr1,
    TempStr2                  : string;
begin
  Result := '';

  if (ScoreString = '') or (AnswerString = '') then
    Exit;

  Result := #13 + 'SST - Initial Quality of Life Domains' + #13 + #13;

  for i := 1 to NumberOfPieces(ScoreString, RECORD_END) - 1 do
    begin
      TempStr1 := Piece(Piece(ScoreString, RECORD_END, i), U, 1);
      TempStr2 := Piece(Piece(ScoreString, RECORD_END, i), U, 3);

      Result := Result + Indent + TempStr1 + ': ' + TempStr2 + #13;
    end;

  Result := Result + #13 + Indent +
    'Range is 1 to 7 where a higher score indicates greater level of quality.';

end;

end.

