unit uMHA_AUX_STMS;

interface

uses
  SysUtils;

function ScoreAnswers_STMS(
  PatientName,
  PatientDOB,
  PatientSex,
  AnswerString: string): string;

function ProgressNote_STMS(
  PatientName,
  PatientDOB,
  PatientSex,
  ScoreString,
  AnswerString: string): string;

implementation

uses
  uMHA_AUX_Const,
  //patch 129 removed fIA_dialogs,
  //fIA_dialogs,
  uMHA_AUX_Utils;


{*
  The format for output is used by Clinical reminders as their preferred method.
  The second field in the output is the Scale IEN, which is not used in IAT.

  The STMS has the subject do a series of mental status tasks.
*}

function ScoreAnswers_STMS(
  PatientName,
  PatientDOB,
  PatientSex,
  AnswerString: string): string;

const
  Spacer                      : string = '      ';

var
  Count, i,
    DigitSpan,
    Total                     : Integer;
  TempResult,
    TempAnswer                : string;

begin
  // AnswerString Format:
  //      83=1^1330~
  //      84=0^214~...
  //
  // '83' is QuestionID,
  // '1' is ItemIndex for radio group, but can be text from edit field
  // '1130' is ChoiceID (not used here)

  // This is used by InstrumentResults to do a quick check on the
  // instrument as to its use of the dll
  if AnswerString = 'STMS' then
    begin
      Result := 'YES';
      exit;
    end;

  Result := '';
  DigitSpan := 0;
  Total := 0;
  Count := NumberOfPieces(AnswerString, RECORD_END);

  if Count < 1 then
    Exit;

  for i := 1 to Count - 1 do
    begin
      TempResult := Piece(AnswerString, RECORD_END, i);
      TempAnswer := Piece(Piece(TempResult, U, 1), EQ, 2);

      if (i = 9) or (i = 10) or (i = 11) or
        (i = 12) or (i = 13) or (i = 23) then
        begin
          // Digit Span Items. Keep highest, max is seven
          if (i = 9) then
            DigitSpan := StrToIntDef(TempAnswer, 0);
          if (i = 10) then
            if TempAnswer <> '1156' then
              if StrToIntDef(TempAnswer, 0) > DigitSpan then
                DigitSpan := StrToIntDef(TempAnswer, 0);
          if (i = 11) then
            begin
              if (TempAnswer <> '1156') then
                if StrToIntDef(TempAnswer, 0) > DigitSpan then
                  DigitSpan := StrToIntDef(TempAnswer, 0);
            end;

          if i = 12 then // Number of attempts to learn words
            begin
              if TempAnswer = '3' then // 4 trials
                Total := Total - 3;
              if TempAnswer = '2' then // 3 trials
                Total := Total - 2;
              if TempAnswer = '1' then // 2 trials
                Total := Total - 1;
            end;

          if i = 13 then // Number of words learned
            Total := Total + StrToIntDef(TempAnswer, 0);

          if i = 23 then // Answer for drawing cube
            begin
              if TempAnswer = '0' then // Full credit
                Total := Total + 2;
              if TempAnswer = '1' then // Partial credit
                Total := Total + 1;
            end
        end // special questions
      else
        if TempAnswer = '0' then
          inc(Total);

    end; // for i

  // Note: "STMS Score" is the name of the scale and it must be used here.
  // ~ScaleName^ScaleIEN^ScaleScore~
  Result := 'STMS Score^^' + IntToStr(Total + DigitSpan) + RECORD_END;

end;


function ProgressNote_STMS(
  PatientName,
  PatientDOB,
  PatientSex,
  ScoreString,
  AnswerString: string): string;
// The header, footer and questions and answers are handled elsewhere; avoiding
// broker calls here. <.DLL_String.> provides location for this string in the
// report.

const
  Indent                      : string = '   ';
var
  TempStr                     : string;

begin
  Result := '';
  TempStr := '';

  if (ScoreString = '') then
    Exit;

  TempStr := Piece(ScoreString, U, 3); // 6~

  Delete(TempStr, Length(TempStr), 1);

  Result := #13 + #13 +
    'STMS score: ' + TempStr + #13 + #13;
end;




end.

