unit uIA_Rule_OPS;

interface

uses
  Classes,
  SysUtils,
  DateUtils,
  uIA_Instrument,
  uIA_Rule;

type

  TRule_OPS = class
  private
    FSkipQuestions: string;
    function TestRule(IsFirstCondition: boolean;
      ARule: TRule; AnswerStr: string): Boolean;
    function GetAnswer(aInstrument: TInstrument; QID: string): string;
    procedure RemoveSkipQuestionsByRule(Questions: string);
  protected
  public
    destructor Destroy; override;
    constructor Create;

    property SkipQuestions: string read FSkipQuestions write FSkipQuestions;
    function ProcessRule(aInstrument: TInstrument; QID, AnswerStr: string): string;
    function QuestionOperatorToStr(ThisRule: TRule; IsFirstQuestion: BOOLEAN): string;
    function BoolOperatorToString(ThisRule: TRule): string;
    function IndexToQuestionOperator(const AIndex: integer): TOperatorType;
    function DataTypeToString(ADataValueType: TValueDataType): string;

    //patch 108 llh - custom rule/skip question logic for the FOCI instrument
    procedure TestFOCISkipQues(aInstrument: TInstrument; QID, AnswerStr: string);

  published
  end;

var
  Rule_OPS                    : TRule_OPS;


implementation

uses
  uIA_Const,
  uIA_Utils,
  dialogs,
  uIA_Question,
  uIA_Answer;

var
  lvQuestion                  : TQuestion;


constructor TRule_OPS.Create;
begin
  inherited Create;
  lvQuestion := TQuestion.Create;
  FSkipQuestions := '';
end;

destructor TRule_OPS.Destroy;
begin
  lvQuestion.Free;
  inherited Destroy;
end;

function TRule_OPS.TestRule(IsFirstCondition: boolean;
  ARule: TRule; AnswerStr: string): Boolean;

var
  tmpValueDataType            : TValueDataType;
  tmpValue                    : string;
  tmpOperator                 : TOperatorType;
  tmpInterval                 : integer;

  function MyStrToOperator(AOperatorString: string): TOperatorType;
  var
    tmpIndex                  : integer;
    tmpString                 : string;
  begin
    //  toEQ, toLT, toLE, toGT, toGE, toNE
    //  OPERATOR_STRINGS = 'Equals^Does not equal^Is less than^Is greater than^'
    //    + 'Equals or is less than^Equals or is greater than';

    tmpString := AOperatorString;
    tmpIndex := PieceIndex(OPERATOR_STRINGS, tmpString, U);

    case tmpIndex of
      2: Result := toNE;
      3: Result := toLT;
      4: Result := toGT;
      5: Result := toLE;
      6: Result := toGE;
    else
      Result := toEQ;
    end; //case

  end; //MyStrToSecondOp

  function MyStrToDataType(ADataTypeString: string): TValueDataType;
  begin
    //dtString, dtDate, dtNumber
    Result := dtString;
    if (ADataTypeString = DATE_STR) then
      Result := dtDate;
    if (ADataTypeString = NUMBER_STR) then
      Result := dtNumber;
    if (ADataTypeString = INTERVAL_STR) then
      Result := dtTimeInterval;
  end; //MyStrToDataType

begin

  Result := False;
  tmpInterval := 0;

  if IsFirstCondition then
    begin
      tmpValueDataType := ARule.FirstValueDataType;
      tmpValue := ARule.FirstValue;
      tmpOperator := ARule.FirstOperator;
    end
  else
    begin
      tmpValueDataType := ARule.SecondValueDataType;
      tmpValue := ARule.SecondValue;
      tmpOperator := ARule.SecondOperator;
    end;

  if (tmpValueDataType = dtTimeInterval) then
    begin
      tmpInterval := StrToIntDef(Piece(tmpValue, '*', 1), 0);
      tmpValue := Piece(tmpValue, '*', 2);
      if (tmpValue = 'YEAR') then
        tmpInterval := tmpInterval * 365;
      if (tmpValue = 'MONTH') then
        tmpInterval := tmpInterval * 30;
      if (tmpValue = 'WEEK') then
        tmpInterval := tmpInterval * 7;
      if (tmpValue = 'DAY') then
        tmpInterval := tmpInterval * 1;
    end; //if (tmpValueDataType...

  case tmpOperator of
    toEQ:
      begin
        case tmpValueDataType of
          dtString: Result := (AnswerStr = tmpValue);
          dtDate: Result := (StrToDate(AnswerStr) = StrToDate(tmpValue));
          dtNumber: Result := (StrToFloat(AnswerStr) = StrToFloat(tmpValue));
          dtTimeInterval:
            Result := (DaysBetween(StrToDate(AnswerStr), Now) = tmpInterval)
        end; //case tmpValueDataType
      end;
    //if (tmpValueDataType = dtTimeInterval) then
    //  Result := (DaysBetween(StrToDate(AnswerStr), Now) = tmpInterval)
    //else
    //  Result := (AnswerStr = tmpValue);
    toLT:
      begin
        case tmpValueDataType of
          dtString: Result := (AnswerStr < tmpValue);
          dtDate: Result := (StrToDate(AnswerStr) < StrToDate(tmpValue));
          dtNumber: Result := (StrToFloat(AnswerStr) < StrToFloat(tmpValue));
          dtTimeInterval: Result := (DaysBetween(StrToDate(AnswerStr), Now) < tmpInterval);
        end; //case tmpValueDataType
      end; //toLT
    toLE:
      begin
        case tmpValueDataType of
          dtString: Result := (AnswerStr <= tmpValue);
          dtDate: Result := (StrToDate(AnswerStr) <= StrToDate(tmpValue));
          dtNumber: Result := (StrToFloat(AnswerStr) <= StrToFloat(tmpValue));
          dtTimeInterval: Result := (DaysBetween(StrToDate(AnswerStr), Now) <= tmpInterval);
        end; //case tmpValueDataType
      end; //toLE
    toGT:
      begin
        case tmpValueDataType of
          dtString: Result := (AnswerStr > tmpValue);
          dtDate: Result := (StrToDate(AnswerStr) > StrToDate(tmpValue));
          dtNumber: Result := (StrToFloat(AnswerStr) > StrToFloat(tmpValue));
          dtTimeInterval: Result := (DaysBetween(StrToDate(AnswerStr), Now) > tmpInterval);
        end; //case tmpValueDataType
      end; //toGT
    toGE:
      begin
        case tmpValueDataType of
          dtString: Result := (AnswerStr >= tmpValue);
          dtDate: Result := (StrToDate(AnswerStr) >= StrToDate(tmpValue));
          dtNumber: Result := (StrToFloat(AnswerStr) >= StrToFloat(tmpValue));
          dtTimeInterval: Result := (DaysBetween(StrToDate(AnswerStr), Now) >= tmpInterval);
        end; //case tmpValueDataType
      end; //toGT
    toNE:
      begin
        case tmpValueDataType of
          dtString: Result := (AnswerStr <> tmpValue);
          dtDate: Result := (StrToDate(AnswerStr) <> StrToDate(tmpValue));
          dtNumber: Result := (StrToFloat(AnswerStr) <> StrToFloat(tmpValue));
          dtTimeInterval: Result := (DaysBetween(StrToDate(AnswerStr), Now) <> tmpInterval);
        end; //case tmpValueDataType
      end; //toGT
  end; //case tmpOperator
end;


procedure TRule_OPS.RemoveSkipQuestionsByRule(Questions: string);
var
  i, j                        : integer;
begin
  for i := NumberOfPieces(FSkipQuestions, U) downto 1 do
    for j := NumberOfPieces(Questions, U) downto 1 do
      begin
        if Piece(Questions, U, j) = Piece(FSkipQuestions, U, i) then
          SetPiece(FSkipQuestions, U, i, ' ');
      end;
end;


function TRule_OPS.GetAnswer(aInstrument: TInstrument; QID: string): string;
var
  i                           : Integer;
begin
  Result := '';
  for i := 0 to aInstrument.ListOfAnswers.Count - 1 do
    begin
      if QID = TAnswer(aInstrument.ListOfAnswers[i]).QuestionID then
        begin
          if TAnswer(aInstrument.ListOfAnswers[i]).IsMultiChoice then
            Result := TAnswer(aInstrument.ListOfAnswers[i]).AnswerValue
          else
            Result := TAnswer(aInstrument.ListOfAnswers[i]).AnswerText;
          break;
        end;
    end;
end;


function TRule_OPS.ProcessRule(aInstrument: TInstrument; QID, AnswerStr: string): string;
// This will:
// 1. Check for Second Condition: if true, check the first condition and if it is
//    true then present the message and/or update list of questions to skip.
// 2. When no Second Condition, check the First Condition and if true then
//    present message and/or update list of questions to skip.
// The out-going string is the message, if one exists, when the rule is true.

var
  i                           : Integer;
  First, Second               : boolean;
  TempSkips                   : string;

begin
  Result := '';
  TempSkips := '';

  if aInstrument.ListOfRules.Count = 0 then
    Exit;

  //patch 108 llh -- if instrument is the FOCI, do other logic to test for
  //                 skipped questions.  NOTE:  this logic mimics that which is in MHA
  //                 but is not exact as there is no
  if aInstrument.InstrumentName = 'FOCI' then
  begin
    TestFOCISkipQues(aInstrument, QID, AnswerStr);
    Exit ;
  end;
  // end FOCI test block;

  for i := 0 to aInstrument.ListOfRules.Count - 1 do
    begin
      First := false;
      Second := False;

      // Check for first condition when no second condition exists.
      if (QID = TRule(aInstrument.ListOfRules[i]).FirstQuestionID) then
        begin
          if (TRule(aInstrument.ListOfRules[i]).SecondQuestionID = '') then
            begin
              First := TestRule(True {1st Condition}, TRule(aInstrument.ListOfRules[i]), AnswerStr);
              if First then
                begin
                  Result := TRule(aInstrument.ListOfRules[i]).MessageText;
                  TempSkips := TRule(aInstrument.ListOfRules[i]).SkipQuestionIDs;
                end
              else
                begin
                  if TRule(aInstrument.ListOfRules[i]).SkipQuestionIDs <> '' then
                    RemoveSkipQuestionsByRule(TRule(aInstrument.ListOfRules[i]).SkipQuestionIDs);
                end;
            end; // 1st question only

          // Check First Question, and then the second question
          if (TRule(aInstrument.ListOfRules[i]).SecondQuestionID <> '') then
            begin
              First := TestRule(True {1st Condition}, TRule(aInstrument.ListOfRules[i]), AnswerStr);
              if First then
                begin
                  Second := TestRule(False {2nd Condition},
                    TRule(aInstrument.ListOfRules[i]),
                    GetAnswer(aInstrument, TRule(aInstrument.ListOfRules[i]).SecondQuestionID));
                  if not Second then
                    if TRule(aInstrument.ListOfRules[i]).SkipQuestionIDs <> '' then
                      RemoveSkipQuestionsByRule(TRule(aInstrument.ListOfRules[i]).SkipQuestionIDs);
                end
              else // Not First
                if TRule(aInstrument.ListOfRules[i]).SkipQuestionIDs <> '' then
                  RemoveSkipQuestionsByRule(TRule(aInstrument.ListOfRules[i]).SkipQuestionIDs);

              if First and Second then
                begin
                  Result := TRule(aInstrument.ListOfRules[i]).MessageText;
                  //FSkipQuestions := FSkipQuestions + U +
                  //  TRule(Instrument.ListOfRules[i]).SkipQuestionIDs;
                  TempSkips := TRule(aInstrument.ListOfRules[i]).SkipQuestionIDs;
                end;
            end;
        end; // First Question is current

      // Second Question is current
      if QID = TRule(aInstrument.ListOfRules[i]).SecondQuestionID then
        begin
          Second := TestRule(False {2nd Condition},
            TRule(aInstrument.ListOfRules[i]), AnswerStr);
          if Second then
            begin
              First := TestRule(True {1st Condition},
                TRule(aInstrument.ListOfRules[i]),
                GetAnswer(aInstrument, TRule(aInstrument.ListOfRules[i]).FirstQuestionID));
              if not First then
                if TRule(aInstrument.ListOfRules[i]).SkipQuestionIDs <> '' then
                  RemoveSkipQuestionsByRule(TRule(aInstrument.ListOfRules[i]).SkipQuestionIDs);
            end
          else // not second
            if TRule(aInstrument.ListOfRules[i]).SkipQuestionIDs <> '' then
              RemoveSkipQuestionsByRule(TRule(aInstrument.ListOfRules[i]).SkipQuestionIDs);

          if First and Second then
            begin
              Result := TRule(aInstrument.ListOfRules[i]).MessageText;
              //FSkipQuestions := FSkipQuestions + U +
              //  TRule(Instrument.ListOfRules[i]).SkipQuestionIDs;
              TempSkips := TRule(aInstrument.ListOfRules[i]).SkipQuestionIDs;
            end;
        end; // QID = SecondQuestionID
    end; // for i

  // Each QuestionID must be bracketed by a delimiter; this ensures the ID
  // is read properly.
  FSkipQuestions := FSkipQuestions + U + TempSkips + U;

end;

// patch 108 llh -- skip logic for the FOCI is more complex than editor can
//               handle, do not want to remove the skipquestionids as all
//               20 questions use the same logic
procedure TRule_OPS.TestFOCISkipQues(aInstrument: TInstrument; QID, AnswerStr: string);
var
  i                         : integer ;
  clearSkips                : boolean ;

begin
  // yes this is bad, BUT can be done because all the rule logic is the
  // same for the skipped questions and is specific to the FOCI instrument.

  // DO NOT CHANGE RULE LOGIC IN THE FOCI UNLESS THIS CODE IS RE-EVALUATED

  //initialize
  clearSkips := false ;
  FSkipQuestions := TRule(aInstrument.ListOfRules[1]).SkipQuestionIDs + U;

  for i := 0 to aInstrument.ListOfAnswers.Count - 6 do // -6 don't need to evaluate last 5 questions
  begin
    if QID = TAnswer(aInstrument.ListOfAnswers[i]).QuestionID then
      begin
        if (AnswerStr = '0') or (AnswerStr = '241') then clearskips := true ;
        //else clearskips := false ;
        //if clearskips then break;
      end
    else
      if (AnswerStr = '0') or
        (TAnswer(aInstrument.ListOfAnswers[i]).AnswerText = '1. Yes' ) or
        (TAnswer(aInstrument.ListOfAnswers[i]).AnswerText = 'Yes' ) or
        (TAnswer(aInstrument.ListOfAnswers[i]).AnswerText = '241' ) then
        clearskips := true ;
  end;

  if clearSkips then
    RemoveSkipQuestionsByRule(TRule(aInstrument.ListOfRules[1]).SkipQuestionIDs)  ;

end;


function TRule_OPS.QuestionOperatorToStr(ThisRule: TRule; IsFirstQuestion: BOOLEAN): string;
var
  tmpOperator                 : TOperatorType;
begin
  with ThisRule do
    begin
      if IsFirstQuestion then
        tmpOperator := FirstOperator
      else
        tmpOperator := SecondOperator;

      Result := '';

  //OPERATOR_STRINGS = 'Equals^Does not equal^Is less than^Is greater than^'
  //+ 'Equals or is less than^Equals or is greater than';

      case tmpOperator of
        toEQ: Result := Piece(OPERATOR_STRINGS, U, 1);
        toNE: Result := Piece(OPERATOR_STRINGS, U, 2);
        toLT: Result := Piece(OPERATOR_STRINGS, U, 3);
        toGT: Result := Piece(OPERATOR_STRINGS, U, 4);
        toLE: Result := Piece(OPERATOR_STRINGS, U, 5);
        toGE: Result := Piece(OPERATOR_STRINGS, U, 6);
      end;
    end;
end;


function TRule_OPS.BoolOperatorToString(ThisRule: TRule): string;
begin
  Result := 'AND';

  if (ThisRule.BoolOperator = boOR) then
    Result := 'OR';

end;


function TRule_OPS.IndexToQuestionOperator(const AIndex: integer): TOperatorType;
begin
  case AIndex of
    1: Result := toNE;
    2: Result := toLT;
    3: Result := toGT;
    4: Result := toLE;
    5: Result := toGE;
  else
    Result := toEQ;
  end;
end;


(*function TRule.IndexToBoolOperator(const AIndex: integer): TBoolOperatorType;
begin
  if (AIndex = 1) then
    Result := boOR
  else
    Result := boAND;
end;*)


function TRule_OPS.DataTypeToString(ADataValueType: TValueDataType): string;
begin
  //  TValueDataType = (dtString, dtDate, dtNumber, dtTimeInterval);
  case ADataValueType of
    dtDate: Result := DATE_STR;
    dtNumber: Result := NUMBER_STR;
    dtTimeInterval: Result := INTERVAL_STR;
  else
    Result := STRING_STR;
  end; //case
end;


initialization
  Rule_OPS := TRule_OPS.Create;

finalization
  Rule_OPS.Free;


end.

