unit uIA_Scales_IO;


interface

uses
  Trpcb,
  uIA_Instrument;

function GetScalesFromVista(WhichInstrument: string; Broker: TRPCBroker): string;
function ReadScales(Broker: TRPCBroker; var aInstrument: TInstrument): Boolean;
function ReadScaleGroupsAndScales(Broker: TRPCBroker; var aInstrument: TInstrument): string;

implementation

uses
  SysUtils,
  fIA_dialogs,
  uIA_Const,
  uIA_Utils,
  uIA_Scale,
  uIA_ScaleGroup;


function GetScalesFromVista(WhichInstrument: string; Broker: TRPCBroker): string;
var
  i                           : integer;
  // Results look like this:

  // Group1= 100000^AllanS 1^Allan's Sizes^1^Ego Strength^0^5^50^10^30^40~
  // Scale1= 100000^100000^1^Ego Size^ES~
  // Scale2= 100001^100000^2^Shoe Size^Sz~
  // Group2= 100005^AllanS 1^Bob^1^^0^10^100^-1^-1^0~
  // Group3= 100001^AllanS 1^Allan's Bank Statement^2^Cash^22^4^88^40^60^70~
  // Scale3= 100002^100001^1^Cash Flow~
  // Scale4= 100003^100001^2^DebT^D~
  // Group4= 100007^AllanS 1^Bob^100^^0^10^100^-1^-1^0  //First record is unique~
begin
  with Broker do
    begin
      ClearResults := True;
      ClearParameters := True;
      CreateContext('YS BROKER1');
      RemoteProcedure := 'YTQ GET SCALES';
      Param[0].Mult['"CODE"'] := WhichInstrument;
      Param[0].PType := List;
      Call;

      if not (Results[0] = DATA) then
        begin
          Result := BAD_DATA;
          Exit;
        end;

      Result := '';

      // start at 1 to skip [DATA] line
      for i := 1 to Results.Count - 1 do
        Result := Result + Results[i] + RECORD_END;

      // Needed for DLL, in case there is a crash
      CreateContext('OR CPRS GUI CHART');
    end; //with RCPBroker

//  Result := DropTrailer(Result);

end;


function ReadScales(Broker: TRPCBroker; var aInstrument: TInstrument): Boolean;
var
  TempStr, SubStr, SubSubStr  : string;
  i, Count                    : Integer;
  AScale                      : TScale;
begin
  Result := True;

  TempStr := GetScalesFromVista(aInstrument.InstrumentName, Broker);

  if TempStr = '' then
    Exit;

  if TempStr = BAD_DATA then
    begin
      Result := False;
      Exit;
    end;

  // Usually there will be one scale, but check for more than one.
  // All we really need here is the Scale.IEN and the label.
  Count := NumberOfPieces(TempStr, RECORD_END);
  if Count > 1 then
    for i := 2 to Count do
      begin
        SubStr := Piece(TempStr, RECORD_END, i);

        if Pos('Scale', SubStr) > 0 then
          begin
            AScale := TScale.Create;
            SubSubStr := Piece(SubStr, EQ, 2);
            AScale.IEN := Piece(SubSubStr, U, 1);
            AScale.GroupIEN := Piece(SubSubStr, U, 2);
            AScale.Sequence := StrToIntDef(Piece(SubSubStr, U, 3), -1);
            AScale.ScaleName := Piece(SubSubStr, U, 4);
            AScale.XLabel := Piece(SubSubStr, U, 5);
            aInstrument.ListOfScales.Add(AScale);
          end;
      end;
end;


function ReadScaleGroupsAndScales(Broker: TRPCBroker;
  var aInstrument: TInstrument): string;
// This will populate Instrument.ListOfScales and Instrument.ListOfScaleGroups
var
  TempStr, SubStr, SubSubStr  : string;
  i                           : Integer;
  AScale                      : TScale;
  AGroup                      : TScaleGroup;


begin
  Result := OK;
  gvWhichScaleGroup := -1;
  gvWhichScale := -1;

  TempStr := GetScalesFromVista(aInstrument.InstrumentName, Broker);

  if TempStr = '' then // No scales
    Exit;

  if TempStr = BAD_DATA then
    begin
      Result := BAD_DATA;
      Exit;
    end;

  for i := 1 to NumberOfPieces(TempStr, RECORD_END) do
    begin
      SubStr := Piece(TempStr, RECORD_END, i);
      if Pos('Scale', SubStr) > 0 then
        begin
          AScale := TScale.Create;
          SubSubStr := Piece(SubStr, EQ, 2);
          AScale.IEN := Piece(SubSubStr, U, 1);
          AScale.GroupIEN := Piece(SubSubStr, U, 2);
          AScale.Sequence := StrToIntDef(Piece(SubSubStr, U, 3), -1);
          AScale.ScaleName := Piece(SubSubStr, U, 4);
          AScale.XLabel := Piece(SubSubStr, U, 5);

          aInstrument.ListOfScales.Add(AScale);
          Inc(gvWhichScale);
        end;

      if Pos('Group', SubStr) > 0 then
        begin
    //  GroupX= [1] SCALEGROUP ID^
    //          [2] INSTRUMENT NAME^
    //          [3] SCALEGROUP NAME^
    //          [4] GROUP SEQUENCE^
    //          [5-11] ORDINATE TITLE^MIN^INCREMENT^MAX^GRID1^GRID2^GRID3
          AGroup := TScaleGroup.Create;
          SubSubStr := Piece(SubStr, EQ, 2);
          AGroup.IEN := Piece(SubSubStr, U, 1);
          // 2nd piece is instrument name
          AGroup.GroupName := Piece(SubSubStr, U, 3);
          AGroup.Sequence := StrToIntDef(Piece(SubSubStr, U, 4), -1);
          AGroup.OrdinateTitle := Piece(SubSubStr, U, 5);
          AGroup.OrdinateMin := StrToIntDef(Piece(SubSubStr, U, 6), -1);
          AGroup.OrdinateIncrement := StrToIntDef(Piece(SubSubStr, U, 7), -1);
          AGroup.OrdinateMax := StrToIntDef(Piece(SubSubStr, U, 8), -1);
          AGroup.Grid1Ordinate := StrToIntDef(Piece(SubSubStr, U, 9), -1);
          AGroup.Grid2Ordinate := StrToIntDef(Piece(SubSubStr, U, 10), -1);
          AGroup.Grid3Ordinate := StrToIntDef(Piece(SubSubStr, U, 11), -1);

          aInstrument.ListOfScaleGroups.Add(AGroup);
          Inc(gvWhichScaleGroup);
        end;
    end;
end;

(* M side

.01 SCALEGROUP ID(RNJ5, 0 X), [0; 1]
1 INSTRUMENT ID(RP601.71'), [0;2]
  2 SCALEGROUP NAME(RF), [0; 3]
  3 GROUP SEQUENCE(NJ3, 0), [0; 4]
  4 ORDINATE TITLE(F), [0; 5]
  5 ORDINATEMIN(NJ4, 0), [0; 6]
  6 ORDINATEINCREMENT(NJ4, 0), [0; 7]
  7 ORDINATEMAX(NJ4, 0), [0; 8]
  8 GRID1(NJ4, 0), [0; 9]
  9 GRID2(NJ4, 0), [0; 10]
  10 GRID3(NJ4, 0), [0; 11]*)



end.

