unit uIA_ThisUser_IO;


interface

uses
  dialogs,
  sysutils,
  Trpcb;

function ReadUserProperties(aDUZ: string;
  Broker: TRPCBroker): string;

function ReadUserPermissions(aDUZ: string;
  Broker: TRPCBroker): string;

function ReadCosigner(aDUZ, ProgNoteTitle: string;
  Broker: TRPCBroker): boolean;

implementation


uses
  uIA_Const;

function ReadCosigner(aDUZ, ProgNoteTitle: string;
  Broker: TRPCBroker): boolean;
// Return: default is false, meaning no co-signer required
// This will work for consults as well as progress notes
var
  i                           : integer;
  TempStr                     : string;
begin
//  Returns for a student:                Non-student
//  [DATA]                                  [DATA]
//  STUDENT,DDD^6739                        MHPROVIDER,ONE^2222222345
//                 <---- always blank --->
//  522^ZZ BONHAM^522^                      620^HINES ISC^499^
//  1^cosigner required                     0^cosigner not required

  result := false;

  with Broker do
    begin
      ClearResults := True;
      ClearParameters := True;
      CreateContext('YS BROKER1');

      RemoteProcedure := 'YTQ USERQ';

      Param[0].Mult['"TITLE"'] := ProgNoteTitle; // 'MENTAL HEALTH DIAGNOSTIC STUDY NOTE'
           // or 'MENTAL HEALTH CONSULT NOTE'
      Param[0].Mult['"DUZ"'] := aDUZ;
      Param[0].PType := List;

      Call;

      if results[0] <> DATA then
        exit;

      if Results[4] <> '' then
        begin
          TempStr := Results[4];
          if TempStr[1] = '1' then
            result := true;
        end;
    end;
end;


function ReadUserPermissions(aDUZ: string; Broker: TRPCBroker): string;
// Returns all the keys for this user. Each key has its own line.
//      [DATA]
//      YSP
//      YSPROG
// If none, then a blank is follows [DATA]
var
  i                           : Integer;
begin
  Result := '';

  if aDUZ = '' then
    exit;

  with Broker do
    begin
      ClearParameters := True;
      ClearResults := True;
      CreateContext('YS BROKER1');
      RemoteProcedure := 'YTQ ALLKEYS';
      Param[0].Mult['"IEN"'] := aDUZ;
      Param[0].PType := List;
      Call;

      if (Results[0] <> DATA) then
        Result := BAD_DATA
      else
        for i := 1 to Results.Count - 1 do // Skip [DATA]
          Result := Result + Results[i] + U;
    end; //with RPCBroker1
end;


function ReadUserProperties(aDUZ: string;
  Broker: TRPCBroker): string;
// Returns:
//   [DATA]
//   YSSTUDENT,FRANK^2222222346
//                               <---- always blank
//   6020^HINES ISC^449^
//   ^no title sent
var
  i                           : Integer;
begin
  Result := '';

  if aDUZ = '' then
    exit;

  with Broker do
    begin

      ClearParameters := True;
      ClearResults := True;
      CreateContext('YS BROKER1');
      RemoteProcedure := 'YTQ USERQ';

      Param[0].Mult['"DUZ"'] := aDUZ;
      Param[0].PType := List;
      Call;

      if not (Results[0] = DATA) then
        Result := BAD_DATA
      else
        begin
          for i := 1 to Results.Count - 1 do // Skip [DATA]
            Result := Result + Results[i] + U;
        end;
    end; //with Broker
end;


end.

 